@@ -18,6 +18,17 @@ cat << EOF > "$debusine_client_config_directory/config.ini"
 default-server = integration-test
 
 [server:integration-test]
-api-url = http://localhost/api
+api-url = https://$(hostname -f)/api
 token = $token_client
 EOF
+
+# Allow test user to add workflow templates
+sudo -u debusine-server debusine-admin shell << EOF
+from django.contrib.auth.models import Permission
+from debusine.db.models import User
+user = User.objects.get(username="$user")
+permission = Permission.objects.get_by_natural_key(
+    "add_workflowtemplate", "db", "workflowtemplate"
+)
+user.user_permissions.add(permission)
+EOF
