@@ -9,9 +9,10 @@
 """Waiter utility: waits until success/failure and fails after timeout."""
 
 import time
+from collections.abc import Callable
 from datetime import datetime, timedelta
 from functools import partialmethod
-from typing import Any, Callable
+from typing import Any
 
 
 class Waiter:
