@@ -17,6 +17,9 @@ from debusine.artifacts.local_artifact import (  # noqa: F401
     LintianArtifact,
     LocalArtifact,
     PackageBuildLog,
+    SigningInputArtifact,
+    SigningKeyArtifact,
+    SigningOutputArtifact,
     SourcePackage,
     Upload,
     WorkRequestDebugLogs,
