@@ -13,13 +13,14 @@ Local artifact's representation.
 See docs/design/ontology.html for the semantics of the Artifacts.
 """
 import abc
+import base64
 import json
 import re
 from collections.abc import Iterable, Sequence
 from datetime import datetime
 from json import JSONDecodeError
 from pathlib import Path
-from typing import Any, ClassVar, Generic, Optional, TypeVar, Union
+from typing import Any, ClassVar, Generic, Self, TypeVar
 
 from debian import deb822, debfile
 
@@ -37,7 +38,7 @@ from debusine.client.models import (
     FilesRequestType,
     model_to_json_serializable_dict,
 )
-from debusine.utils import extract_generic_type_argument
+from debusine.utils import extract_generic_type_arguments
 
 AD = TypeVar("AD", bound=data_models.ArtifactData)
 
@@ -52,17 +53,17 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         extra = pydantic.Extra.forbid
 
     #: Artifact type
-    category: str
+    category: data_models.ArtifactCategory
     # Keys are paths in the artifact. Values the paths in the local system
     files: dict[str, Path] = pydantic.Field(default_factory=dict)
     #: Artifact data
     data: AD
     # TODO: it would be great to not have to redefine data in subclasses, but
     # it needs pydantic's Generics support, which might require work to work
-    # with extract_generic_type_argument
+    # with extract_generic_type_arguments
 
     #: Default value for category
-    _category: ClassVar[str]
+    _category: ClassVar[data_models.ArtifactCategory]
 
     #: Class used as the in-memory representation of artifact data.
     _data_type: type[AD] = pydantic.PrivateAttr()
@@ -91,9 +92,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
 
         # The task data type, computed by introspecting the type argument
         # used to specialize this generic class.
-        cls._data_type = extract_generic_type_argument(
-            cls, LocalArtifact, data_models.ArtifactData
-        )
+        [cls._data_type] = extract_generic_type_arguments(cls, LocalArtifact)
 
         LocalArtifact._local_artifacts_category_to_class[cls._category] = cls
 
@@ -107,7 +106,7 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         """Return list of artifact categories."""
         return list(LocalArtifact._local_artifacts_category_to_class.keys())
 
-    @pydantic.validator("category")
+    @pydantic.validator("category", pre=True)
     def _validate_category(cls, category: str) -> str:
         """Validate that the category is known."""
         if category not in cls._local_artifacts_category_to_class:
@@ -127,8 +126,8 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
         self,
         file: Path,
         *,
-        artifact_base_dir: Optional[Path] = None,
-        override_name: Optional[str] = None,
+        artifact_base_dir: Path | None = None,
+        override_name: str | None = None,
     ):
         """
         Add a local file in the artifact.
@@ -194,9 +193,9 @@ class LocalArtifact(pydantic.BaseModel, Generic[AD], abc.ABC):
     def serialize_for_create_artifact(
         self,
         *,
-        workspace: Optional[str],
-        work_request: Optional[int] = None,
-        expire_at: Optional[datetime] = None,
+        workspace: str | None,
+        work_request: int | None = None,
+        expire_at: datetime | None = None,
     ) -> dict[str, Any]:
         """Return dictionary to be used by the API to create an artifact."""
         files: FilesRequestType = FilesRequestType({})
@@ -273,17 +272,15 @@ class WorkRequestDebugLogs(LocalArtifact[data_models.EmptyArtifactData]):
     WorkRequests.
     """
 
-    _category = "debusine:work-request-debug-logs"
+    _category = data_models.ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
     data: data_models.EmptyArtifactData = pydantic.Field(
         default_factory=data_models.EmptyArtifactData
     )
 
     @classmethod
-    def create(cls, *, files: Iterable[Path]) -> "WorkRequestDebugLogs":
+    def create(cls, *, files: Iterable[Path]) -> Self:
         """Return a WorkRequestDebugLogs."""
-        artifact = WorkRequestDebugLogs(
-            category=cls._category,
-        )
+        artifact = cls(category=cls._category)
 
         for file in files:
             artifact.add_local_file(file)
@@ -291,11 +288,19 @@ class WorkRequestDebugLogs(LocalArtifact[data_models.EmptyArtifactData]):
         return artifact
 
 
+class DebusineTest(LocalArtifact[data_models.EmptyArtifactData]):
+    """DebusineTest: noop artifact used in Debusine tests."""
+
+    _category = data_models.ArtifactCategory.TEST
+    data: data_models.EmptyArtifactData = pydantic.Field(
+        default_factory=data_models.EmptyArtifactData
+    )
+
+
 class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
     """PackageBuildLog: represents a build log file."""
 
-    _category = "debian:package-build-log"
-    data: data_models.DebianPackageBuildLog
+    _category = data_models.ArtifactCategory.PACKAGE_BUILD_LOG
 
     @classmethod
     def create(
@@ -304,12 +309,16 @@ class PackageBuildLog(LocalArtifact[data_models.DebianPackageBuildLog]):
         file: Path,
         source: str,
         version: str,
-    ) -> "PackageBuildLog":
+        bd_uninstallable: data_models.DoseDistCheck | None = None,
+    ) -> Self:
         """Return a PackageBuildLog."""
         artifact = cls(
             category=cls._category,
             data=data_models.DebianPackageBuildLog(
-                source=source, version=version, filename=file.name
+                source=source,
+                version=version,
+                filename=file.name,
+                bd_uninstallable=bd_uninstallable,
             ),
         )
 
@@ -360,16 +369,15 @@ def deb822dict_to_dict(element):
 class Upload(LocalArtifact[data_models.DebianUpload]):
     """Upload: encapsulate a .changes and files listed in it."""
 
-    _category = "debian:upload"
-    data: data_models.DebianUpload
+    _category = data_models.ArtifactCategory.UPLOAD
 
     @classmethod
     def create(
         cls,
         *,
         changes_file: Path,
-        exclude_files: Union[frozenset[Path], set[Path]] = frozenset(),
-    ) -> "Upload":
+        exclude_files: frozenset[Path] | set[Path] = frozenset(),
+    ) -> Self:
         """
         Return a Upload. Add the changes_files and files listed in it.
 
@@ -422,13 +430,10 @@ class Upload(LocalArtifact[data_models.DebianUpload]):
 class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
     """SourcePackage: contains source code to be built into BinaryPackages."""
 
-    _category = "debian:source-package"
-    data: data_models.DebianSourcePackage
+    _category = data_models.ArtifactCategory.SOURCE_PACKAGE
 
     @classmethod
-    def create(
-        cls, *, name: str, version: str, files: list[Path]
-    ) -> "SourcePackage":
+    def create(cls, *, name: str, version: str, files: list[Path]) -> Self:
         """Return a SourcePackage setting files and data."""
         dsc_fields = {}
         for file in files:
@@ -467,11 +472,10 @@ class SourcePackage(LocalArtifact[data_models.DebianSourcePackage]):
 class BinaryPackage(LocalArtifact[data_models.DebianBinaryPackage]):
     r"""BinaryPackage: encapsulates a single \*.deb / \*.udeb."""
 
-    _category = "debian:binary-package"
-    data: data_models.DebianBinaryPackage
+    _category = data_models.ArtifactCategory.BINARY_PACKAGE
 
     @classmethod
-    def create(cls, *, file: Path) -> "BinaryPackage":
+    def create(cls, *, file: Path) -> Self:
         """Return a BinaryPackage setting file and data."""
         pkg = debfile.DebFile(file)
         try:
@@ -522,8 +526,7 @@ class BinaryPackage(LocalArtifact[data_models.DebianBinaryPackage]):
 class BinaryPackages(LocalArtifact[data_models.DebianBinaryPackages]):
     r"""BinaryPackages: encapsulates a group of \*.deb / \*.udeb."""
 
-    _category = "debian:binary-packages"
-    data: data_models.DebianBinaryPackages
+    _category = data_models.ArtifactCategory.BINARY_PACKAGES
 
     @classmethod
     def create(
@@ -534,7 +537,7 @@ class BinaryPackages(LocalArtifact[data_models.DebianBinaryPackages]):
         version: str,
         architecture: str,
         files: list[Path],
-    ) -> "BinaryPackages":
+    ) -> Self:
         """Return a BinaryPackages setting files and data."""
         data = data_models.DebianBinaryPackages(
             srcpkg_name=srcpkg_name,
@@ -575,13 +578,15 @@ class BinaryPackages(LocalArtifact[data_models.DebianBinaryPackages]):
 class LintianArtifact(LocalArtifact[data_models.DebianLintian]):
     """LintianArtifact: encapsulate result of the Lintian run."""
 
-    _category = "debian:lintian"
-    data: data_models.DebianLintian
+    _category = data_models.ArtifactCategory.LINTIAN
 
     @classmethod
     def create(
-        cls, analysis: Path, lintian_output: Path, summary: dict[str, Any]
-    ) -> "LintianArtifact":
+        cls,
+        analysis: Path,
+        lintian_output: Path,
+        summary: data_models.DebianLintianSummary,
+    ) -> Self:
         """Return a LintianArtifact with the files set."""
         data = data_models.DebianLintian(summary=summary)
 
@@ -631,13 +636,12 @@ class LintianArtifact(LocalArtifact[data_models.DebianLintian]):
 class AutopkgtestArtifact(LocalArtifact[data_models.DebianAutopkgtest]):
     """Autopkgtest: encapsulate result of the Autopkgtest run."""
 
-    _category = "debian:autopkgtest"
-    data: data_models.DebianAutopkgtest
+    _category = data_models.ArtifactCategory.AUTOPKGTEST
 
     @classmethod
     def create(
         cls, artifact_directory: Path, data: data_models.DebianAutopkgtest
-    ) -> "AutopkgtestArtifact":
+    ) -> Self:
         """Return AutopkgtestArtifact with the files and data set."""
         artifact = cls(category=cls._category, data=data)
 
@@ -664,13 +668,10 @@ class DebianSystemTarballArtifact(
     Can be used by a chroot, container, etc.
     """
 
-    _category = "debian:system-tarball"
-    data: data_models.DebianSystemTarball
+    _category = data_models.ArtifactCategory.SYSTEM_TARBALL
 
     @classmethod
-    def create(
-        cls, tarball: Path, data: dict[str, Any]
-    ) -> "DebianSystemTarballArtifact":
+    def create(cls, tarball: Path, data: dict[str, Any]) -> Self:
         """Return a DebianSystemTarballArtifact with the tarball file."""
         data = data.copy()
         data["filename"] = tarball.name
@@ -701,13 +702,13 @@ class DebianSystemTarballArtifact(
 class BlhcArtifact(LocalArtifact[data_models.EmptyArtifactData]):
     """BlhcArtifact: encapsulate result of the blhc run."""
 
-    _category = "debian:blhc"
+    _category = data_models.ArtifactCategory.BLHC
     data: data_models.EmptyArtifactData = pydantic.Field(
         default_factory=data_models.EmptyArtifactData
     )
 
     @classmethod
-    def create(cls, blhc_output: Path) -> "BlhcArtifact":
+    def create(cls, blhc_output: Path) -> Self:
         """Return a BlhcArtifact with the files set."""
         artifact = cls(category=cls._category)
 
@@ -723,13 +724,10 @@ class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
     Can be used by a VM.
     """
 
-    _category = "debian:system-image"
-    data: data_models.DebianSystemImage
+    _category = data_models.ArtifactCategory.SYSTEM_IMAGE
 
     @classmethod
-    def create(
-        cls, image: Path, data: dict[str, Any]
-    ) -> "DebianSystemImageArtifact":
+    def create(cls, image: Path, data: dict[str, Any]) -> Self:
         """Return a DebianSystemImageArtifact with the image file."""
         data = data.copy()
         data["filename"] = image.name
@@ -758,3 +756,82 @@ class DebianSystemImageArtifact(LocalArtifact[data_models.DebianSystemImage]):
             )
 
         return files
+
+
+class SigningKeyArtifact(LocalArtifact[data_models.DebusineSigningKey]):
+    """Record the existence of a key in the signing service."""
+
+    _category = data_models.ArtifactCategory.SIGNING_KEY
+
+    @classmethod
+    def create(
+        cls,
+        purpose: data_models.KeyPurpose,
+        fingerprint: str,
+        public_key: bytes,
+    ) -> Self:
+        """Return a SigningKeyArtifact with the given properties."""
+        data = data_models.DebusineSigningKey(
+            purpose=purpose,
+            fingerprint=fingerprint,
+            public_key=base64.b64encode(public_key).decode(),
+        )
+        return cls(category=cls._category, data=data)
+
+    @property
+    def public_key_bytes(self) -> bytes:
+        """Return the public key as bytes."""
+        return base64.b64decode(self.data.public_key.encode())
+
+
+class SigningInputArtifact(LocalArtifact[data_models.DebusineSigningInput]):
+    """Input to a Sign task."""
+
+    _category = data_models.ArtifactCategory.SIGNING_INPUT
+
+    @classmethod
+    def create(
+        cls,
+        files: Iterable[Path],
+        base_dir: Path,
+        trusted_certs: list[str] | None = None,
+    ) -> Self:
+        """Return a new SigningInputArtifact."""
+        data = data_models.DebusineSigningInput(
+            trusted_certs=None if trusted_certs is None else list(trusted_certs)
+        )
+        artifact = cls(category=cls._category, data=data)
+        for file in files:
+            artifact.add_local_file(file, artifact_base_dir=base_dir)
+        return artifact
+
+    @pydantic.validator("files")
+    def validate_at_least_one_file(cls, files: dict[str, Path]):  # noqa: U100
+        """Validate that artifact has at least one file."""
+        if not files:
+            raise ValueError("Expected at least one file")
+        return files
+
+
+class SigningOutputArtifact(LocalArtifact[data_models.DebusineSigningOutput]):
+    """Output of a Sign task."""
+
+    _category = data_models.ArtifactCategory.SIGNING_OUTPUT
+
+    @classmethod
+    def create(
+        cls,
+        purpose: data_models.KeyPurpose,
+        fingerprint: str,
+        results: Iterable[data_models.SigningResult],
+        files: Iterable[Path],
+        base_dir: Path,
+    ) -> Self:
+        """Return a new SigningOutputArtifact."""
+        data = data_models.DebusineSigningOutput(
+            purpose=purpose, fingerprint=fingerprint, results=list(results)
+        )
+        artifact = cls(category=cls._category, data=data)
+        for file in files:
+            artifact.add_local_file(file, artifact_base_dir=base_dir)
+        return artifact
