@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine developers
+# Copyright 2023 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,10 +9,11 @@
 
 """Unit tests for classes in local_artifacts.py."""
 
+import base64
 import json
 from datetime import datetime, timedelta
 from pathlib import Path
-from unittest import TestCase, mock
+from unittest import mock
 
 import debian.deb822 as deb822
 
@@ -24,6 +25,9 @@ from debusine.artifacts import (
     LintianArtifact,
     LocalArtifact,
     PackageBuildLog,
+    SigningInputArtifact,
+    SigningKeyArtifact,
+    SigningOutputArtifact,
     SourcePackage,
     Upload,
     WorkRequestDebugLogs,
@@ -35,10 +39,12 @@ from debusine.artifacts.local_artifact import (
     deb822dict_to_dict,
 )
 from debusine.client.models import FileRequest
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
-def unregister_local_artifact_subclass(category: str) -> None:
+def unregister_local_artifact_subclass(
+    category: data_models.ArtifactCategory,
+) -> None:
     """
     Unregister an artifact category.
 
@@ -47,7 +53,7 @@ def unregister_local_artifact_subclass(category: str) -> None:
     del LocalArtifact._local_artifacts_category_to_class[category]
 
 
-class LocalArtifactTests(TestHelpersMixin, TestCase):
+class LocalArtifactTests(TestCase):
     """Tests for LocalArtifact class."""
 
     def setUp(self):
@@ -58,7 +64,7 @@ class LocalArtifactTests(TestHelpersMixin, TestCase):
         class TestArtifact(LocalArtifact[data_models.EmptyArtifactData]):
             """Custom artifact for testing."""
 
-            _category = "test"
+            _category = data_models.ArtifactCategory.TEST
 
         self.addCleanup(
             unregister_local_artifact_subclass, TestArtifact._category
@@ -278,7 +284,7 @@ class LocalArtifactTests(TestHelpersMixin, TestCase):
         self.assertEqual(serialized["expire_at"], expire_at.isoformat())
 
 
-class PackageBuildLogTests(TestHelpersMixin, TestCase):
+class PackageBuildLogTests(TestCase):
     """Tests for PackageBuildLog."""
 
     def setUp(self):
@@ -365,7 +371,7 @@ class PackageBuildLogTests(TestHelpersMixin, TestCase):
         )
 
 
-class UploadTests(TestHelpersMixin, TestCase):
+class UploadTests(TestCase):
     """Tests for Upload."""
 
     def setUp(self):
@@ -554,7 +560,7 @@ class Deb822DictToDictTests(TestCase):
         self.assert_deb822dict_return_correct_value(to_convert, expected)
 
 
-class BinaryPackageTests(TestHelpersMixin, TestCase):
+class BinaryPackageTests(TestCase):
     """Tests for BinaryPackage."""
 
     def test_create(self):
@@ -581,7 +587,7 @@ class BinaryPackageTests(TestHelpersMixin, TestCase):
                 "Package": "hello",
                 "Version": version,
                 "Architecture": architecture,
-                "Maintainer": "Example Person <person@example.org>",
+                "Maintainer": "Example Maintainer <example@example.org>",
                 "Description": "Example description",
                 "Source": f"{srcpkg_name} ({srcpkg_version})",
             },
@@ -645,7 +651,7 @@ class BinaryPackageTests(TestHelpersMixin, TestCase):
         )
 
 
-class BinaryPackagesTests(TestHelpersMixin, TestCase):
+class BinaryPackagesTests(TestCase):
     """Tests for BinaryPackages."""
 
     def test_create(self):
@@ -738,7 +744,7 @@ class BinaryPackagesTests(TestHelpersMixin, TestCase):
         )
 
 
-class WorkRequestDebugLogsTests(TestHelpersMixin, TestCase):
+class WorkRequestDebugLogsTests(TestCase):
     """Tests for WorkRequestDebugLogs class."""
 
     def test_create(self):
@@ -772,7 +778,7 @@ class WorkRequestDebugLogsTests(TestHelpersMixin, TestCase):
         )
 
 
-class SourcePackageTests(TestHelpersMixin, TestCase):
+class SourcePackageTests(TestCase):
     """Tests for SourcePackage class."""
 
     def test_create(self):
@@ -865,7 +871,7 @@ class SourcePackageTests(TestHelpersMixin, TestCase):
         )
 
 
-class LintianTests(TestHelpersMixin, TestCase):
+class LintianTests(TestCase):
     """Tests for the LintianArtifact."""
 
     @classmethod
@@ -876,9 +882,17 @@ class LintianTests(TestHelpersMixin, TestCase):
             b"paragraph-in-dep5-copyright gpl-3 [debian/copyright:33]\n"
             b"P: python-ping3 source: maintainer-manual-page [debian/ping3.1]\n"
         )
-        cls.summary_content = json.dumps({"warning": 2, "pedantic": 1}).encode(
-            "utf-8"
-        )
+        cls.summary = {
+            "tags_count_by_severity": {"error": 1, "warning": 1},
+            "package_filename": {
+                "binutils": "binutils_1.0.dsc",
+                "cynthiune.app": "cynthiune.app_1.0_amd64.deb",
+            },
+            "tags_found": ["license-problem-convert-utf-code", "vcs-obsolete"],
+            "overridden_tags_found": [],
+            "lintian_version": "1.0.0",
+            "distribution": "bookworm",
+        }
         cls.analysis_content = json.dumps(
             {
                 "tags": [
@@ -912,15 +926,14 @@ class LintianTests(TestHelpersMixin, TestCase):
         analysis = self.create_temporary_file(
             suffix=".json", contents=self.analysis_content
         )
-        summary = json.loads(self.summary_content.decode())
 
         package = LintianArtifact.create(
-            lintian_output=output, analysis=analysis, summary=summary
+            lintian_output=output, analysis=analysis, summary=self.summary
         )
 
         self.assertEqual(package._category, LintianArtifact._category)
         self.assertEqual(
-            package.data, data_models.DebianLintian(summary=summary)
+            package.data, data_models.DebianLintian(summary=self.summary)
         )
         self.assertEqual(
             package.files,
@@ -932,9 +945,8 @@ class LintianTests(TestHelpersMixin, TestCase):
 
     def test_validate_file_analysis_is_json_valid(self):
         """Test validate_file_analysis_is_json: return files."""
-        summary = self.create_temporary_file(contents=self.summary_content)
         analysis = self.create_temporary_file(contents=b"{}")
-        files = {"summary.json": summary, "analysis.json": analysis}
+        files = {"analysis.json": analysis}
 
         self.assertEqual(
             LintianArtifact._validate_file_analysis_is_json(files),
@@ -972,9 +984,8 @@ class LintianTests(TestHelpersMixin, TestCase):
         analysis = self.create_temporary_file(
             suffix=".json", contents=self.analysis_content
         )
-        summary = json.loads(self.summary_content.decode())
         artifact = LintianArtifact.create(
-            lintian_output=output, analysis=analysis, summary=summary
+            lintian_output=output, analysis=analysis, summary=self.summary
         )
 
         # This is to verify that serialize_for_create_artifact() is
@@ -985,7 +996,7 @@ class LintianTests(TestHelpersMixin, TestCase):
         )
 
 
-class AutopkgtestArtifactTests(TestHelpersMixin, TestCase):
+class AutopkgtestArtifactTests(TestCase):
     """Tests for AutopkgtestArtifact class."""
 
     def setUp(self):
@@ -1058,7 +1069,7 @@ class AutopkgtestArtifactTests(TestHelpersMixin, TestCase):
         )
 
 
-class DebianSystemTarballArtifactTests(TestHelpersMixin, TestCase):
+class DebianSystemTarballArtifactTests(TestCase):
     """Tests for DebianSystemTarballArtifactTests."""
 
     def test_create(self):
@@ -1132,7 +1143,7 @@ class DebianSystemTarballArtifactTests(TestHelpersMixin, TestCase):
             )
 
 
-class BlhcArtifactTests(TestHelpersMixin, TestCase):
+class BlhcArtifactTests(TestCase):
     """Tests for BlhcArtifact class."""
 
     @classmethod
@@ -1179,7 +1190,7 @@ class BlhcArtifactTests(TestHelpersMixin, TestCase):
         )
 
 
-class DebianSystemImageArtifactTests(TestHelpersMixin, TestCase):
+class DebianSystemImageArtifactTests(TestCase):
     """Tests for DebianSystemImageArtifactTests."""
 
     def test_create_tar_xz(self):
@@ -1287,3 +1298,106 @@ class DebianSystemImageArtifactTests(TestHelpersMixin, TestCase):
         msg = f"Invalid file name: '{file.name}'. Expected .tar.xz or qcow2"
         with self.assertRaisesRegex(ValueError, msg):
             DebianSystemImageArtifact._validate_files(files)
+
+
+class SigningKeyArtifactTests(TestCase):
+    """Tests for SigningKeyArtifact."""
+
+    def test_create(self) -> None:
+        """`create` correctly encodes the given parameters."""
+        artifact = SigningKeyArtifact.create(
+            data_models.KeyPurpose.UEFI, "0" * 64, b"a public key"
+        )
+
+        self.assertEqual(
+            artifact.data,
+            {
+                "purpose": data_models.KeyPurpose.UEFI,
+                "fingerprint": "0" * 64,
+                "public_key": base64.b64encode(b"a public key").decode(),
+            },
+        )
+        self.assertEqual(artifact.public_key_bytes, b"a public key")
+
+
+class SigningInputArtifactTests(TestCase):
+    """Tests for SigningInputArtifact."""
+
+    def test_create(self) -> None:
+        """Create a valid SigningInputArtifact."""
+        base_dir = self.create_temporary_directory()
+        unsigned = base_dir / "package" / "dir" / "file"
+        unsigned.parent.mkdir(parents=True)
+        unsigned.write_text("contents")
+
+        artifact = SigningInputArtifact.create([unsigned], base_dir)
+
+        artifact.validate_model()
+        self.assertEqual(artifact.data, {"trusted_certs": None})
+        self.assertEqual(artifact.files, {"package/dir/file": unsigned})
+
+    def test_create_trusted_certs(self):
+        """Create a SigningInputArtifact with trusted_certs."""
+        base_dir = self.create_temporary_directory()
+        unsigned_1 = base_dir / "package" / "dir" / "file"
+        unsigned_1.parent.mkdir(parents=True)
+        unsigned_1.write_text("contents")
+        unsigned_2 = base_dir / "package" / "another-dir" / "another-file"
+        unsigned_2.parent.mkdir(parents=True)
+        unsigned_2.write_text("other-contents")
+
+        artifact = SigningInputArtifact.create(
+            [unsigned_1, unsigned_2], base_dir, trusted_certs=["0" * 64]
+        )
+
+        artifact.validate_model()
+        self.assertEqual(artifact.data, {"trusted_certs": ["0" * 64]})
+        self.assertEqual(
+            artifact.files,
+            {
+                "package/dir/file": unsigned_1,
+                "package/another-dir/another-file": unsigned_2,
+            },
+        )
+
+    def test_create_no_files(self):
+        """A SigningInputArtifact must have at least one file."""
+        base_dir = self.create_temporary_directory()
+
+        artifact = SigningInputArtifact.create([], base_dir)
+
+        with self.assertRaisesRegex(ValueError, "Expected at least one file"):
+            artifact.validate_model()
+
+
+class SigningOutputArtifactTests(TestCase):
+    """Tests for SigningOutputArtifact."""
+
+    def test_create(self) -> None:
+        """Create a valid SigningOutputArtifact."""
+        base_dir = self.create_temporary_directory()
+        output_file = base_dir / "package" / "dir" / "file.sig"
+        output_file.parent.mkdir(parents=True)
+        output_file.write_text("signature")
+        result = data_models.SigningResult(
+            file="package/dir/file", output_file="package/dir/file.sig"
+        )
+
+        artifact = SigningOutputArtifact.create(
+            data_models.KeyPurpose.UEFI,
+            "0" * 64,
+            [result],
+            [output_file],
+            base_dir,
+        )
+
+        artifact.validate_model()
+        self.assertEqual(
+            artifact.data,
+            {
+                "purpose": data_models.KeyPurpose.UEFI,
+                "fingerprint": "0" * 64,
+                "results": [result],
+            },
+        )
+        self.assertEqual(artifact.files, {"package/dir/file.sig": output_file})
