@@ -18,11 +18,12 @@ import re
 import shutil
 import signal
 import sys
+from collections.abc import Callable
 from datetime import datetime
 from functools import partial
 from itertools import islice
 from pathlib import Path
-from typing import Any, Callable, Optional
+from typing import Any
 
 from dateutil.parser import isoparse
 
@@ -45,7 +46,9 @@ from debusine.client.config import ConfigHandler
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import DebusineError
 from debusine.client.models import (
+    CreateWorkflowRequest,
     WorkRequestRequest,
+    WorkflowTemplateRequest,
     model_to_json_serializable_dict,
 )
 from debusine.client.utils import get_debian_package
@@ -139,6 +142,80 @@ class Cli:
             "for the work request. YAML format. Defaults to stdin.",
             default="-",
         )
+        create_work_request.add_argument(
+            "--event-reactions",
+            type=argparse.FileType("r"),
+            help="File path in YAML format requesting notifications.",
+        )
+
+        manage_work_request = subparsers.add_parser(
+            "manage-work-request", help="Manage a work request"
+        )
+        manage_work_request.add_argument(
+            "work_request_id", type=int, help="Work request id to manage"
+        )
+        manage_work_request.add_argument(
+            "--set-priority-adjustment",
+            type=int,
+            metavar="ADJUSTMENT",
+            help="Set priority adjustment (positive or negative number)",
+        )
+
+        retry_work_request = subparsers.add_parser(
+            "retry-work-request", help="Retry a failed work request"
+        )
+        retry_work_request.add_argument(
+            "work_request_id", type=int, help="Work request id to retry"
+        )
+
+        create_workflow_template = subparsers.add_parser(
+            "create-workflow-template", help="Create a workflow template"
+        )
+        create_workflow_template.add_argument(
+            "template_name", type=str, help="Name of the new workflow template"
+        )
+        create_workflow_template.add_argument(
+            "task_name",
+            type=str,
+            help="Name of the workflow task that the template will use",
+        )
+        create_workflow_template.add_argument(
+            "--workspace", type=str, help="Workspace name"
+        )
+        create_workflow_template.add_argument(
+            "--data",
+            type=argparse.FileType("r"),
+            default="-",
+            help=(
+                "File path (or - for stdin) to read the data or the workflow "
+                "template. YAML format. Defaults to stdin."
+            ),
+        )
+        create_workflow_template.add_argument(
+            "--priority",
+            type=int,
+            default=0,
+            help="Base priority for work requests created from this template",
+        )
+
+        create_workflow = subparsers.add_parser(
+            "create-workflow", help="Create a workflow from a template"
+        )
+        create_workflow.add_argument(
+            "template_name", type=str, help="Workflow template name"
+        )
+        create_workflow.add_argument(
+            "--workspace", type=str, help="Workspace name"
+        )
+        create_workflow.add_argument(
+            "--data",
+            type=argparse.FileType("r"),
+            default="-",
+            help=(
+                "File path (or - for stdin) to read the data for the workflow. "
+                "YAML format. Defaults to stdin."
+            ),
+        )
 
         create_artifact = subparsers.add_parser(
             "create-artifact", help="Create an artifact"
@@ -310,60 +387,93 @@ class Cli:
         debusine = self._build_debusine_object()
         self._setup_http_logging()
 
-        sub_command = getattr(self.args, 'sub-command')
+        match getattr(self.args, 'sub-command'):
+            case "list-work-requests":
+                self._list_work_requests(debusine, self.args.limit)
+            case "show-work-request":
+                self._show_work_request(debusine, self.args.work_request_id)
+            case "create-work-request":
+                self.execute_create_work_request(debusine)
+            case "manage-work-request":
+                self._manage_work_request(
+                    debusine,
+                    self.args.work_request_id,
+                    priority_adjustment=self.args.set_priority_adjustment,
+                )
+            case "retry-work-request":
+                self._retry_work_request(
+                    debusine,
+                    self.args.work_request_id,
+                )
+            case "create-artifact":
+                if self.args.data is not None:
+                    data = self.args.data.read()
+                    self.args.data.close()
+                    if self.args.data == sys.stdin:
+                        print("---")  # avoid confusion between input and output
+                else:
+                    data = None
 
-        if sub_command == 'list-work-requests':
-            self._list_work_requests(debusine, self.args.limit)
-        elif sub_command == 'show-work-request':
-            self._show_work_request(debusine, self.args.work_request_id)
-        elif sub_command == 'create-work-request':
-            data = self.args.data.read()
-            self.args.data.close()
+                self._create_artifact(
+                    debusine,
+                    self.args.category,
+                    self.args.workspace,
+                    self.args.upload,
+                    self.args.upload_base_directory,
+                    self.args.expire_at,
+                    data,
+                )
+            case "import-debian-artifact":
+                self._import_debian_artifact(
+                    debusine,
+                    self.args.workspace,
+                    self.args.upload,
+                    self.args.expire_at,
+                )
+            case "download-artifact":
+                self._download_artifact(
+                    debusine,
+                    self.args.id,
+                    tarball=self.args.tarball,
+                    target_directory=self.args.target_directory,
+                )
+            case "show-artifact":
+                self._show_artifact(debusine, self.args.id)
+            case "on-work-request-completed":
+                self._on_work_request_completed(
+                    debusine,
+                    workspaces=self.args.workspace,
+                    last_completed_at=self.args.last_completed_at,
+                    command=self.args.command,
+                )
+            case "create-workflow-template":
+                self.execute_create_workflow_template(debusine)
+            case "create-workflow":
+                self.execute_create_workflow(debusine)
+            case _ as unreachable:
+                raise AssertionError(
+                    f"Unexpected sub-command name: {unreachable}"
+                )
 
-            self._create_work_request(
-                debusine, self.args.task_name, self.args.workspace, data
-            )
-        elif sub_command == "create-artifact":
-            if self.args.data is not None:
-                data = self.args.data.read()
-                self.args.data.close()
-            else:
-                data = None
+    def execute_create_work_request(self, debusine):
+        """Execute the work request action requested by the user."""
+        data = self.args.data.read()
+        self.args.data.close()
+        if self.args.data == sys.stdin:
+            print("---")  # avoid confusion between input and output
 
-            self._create_artifact(
-                debusine,
-                self.args.category,
-                self.args.workspace,
-                self.args.upload,
-                self.args.upload_base_directory,
-                self.args.expire_at,
-                data,
-            )
-        elif sub_command == "import-debian-artifact":
-            self._import_debian_artifact(
-                debusine,
-                self.args.workspace,
-                self.args.upload,
-                self.args.expire_at,
-            )
-        elif sub_command == "download-artifact":
-            self._download_artifact(
-                debusine,
-                self.args.id,
-                tarball=self.args.tarball,
-                target_directory=self.args.target_directory,
-            )
-        elif sub_command == "show-artifact":
-            self._show_artifact(debusine, self.args.id)
-        elif sub_command == "on-work-request-completed":
-            self._on_work_request_completed(
-                debusine,
-                workspaces=self.args.workspace,
-                last_completed_at=self.args.last_completed_at,
-                command=self.args.command,
-            )
-        else:  # pragma: no cover
-            pass  # Can never be reached
+        event_reactions = '{}'
+        if self.args.event_reactions:
+            event_reactions = self.args.event_reactions.read()
+            self.args.event_reactions.close()
+
+        self._create_work_request(
+            debusine,
+            self.args.task_name,
+            self.args.workspace,
+            data,
+            event_reactions,
+        )
 
     @staticmethod
     def _print_yaml(dictionary):
@@ -409,12 +519,21 @@ class Cli:
         self._print_yaml(result)
 
     def _create_work_request(
-        self, debusine: Debusine, task_name: str, workspace: str, task_data
+        self,
+        debusine: Debusine,
+        task_name: str,
+        workspace: str,
+        task_data,
+        event_reactions,
     ):
         task_data = self._parse_yaml_data(task_data)
+        event_reactions = self._parse_yaml_data(event_reactions)
 
         work_request = WorkRequestRequest(
-            task_name=task_name, workspace=workspace, task_data=task_data
+            task_name=task_name,
+            workspace=workspace,
+            task_data=task_data,
+            event_reactions=event_reactions,
         )
 
         try:
@@ -433,6 +552,134 @@ class Cli:
             }
         self._print_yaml(output)
 
+    def _manage_work_request(
+        self,
+        debusine: Debusine,
+        work_request_id: int,
+        *,
+        priority_adjustment: int | None = None,
+    ) -> None:
+        """Make changes to an existing work request."""
+        if priority_adjustment is None:
+            self._fail("Error: no changes specified")
+
+        self._api_call_or_fail(
+            debusine.work_request_update,
+            work_request_id,
+            priority_adjustment=priority_adjustment,
+        )
+
+    def _retry_work_request(
+        self,
+        debusine: Debusine,
+        work_request_id: int,
+    ) -> None:
+        """Retry an aborted/failed work request."""
+        self._api_call_or_fail(
+            debusine.work_request_retry,
+            work_request_id,
+        )
+
+    def _create_workflow_template(
+        self,
+        debusine: Debusine,
+        *,
+        template_name: str,
+        task_name: str,
+        workspace: str | None,
+        task_data: str,
+        priority: int,
+    ) -> None:
+        parsed_task_data = self._parse_yaml_data(task_data)
+
+        workflow_template = WorkflowTemplateRequest(
+            name=template_name,
+            task_name=task_name,
+            workspace=workspace,
+            task_data=parsed_task_data,
+            priority=priority,
+        )
+
+        try:
+            workflow_template_created = self._api_call_or_fail(
+                debusine.workflow_template_create, workflow_template
+            )
+        except DebusineError as err:
+            output = {"result": "failure", "error": err.asdict()}
+        else:
+            output = {
+                "result": "success",
+                "message": (
+                    f"Workflow template registered on {debusine.base_api_url} "
+                    f"with id {workflow_template_created.id}."
+                ),
+                "workflow_template_id": workflow_template_created.id,
+            }
+        self._print_yaml(output)
+
+    def execute_create_workflow_template(self, debusine: Debusine) -> None:
+        """Execute the workflow template action requested by the user."""
+        data = self.args.data.read()
+        self.args.data.close()
+        if self.args.data == sys.stdin:
+            print("---")  # avoid confusion between input and output
+
+        self._create_workflow_template(
+            debusine,
+            template_name=self.args.template_name,
+            task_name=self.args.task_name,
+            workspace=self.args.workspace,
+            task_data=data,
+            priority=self.args.priority,
+        )
+
+    def _create_workflow(
+        self,
+        debusine: Debusine,
+        *,
+        template_name: str,
+        workspace: str | None,
+        task_data: str,
+    ) -> None:
+        parsed_task_data = self._parse_yaml_data(task_data)
+
+        workflow = CreateWorkflowRequest(
+            template_name=template_name,
+            workspace=workspace,
+            task_data=parsed_task_data,
+        )
+
+        try:
+            workflow_created = self._api_call_or_fail(
+                debusine.workflow_create, workflow
+            )
+        except DebusineError as err:
+            output = {"result": "failure", "error": err.asdict()}
+        else:
+            output = {
+                "result": "success",
+                "message": (
+                    f"Workflow created on {debusine.base_api_url} with id "
+                    f"{workflow_created.id}."
+                ),
+                "workflow_id": workflow_created.id,
+            }
+        self._print_yaml(output)
+
+    def execute_create_workflow(self, debusine: Debusine) -> None:
+        """Execute the workflow action requested by the user."""
+        data = self.args.data.read()
+        self.args.data.close()
+        if self.args.data == sys.stdin:
+            print("---")  # avoid confusion between input and output
+
+        self._create_workflow(
+            debusine,
+            template_name=self.args.template_name,
+            workspace=self.args.workspace,
+            task_data=data,
+        )
+
     def _api_call_or_fail(self, method, *args, **kwargs):
         """
         Call method with args and kwargs.
@@ -457,7 +704,7 @@ class Cli:
         return result
 
     @classmethod
-    def _parse_yaml_data(cls, data_yaml: Optional[str]) -> dict[str, Any]:
+    def _parse_yaml_data(cls, data_yaml: str | None) -> dict[str, Any]:
         if data_yaml is not None:
             if data_yaml.strip() == "":
                 cls._fail("Error: data must be a dictionary. It is empty")
@@ -485,8 +732,8 @@ class Cli:
         artifact_type: str,
         workspace: str,
         upload_paths: list[str],
-        upload_base_directory: Optional[str],
-        expire_at: Optional[datetime],
+        upload_base_directory: str | None,
+        expire_at: datetime | None,
         data_yaml,
     ):
         data = self._parse_yaml_data(data_yaml)
@@ -632,7 +879,7 @@ class Cli:
         debusine: Debusine,
         workspace: str,
         upload: str,
-        expire_at: Optional[datetime],
+        expire_at: datetime | None,
     ):
         result: dict[str, Any] = {}
         uploaded = []
@@ -731,7 +978,7 @@ class Cli:
         debusine: Debusine,
         *,
         workspaces: list[str],
-        last_completed_at: Optional[Path],
+        last_completed_at: Path | None,
         command: str,
     ):
         if shutil.which(command) is None:
