@@ -26,7 +26,7 @@ import tempfile
 import urllib
 from collections.abc import Iterable
 from pathlib import Path
-from typing import Any, Optional, Protocol
+from typing import Any, Literal, Protocol, TYPE_CHECKING, overload
 from urllib.parse import urlencode
 
 import aiohttp
@@ -38,11 +38,20 @@ from requests_toolbelt.downloadutils.stream import stream_response_to_file
 import tenacity
 
 from debusine.artifacts import LocalArtifact
+from debusine.artifacts.models import CollectionCategory
 from debusine.client.debusine_http_client import DebusineHttpClient
 from debusine.client.exceptions import TokenDisabledError
 from debusine.client.file_uploader import FileUploader
 from debusine.client.models import (
     ArtifactResponse,
+    CollectionItemType,
+    CreateWorkflowRequest,
+    LookupMultipleRequest,
+    LookupMultipleResponse,
+    LookupSingleRequest,
+    LookupSingleResponse,
+    LookupSingleResponseArtifact,
+    LookupSingleResponseCollection,
     OnWorkRequestCompleted,
     RelationCreateRequest,
     RelationCreateRequestType,
@@ -50,8 +59,13 @@ from debusine.client.models import (
     RemoteArtifact,
     WorkRequestRequest,
     WorkRequestResponse,
+    WorkflowTemplateRequest,
+    WorkflowTemplateResponse,
 )
 
+if TYPE_CHECKING:
+    from debusine.tasks.models import LookupMultiple, LookupSingle
+
 
 class _MessageProcessor(Protocol):
     """A callable that processes a message from the server."""
@@ -67,7 +81,7 @@ class Debusine:
     def __init__(
         self,
         base_api_url: str,
-        api_token: Optional[str] = None,
+        api_token: str | None = None,
         *,
         logger: logging.Logger,
     ):
@@ -78,7 +92,7 @@ class Debusine:
         :param api_token: optional token to be used for the calls.
         """
         self.base_api_url: str = base_api_url
-        self.token: Optional[str] = api_token
+        self.token: str | None = api_token
         self.api_url = base_api_url.rstrip("/") + "/" + self.API_VERSION
 
         self._logger = logger
@@ -108,6 +122,27 @@ class Debusine:
             f"/work-request/{work_request_id}/", WorkRequestResponse
         )
 
+    def work_request_update(
+        self, work_request_id: int, *, priority_adjustment: int
+    ) -> None:
+        """Update properties of work_request_id."""
+        return self._debusine_http_client.patch(
+            path=f"/work-request/{work_request_id}/",
+            data={"priority_adjustment": priority_adjustment},
+            expected_class=WorkRequestResponse,
+        )
+
+    def work_request_retry(
+        self,
+        work_request_id: int,
+    ) -> WorkRequestResponse:
+        """Retry a work request."""
+        return self._debusine_http_client.post(
+            path=f"/work-request/{work_request_id}/retry/",
+            data={},
+            expected_class=WorkRequestResponse,
+        )
+
     @staticmethod
     def work_request_completed_path(work_request_id: int) -> str:
         """Return path to update the completed result for work_request_id."""
@@ -144,6 +179,54 @@ class Debusine:
             data=data,
         )
 
+    def workflow_template_create(
+        self, workflow_template: WorkflowTemplateRequest
+    ) -> WorkflowTemplateResponse:
+        """
+        Create a workflow template (via POST /workflow-template/).
+
+        :return: WorkflowTemplate returned by the server.
+        :raises: see _api_request method documentation.
+        """
+        data = workflow_template.dict()
+
+        # If workspace is None: the client does not want to specify a
+        # workspace. Do not send it to the server: the server will assign
+        # the default one.
+        if data["workspace"] is None:
+            del data["workspace"]
+
+        return self._debusine_http_client.post(
+            "/workflow-template/",
+            WorkflowTemplateResponse,
+            data=data,
+            expected_statuses=[requests.codes.created],
+        )
+
+    def workflow_create(
+        self, create_workflow: CreateWorkflowRequest
+    ) -> WorkRequestResponse:
+        """
+        Create a workflow (via POST /workflow/).
+
+        :return: Workflow WorkRequest returned by the server.
+        :raises: see _api_request method documentation.
+        """
+        data = create_workflow.dict()
+
+        # If workspace is None: the client does not want to specify a
+        # workspace. Do not send it to the server: the server will assign
+        # the default one.
+        if data["workspace"] is None:
+            del data["workspace"]
+
+        return self._debusine_http_client.post(
+            "/workflow/",
+            WorkRequestResponse,
+            data=data,
+            expected_statuses=[requests.codes.created],
+        )
+
     def artifact_get(self, artifact_id: int) -> ArtifactResponse:
         """
         Get artifact information.
@@ -159,9 +242,9 @@ class Debusine:
         self,
         artifact: LocalArtifact[Any],
         *,
-        workspace: Optional[str],
-        work_request: Optional[int] = None,
-        expire_at: Optional[datetime.datetime] = None,
+        workspace: str | None,
+        work_request: int | None = None,
+        expire_at: datetime.datetime | None = None,
     ) -> ArtifactResponse:
         """Create artifact in the debusine server."""
         return self._debusine_http_client.post(
@@ -190,7 +273,7 @@ class Debusine:
         self,
         artifact_id: int,
         upload_files: dict[str, Path],
-        base_directory: Optional[Path] = None,
+        base_directory: Path | None = None,
     ) -> None:
         """
         Upload into artifact the files.
@@ -220,9 +303,9 @@ class Debusine:
         self,
         local_artifact: LocalArtifact[Any],
         *,
-        workspace: Optional[str],
-        work_request: Optional[int] = None,
-        expire_at: Optional[datetime.datetime] = None,
+        workspace: str | None,
+        work_request: int | None = None,
+        expire_at: datetime.datetime | None = None,
     ) -> RemoteArtifact:
         """Upload (create and upload files) the local_artifact to the server."""
         artifact_response = self.artifact_create(
@@ -274,13 +357,10 @@ class Debusine:
         )
 
     @staticmethod
-    def _get_fileobj_streaming(url: str, *, headers: dict[str, Optional[str]]):
+    def _get_fileobj_streaming(url: str, *, headers: dict[str, str | None]):
         # Implemented to mock it in unit tests (since responses does not support
         # mocking when stream=True)
-        # TODO: Simplify to just requests.get once
-        # https://github.com/python/typeshed/pull/11370 has landed.
-        with requests.Session() as session:
-            return session.get(url, headers=headers, stream=True)
+        return requests.get(url, headers=headers, stream=True)
 
     def download_artifact(
         self,
@@ -398,6 +478,96 @@ class Debusine:
 
         return artifact
 
+    @overload
+    def lookup_single(
+        self,
+        lookup: "LookupSingle",
+        work_request: int,
+        expect_type: Literal[CollectionItemType.ARTIFACT],
+        default_category: CollectionCategory | None = None,
+    ) -> LookupSingleResponseArtifact: ...
+
+    @overload
+    def lookup_single(
+        self,
+        lookup: "LookupSingle",
+        work_request: int,
+        expect_type: Literal[CollectionItemType.COLLECTION],
+        default_category: CollectionCategory | None = None,
+    ) -> LookupSingleResponseCollection: ...
+
+    @overload
+    def lookup_single(
+        self,
+        lookup: "LookupSingle",
+        work_request: int,
+        expect_type: CollectionItemType,
+        default_category: CollectionCategory | None = None,
+    ) -> LookupSingleResponse: ...
+
+    def lookup_single(
+        self,
+        lookup: "LookupSingle",
+        work_request: int,
+        expect_type: CollectionItemType,
+        default_category: CollectionCategory | None = None,
+    ) -> LookupSingleResponse:
+        """Look up a single collection item."""
+        request = LookupSingleRequest(
+            lookup=lookup,
+            work_request=work_request,
+            expect_type=expect_type,
+            default_category=default_category,
+        )
+        return self._debusine_http_client.post(
+            "/lookup/single/", LookupSingleResponse, data=request.dict()
+        )
+
+    @overload
+    def lookup_multiple(
+        self,
+        lookup: "LookupMultiple",
+        work_request: int,
+        expect_type: Literal[CollectionItemType.ARTIFACT],
+        default_category: CollectionCategory | None = None,
+    ) -> LookupMultipleResponse[LookupSingleResponseArtifact]: ...
+
+    @overload
+    def lookup_multiple(
+        self,
+        lookup: "LookupMultiple",
+        work_request: int,
+        expect_type: Literal[CollectionItemType.COLLECTION],
+        default_category: CollectionCategory | None = None,
+    ) -> LookupMultipleResponse[LookupSingleResponseCollection]: ...
+
+    @overload
+    def lookup_multiple(
+        self,
+        lookup: "LookupMultiple",
+        work_request: int,
+        expect_type: CollectionItemType,
+        default_category: CollectionCategory | None = None,
+    ) -> LookupMultipleResponse[LookupSingleResponse]: ...
+
+    def lookup_multiple(
+        self,
+        lookup: "LookupMultiple",
+        work_request: int,
+        expect_type: CollectionItemType,
+        default_category: CollectionCategory | None = None,
+    ) -> LookupMultipleResponse[LookupSingleResponse]:
+        """Look up multiple collection items."""
+        request = LookupMultipleRequest(
+            lookup=lookup.dict()["__root__"],
+            work_request=work_request,
+            expect_type=expect_type,
+            default_category=default_category,
+        )
+        return self._debusine_http_client.post(
+            "/lookup/multiple/", LookupMultipleResponse, data=request.dict()
+        )
+
     def _log_tenacity_exception(self, retry_state: tenacity.RetryCallState):
         # outcome is always non-None in an "after" hook.
         assert retry_state.outcome is not None
@@ -410,7 +580,7 @@ class Debusine:
         *,
         command: str,
         working_directory: Path,
-        last_completed_at: Optional[Path],
+        last_completed_at: Path | None,
         msg_content,
     ):
         on_work_request_completed = OnWorkRequestCompleted.parse_obj(
@@ -438,7 +608,7 @@ class Debusine:
         url: str,
         command: str,
         working_directory: Path,
-        last_completed_at: Optional[Path],
+        last_completed_at: Path | None,
     ):
         """
         Connect using websockets to URL and wait for msg. Execute command.
@@ -462,14 +632,17 @@ class Debusine:
 
             self._logger.info("Requesting %s", url)
 
+            def connected(msg_content: dict[Any, Any]) -> None:  # noqa: U100
+                self._logger.info("Connected!")
+
             async with session.ws_connect(
                 url, headers=headers, heartbeat=60
             ) as ws:
-                self._logger.info("Connected!")
                 async for msg in ws:  # pragma: no branch
                     await Debusine.process_async_message(
                         msg,
                         {
+                            "connected": connected,
                             "work_request_completed": functools.partial(
                                 self._on_work_request_completed,
                                 command=command,
@@ -485,8 +658,8 @@ class Debusine:
 
     @staticmethod
     def write_last_completed_at(
-        completed_at_file: Optional[Path],
-        completed_at: Optional[datetime.datetime],
+        completed_at_file: Path | None,
+        completed_at: datetime.datetime | None,
     ) -> None:
         """Write to completed_at_file the completed_at datetime."""
         if completed_at_file is None:
@@ -503,8 +676,8 @@ class Debusine:
 
     @staticmethod
     def _read_last_completed_at(
-        last_completed_at: Optional[Path],
-    ) -> Optional[str]:
+        last_completed_at: Path | None,
+    ) -> str | None:
         if last_completed_at is None:
             return None
 
@@ -519,8 +692,8 @@ class Debusine:
     def on_work_request_completed(
         self,
         *,
-        workspaces: Optional[list[str]] = None,
-        last_completed_at: Optional[Path] = None,
+        workspaces: list[str] | None = None,
+        last_completed_at: Path | None = None,
         command: str,
         working_directory: Path,
     ):
