@@ -10,9 +10,11 @@
 """Models used by debusine client."""
 
 import json
+from collections.abc import Sequence
 from datetime import datetime
+from enum import StrEnum
 from pathlib import Path
-from typing import Any, Literal, NewType, Optional
+from typing import Annotated, Any, Generic, Literal, NewType, TypeVar
 
 try:
     import pydantic.v1 as pydantic
@@ -34,9 +36,9 @@ class StrictBaseModel(pydantic.BaseModel):
 class PaginatedResponse(StrictBaseModel):
     """Paginated response from the API."""
 
-    count: Optional[int]
-    next: Optional[pydantic.AnyUrl]
-    previous: Optional[pydantic.AnyUrl]
+    count: int | None
+    next: pydantic.AnyUrl | None
+    previous: pydantic.AnyUrl | None
     results: list[dict[str, Any]]
 
 
@@ -44,8 +46,9 @@ class WorkRequestRequest(StrictBaseModel):
     """Client send a WorkRequest to the server."""
 
     task_name: str
-    workspace: Optional[str] = None
+    workspace: str | None = None
     task_data: dict[str, Any]
+    event_reactions: dict[str, Any]
 
 
 class WorkRequestResponse(StrictBaseModel):
@@ -53,14 +56,18 @@ class WorkRequestResponse(StrictBaseModel):
 
     id: int
     created_at: datetime
-    started_at: Optional[datetime] = None
-    completed_at: Optional[datetime] = None
-    duration: Optional[int] = None
+    started_at: datetime | None = None
+    completed_at: datetime | None = None
+    duration: int | None = None
     status: str
     result: str
-    worker: Optional[int] = None
+    worker: int | None = None
+    task_type: str
     task_name: str
     task_data: dict[str, Any]
+    dynamic_task_data: dict[str, Any] | None
+    priority_base: int
+    priority_adjustment: int
     artifacts: list[int]
     workspace: str
 
@@ -81,35 +88,46 @@ class OnWorkRequestCompleted(StrictBaseModel):
     result: str
 
 
-# Backport of pydantic.NonNegativeInt from 1.8.
-class NonNegativeInt(pydantic.ConstrainedInt):
-    """A non-negative integer."""
+class WorkflowTemplateRequest(StrictBaseModel):
+    """Client sends a WorkflowTemplate to the server."""
 
-    ge = 0
+    name: str
+    task_name: str
+    workspace: str | None = None
+    task_data: dict[str, Any]
+    priority: int
 
 
-# With pydantic >= 2.1.0, use Annotated[str,
-# pydantic.StringConstraints(max_length=255)] instead.  Unfortunately there
-# doesn't seem to be a less ugly way to write this with earlier versions of
-# pydantic that mypy is happy with.
-class StrMaxLength255(pydantic.ConstrainedStr):
-    """A string with a maximum length of 255 characters."""
+class WorkflowTemplateResponse(StrictBaseModel):
+    """Server returns a WorkflowTemplate to the server."""
 
-    max_length = 255
+    id: int
+    task_name: str
+    workspace: str
+    task_data: dict[str, Any]
+    priority: int
+
+
+class CreateWorkflowRequest(StrictBaseModel):
+    """Client sends a workflow creation request to the server."""
+
+    template_name: str
+    workspace: str | None = None
+    task_data: dict[str, Any]
 
 
 class FileRequest(StrictBaseModel):
     """Declare a FileRequest: client sends it to the server."""
 
-    size: NonNegativeInt
-    checksums: dict[str, StrMaxLength255]
+    size: int = pydantic.Field(ge=0)
+    checksums: dict[str, Annotated[str, pydantic.Field(max_length=255)]]
     type: Literal["file"]
 
     @staticmethod
     def create_from(path: Path) -> "FileRequest":
         """Return a FileRequest for the file path."""
         return FileRequest(
-            size=pydantic.parse_obj_as(NonNegativeInt, path.stat().st_size),
+            size=path.stat().st_size,
             checksums={"sha256": calculate_hash(path, "sha256").hex()},
             type="file",
         )
@@ -118,8 +136,8 @@ class FileRequest(StrictBaseModel):
 class FileResponse(StrictBaseModel):
     """Declare a FileResponse: server sends it to the client."""
 
-    size: NonNegativeInt
-    checksums: dict[str, StrMaxLength255]
+    size: int = pydantic.Field(ge=0)
+    checksums: dict[str, Annotated[str, pydantic.Field(max_length=255)]]
     type: Literal["file"]
     url: pydantic.AnyUrl
 
@@ -132,11 +150,11 @@ class ArtifactCreateRequest(StrictBaseModel):
     """Declare an ArtifactCreateRequest: client sends it to the server."""
 
     category: str
-    workspace: Optional[str] = None
+    workspace: str | None = None
     files: FilesRequestType = FilesRequestType({})
     data: dict[str, Any] = {}
-    work_request: Optional[int] = None
-    expire_at: Optional[datetime] = None
+    work_request: int | None = None
+    expire_at: datetime | None = None
 
 
 class ArtifactResponse(StrictBaseModel):
@@ -149,7 +167,7 @@ class ArtifactResponse(StrictBaseModel):
     data: dict[str, Any]
     download_tar_gz_url: pydantic.AnyUrl
     files_to_upload: list[str]
-    expire_at: Optional[datetime] = None
+    expire_at: datetime | None = None
     files: FilesResponseType = FilesResponseType({})
 
 
@@ -177,6 +195,77 @@ class RelationResponse(RelationCreateRequest):
     id: int
 
 
+class CollectionItemType(StrEnum):
+    """A collection item type."""
+
+    BARE = "b"
+    ARTIFACT = "a"
+    COLLECTION = "c"
+
+
+class LookupSingleRequest(StrictBaseModel):
+    """A request from the client to look up a single collection item."""
+
+    lookup: int | str
+    work_request: int
+    expect_type: CollectionItemType
+    default_category: str | None = None
+
+
+class LookupMultipleRequest(StrictBaseModel):
+    """A request from the client to look up multiple collection items."""
+
+    lookup: list[int | str | dict[str, Any]]
+    work_request: int
+    expect_type: CollectionItemType
+    default_category: str | None = None
+
+
+class LookupSingleResponse(StrictBaseModel):
+    """A response from the server with a single lookup result."""
+
+    result_type: CollectionItemType
+    collection_item: int | None = None
+    artifact: int | None = None
+    collection: int | None = None
+
+
+class LookupSingleResponseArtifact(LookupSingleResponse):
+    """
+    A response from the server with a single lookup result for an artifact.
+
+    Used to assist type annotations.
+    """
+
+    result_type: Literal[CollectionItemType.ARTIFACT]
+    artifact: int
+
+
+class LookupSingleResponseCollection(LookupSingleResponse):
+    """
+    A response from the server with a single lookup result for a collection.
+
+    Used to assist type annotations.
+    """
+
+    result_type: Literal[CollectionItemType.COLLECTION]
+    collection: int
+
+
+LSR = TypeVar("LSR", bound=LookupSingleResponse, covariant=True)
+
+
+class LookupMultipleResponse(StrictBaseModel, Generic[LSR]):
+    """A response from the server with multiple lookup results."""
+
+    # This model parses a list, not an object/dict.
+    __root__: Sequence[LSR]
+
+    def __iter__(self):
+        """Iterate over individual results."""
+        return iter(self.__root__)
+
+
 def model_to_json_serializable_dict(
     model: pydantic.BaseModel,
 ) -> dict[Any, Any]:
