@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine developers
+# Copyright 2022 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -19,16 +19,18 @@ import os
 import signal
 import stat
 import textwrap
+from collections.abc import Callable
 from datetime import datetime, timedelta
 from itertools import count
 from pathlib import Path
-from typing import Any, Callable, Literal, Optional, Union
+from typing import Any, Literal
 from unittest import TestCase, mock
 from unittest.mock import MagicMock
 
 import yaml
 
 from debusine.artifacts import LocalArtifact
+from debusine.artifacts.models import ArtifactCategory
 from debusine.client import exceptions
 from debusine.client.cli import Cli
 from debusine.client.config import ConfigHandler
@@ -36,14 +38,17 @@ from debusine.client.debusine import Debusine
 from debusine.client.exceptions import DebusineError, NotFoundError
 from debusine.client.models import (
     ArtifactResponse,
+    CreateWorkflowRequest,
     RemoteArtifact,
     WorkRequestRequest,
+    WorkflowTemplateRequest,
     model_to_json_serializable_dict,
 )
 from debusine.test import TestHelpersMixin
 from debusine.test.utils import (
     create_artifact_response,
     create_work_request_response,
+    create_workflow_template_response,
 )
 
 
@@ -108,8 +113,8 @@ class BaseCliTests(TestHelpersMixin, TestCase):
     def capture_output(
         self,
         func: Callable[..., Any],
-        args: Optional[list[Any]] = None,
-        assert_system_exit_code: Optional[int] = None,
+        args: list[Any] | None = None,
+        assert_system_exit_code: int | None = None,
     ) -> tuple[str, str]:
         """
         Execute func() and return stderr and stdout output.
@@ -435,7 +440,8 @@ class CliCreateArtifactTests(BaseCliTests):
         expected_files_to_upload_count: int,
         stdout: str,
         stderr: str,
-        expected_expire_at: Optional[datetime] = None,
+        expected_expire_at: datetime | None = None,
+        data_from_stdin: bool = False,
     ) -> LocalArtifact[Any]:
         """
         Assert that mocked_create_artifact was called only once and stdout.
@@ -451,6 +457,8 @@ class CliCreateArtifactTests(BaseCliTests):
         :param expected_files_to_upload_count: expected files to upload.
         :param stdout: contains Cli.execute() output to verify expected output.
         :param stderr: contains Cli.execute() output to verify expected output.
+        :param expected_expire_at: expected artifact expiry time.
+        :param data_from_stdin: if True, input data came from stdin.
         """
         mocked_create_artifact.assert_called_once()
 
@@ -469,6 +477,8 @@ class CliCreateArtifactTests(BaseCliTests):
             sort_keys=False,
             width=math.inf,
         )
+        if data_from_stdin:
+            expected = f"---\n{expected}"
 
         self.assertEqual(stdout, expected)
         self.assertEqual(stderr, "")
@@ -482,16 +492,16 @@ class CliCreateArtifactTests(BaseCliTests):
         self,
         category: str,
         *,
-        workspace: Optional[str] = None,
-        files_to_upload: Optional[list[str]] = None,
-        upload_base_directory: Optional[Path] = None,
-        expire_at: Optional[datetime] = None,
-        data: Optional[bytes] = None,
-        data_file: Optional[Union[Path, Literal["-"]]] = None,
-        stdin: Optional[dict[str, Any]] = None,
+        workspace: str | None = None,
+        files_to_upload: list[str] | None = None,
+        upload_base_directory: Path | None = None,
+        expire_at: datetime | None = None,
+        data: bytes | None = None,
+        data_file: Path | Literal["-"] | None = None,
+        stdin: dict[str, Any] | None = None,
         create_artifact_return_value=None,
         create_artifact_side_effect=None,
-        assert_system_exit_code: Optional[int] = None,
+        assert_system_exit_code: int | None = None,
     ) -> tuple[str, str, MagicMock]:
         """
         Call CLI create-artifact with the correct parameters.
@@ -571,15 +581,15 @@ class CliCreateArtifactTests(BaseCliTests):
     def assert_create_artifact_success(
         self,
         files_to_upload: list[Path],
-        upload_base_directory: Optional[Path] = None,
-        expire_at: Optional[datetime] = None,
+        upload_base_directory: Path | None = None,
+        expire_at: datetime | None = None,
     ):
         """
         Cli parse the command line and calls Debusine.create_artifact.
 
         Verify that the correct files_to_upload has been used.
         """
-        artifact_category = "debusine:work-request-debug-logs"
+        artifact_category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         artifact_id = 2
         workspace = "test"
 
@@ -696,7 +706,7 @@ class CliCreateArtifactTests(BaseCliTests):
         )
 
         stderr, stdout, _ = self.create_artifact_via_cli(
-            "debusine:work-request-debug-logs",
+            ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             workspace=workspace,
             create_artifact_return_value=create_artifact_return_value,
             files_to_upload=list(map(str, files_to_upload)),
@@ -707,7 +717,7 @@ class CliCreateArtifactTests(BaseCliTests):
     def test_create_artifact_read_stdin(self):
         """Cli create an artifact reading the data from stdin."""
         artifact_id = 2
-        artifact_category = "debusine:work-request-debug-logs"
+        artifact_category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         workspace = "default"
         data = {}
 
@@ -725,14 +735,20 @@ class CliCreateArtifactTests(BaseCliTests):
         )
 
         artifact_request = self.assert_artifact_is_created(
-            mocked_create_artifact, workspace, artifact_id, 0, stdout, stderr
+            mocked_create_artifact,
+            workspace,
+            artifact_id,
+            0,
+            stdout,
+            stderr,
+            data_from_stdin=True,
         )
         self.assertEqual(artifact_request.data, data)
 
     def test_create_artifact_invalid_data_file_type(self):
         """Cli try to create an artifact, data file does not contain a dict."""
         stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
-            "debian:source-package",
+            ArtifactCategory.SOURCE_PACKAGE,
             data=b"some-text",
             assert_system_exit_code=3,
         )
@@ -744,7 +760,9 @@ class CliCreateArtifactTests(BaseCliTests):
     def test_create_artifact_invalid_yaml_in_data_file(self):
         """Cli try to create an artifact, data file contains invalid YAML."""
         stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
-            "debian:source-package", data=b'"', assert_system_exit_code=3
+            ArtifactCategory.SOURCE_PACKAGE,
+            data=b'"',
+            assert_system_exit_code=3,
         )
 
         self.assertRegex(stderr, "^Error parsing YAML:")
@@ -754,7 +772,7 @@ class CliCreateArtifactTests(BaseCliTests):
         """Cli try to create an artifact with a data file cannot be read."""
         data_file = "/tmp/debusine-test-file-does-not-exist"
         stderr, stdout, mocked_create_artifact = self.create_artifact_via_cli(
-            "debian:source-package",
+            ArtifactCategory.SOURCE_PACKAGE,
             data_file=Path(data_file),
             assert_system_exit_code=2,
         )
@@ -780,7 +798,7 @@ class CliCreateArtifactTests(BaseCliTests):
             workspace=expected_workspace,
             files_to_upload=files_to_upload,
         )
-        artifact_category = "debian:source-package"
+        artifact_category = ArtifactCategory.SOURCE_PACKAGE
         (
             stderr_actual,
             stdout_actual,
@@ -812,7 +830,7 @@ class CliCreateArtifactTests(BaseCliTests):
             stdout_actual,
             mocked_create_artifact,
         ) = self.create_artifact_via_cli(
-            "debusine:work-request-debug-logs",
+            ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             workspace=workspace_name,
             create_artifact_side_effect=create_artifact_side_effect,
         )
@@ -841,7 +859,7 @@ class CliCreateArtifactTests(BaseCliTests):
         paths_to_upload = [str(file_in_dir1), str(file_in_dir2)]
 
         stderr, stdout, _ = self.create_artifact_via_cli(
-            "debusine:work-request-debug-logs",
+            ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
             workspace="some-workspace",
             files_to_upload=paths_to_upload,
             assert_system_exit_code=3,
@@ -875,7 +893,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
     Remote sources are tested through the DGet test suite, so not repeated here.
     """
 
-    def create_dsc(self, directory: Optional[Path] = None) -> list[Path]:
+    def create_dsc(self, directory: Path | None = None) -> list[Path]:
         """
         Create a simple Debian source package in a temporary directory.
 
@@ -937,8 +955,8 @@ class CliImportDebianArtifactTests(BaseCliTests):
             self,
             local_artifact,  # noqa: U100
             *,
-            workspace: Optional[str],  # noqa: U100
-            expire_at: Optional[datetime],  # noqa: U100
+            workspace: str | None,  # noqa: U100
+            expire_at: datetime | None,  # noqa: U100
         ) -> RemoteArtifact:
             if workspace is None:  # pragma: no cover
                 workspace = "System"
@@ -966,9 +984,9 @@ class CliImportDebianArtifactTests(BaseCliTests):
         expected_files: int,
         stdout: str,
         stderr: str,
-        extended_artifacts: Optional[list[str]] = None,
-        related_artifacts: Optional[list[str]] = None,
-        expected_expire_at: Optional[datetime] = None,
+        extended_artifacts: list[str] | None = None,
+        related_artifacts: list[str] | None = None,
+        expected_expire_at: datetime | None = None,
     ) -> list[LocalArtifact[Any]]:
         """
         Assert that mocked_upload_artifact uploaded expected_artifacts only.
@@ -1036,13 +1054,13 @@ class CliImportDebianArtifactTests(BaseCliTests):
     def import_debian_artifact(
         self,
         *,
-        workspace: Optional[str] = None,
-        upload: Optional[str] = None,
-        expire_at: Optional[datetime] = None,
-        upload_artifact_side_effect: Union[
-            BaseException, type[BaseException], None
-        ] = None,
-        assert_system_exit_code: Optional[int] = None,
+        workspace: str | None = None,
+        upload: str | None = None,
+        expire_at: datetime | None = None,
+        upload_artifact_side_effect: (
+            BaseException | type[BaseException] | None
+        ) = None,
+        assert_system_exit_code: int | None = None,
     ) -> tuple[str, str, MagicMock]:
         """
         Call CLI import-debian-artifact with the correct parameters.
@@ -1091,7 +1109,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         dsc: Path,
         files: list[Path],
         workspace: str = "test",
-        expire_at: Optional[datetime] = None,
+        expire_at: datetime | None = None,
     ):
         """
         Call import-debian-artifact to import a source package.
@@ -1126,7 +1144,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
 
     def assertSourcePackage(self, artifact: LocalArtifact[Any]):
         """Assert that artifact is a .dsc with expected metadata."""
-        self.assertEqual(artifact.category, "debian:source-package")
+        self.assertEqual(artifact.category, ArtifactCategory.SOURCE_PACKAGE)
         self.assertEqual(artifact.data.name, "hello-traditional")
         self.assertEqual(artifact.data.type, "dpkg")
         self.assertEqual(artifact.data.version, "1.0")
@@ -1141,7 +1159,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         self, artifact: LocalArtifact[Any], binnmu: bool = False
     ):
         """Assert that artifact is a .deb with expected metadata."""
-        self.assertEqual(artifact.category, "debian:binary-package")
+        self.assertEqual(artifact.category, ArtifactCategory.BINARY_PACKAGE)
         version = "1.0"
         if binnmu:
             version += "+b1"
@@ -1149,7 +1167,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
             "Package": "pkg",
             "Version": version,
             "Architecture": "all",
-            "Maintainer": "Example Person <person@example.org>",
+            "Maintainer": "Example Maintainer <example@example.org>",
             "Description": "Example description",
         }
         if binnmu:
@@ -1167,7 +1185,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         self, artifact: LocalArtifact[Any], binnmu: bool = False
     ):
         """Assert that artifact is a deb set with expected metadata."""
-        self.assertEqual(artifact.category, "debian:binary-packages")
+        self.assertEqual(artifact.category, ArtifactCategory.BINARY_PACKAGES)
         deb_version = "1.0"
         if binnmu:
             deb_version += "+b1"
@@ -1185,7 +1203,7 @@ class CliImportDebianArtifactTests(BaseCliTests):
         self, artifact: LocalArtifact[Any], file_names: set[str], binnmu=False
     ):
         """Assert that artifact is an Upload with expected metadata."""
-        self.assertEqual(artifact.category, "debian:upload")
+        self.assertEqual(artifact.category, ArtifactCategory.UPLOAD)
         self.assertEqual(artifact.data.type, "dpkg")
         expected = "hello-traditional"
         if binnmu:
@@ -1492,18 +1510,21 @@ class CliCreateWorkRequestTests(BaseCliTests):
             },
             sort_keys=False,
         )
-        self.assertEqual(stdout, expected)
+        self.assertEqual(stdout, f"---\n{expected}")
 
     def assert_create_work_request(
         self,
         cli: Cli,
         mocked_post_work_request: MagicMock,
         work_request_request: WorkRequestRequest,
+        data_from_stdin: bool = False,
     ):
         """
         Call cli.execute() and assert the work request is created.
 
         Assert expected stderr/stdout and network call.
+
+        :param data_from_stdin: if True, input data came from stdin.
         """
         stderr, stdout = self.capture_output(cli.execute)
 
@@ -1521,6 +1542,8 @@ class CliCreateWorkRequestTests(BaseCliTests):
             },
             sort_keys=False,
         )
+        if data_from_stdin:
+            expected = f"---\n{expected}"
         self.assertEqual(stdout, expected)
         self.assertEqual(stderr, "")
 
@@ -1534,7 +1557,8 @@ class CliCreateWorkRequestTests(BaseCliTests):
     def verify_create_work_request_scenario(
         self,
         cli_args: list[str],
-        workspace: Optional[str] = None,
+        workspace: str | None = None,
+        event_reactions: dict[str, Any] = {},
         use_stdin: bool = False,
     ):
         """
@@ -1570,15 +1594,27 @@ class CliCreateWorkRequestTests(BaseCliTests):
             )
             data_option = str(data_file)
 
+        if event_reactions:
+            event_reactions_file = self.create_temporary_file(
+                contents=yaml.safe_dump(event_reactions).encode("utf-8")
+            )
+            cli_args += ["--event-reactions", str(event_reactions_file)]
+
         cli = self.create_cli(
             ["create-work-request"] + cli_args + (["--data", data_option])
         )
 
         expected_work_request_request = WorkRequestRequest(
-            task_name=task_name, workspace=workspace, task_data=data
+            task_name=task_name,
+            workspace=workspace,
+            task_data=data,
+            event_reactions=event_reactions,
         )
         self.assert_create_work_request(
-            cli, mocked_post_work_request, expected_work_request_request
+            cli,
+            mocked_post_work_request,
+            expected_work_request_request,
+            data_from_stdin=use_stdin,
         )
 
     def test_create_work_request_specific_workspace(self):
@@ -1587,6 +1623,13 @@ class CliCreateWorkRequestTests(BaseCliTests):
         args = ["sbuild", "--workspace", workspace_name]
         self.verify_create_work_request_scenario(args, workspace=workspace_name)
 
+    def test_create_work_request_event_reactions(self):
+        """Cli parse the command line and provide even_reactions."""
+        args = ["sbuild"]
+        self.verify_create_work_request_scenario(
+            args, event_reactions={'on_success': [{'send-notifications': {}}]}
+        )
+
     def test_create_work_request_success_data_from_file(self):
         """Cli parse the command line and read file to create a work request."""
         args = ["sbuild"]
@@ -1622,8 +1665,7 @@ class CliCreateWorkRequestTests(BaseCliTests):
                 "comment": "yaml.safe_load raises ScannerError",
             },
             {
-                "task_data": "input:\n  source_url: https://example.com\n"
-                " sbuild_options:\n - --post=some_command\n",
+                "task_data": "input:\n  source_url: https://example.com\n" " )",
                 "comment": "yaml.safe_load raises ParserError",
             },
         ]
@@ -1647,6 +1689,53 @@ class CliCreateWorkRequestTests(BaseCliTests):
                 )
 
 
+class CliManageWorkRequestTests(BaseCliTests):
+    """Tests for CLI manage-work-request subcommand."""
+
+    def test_no_changes(self) -> None:
+        """manage-work-request fails if no changes are specified."""
+        cli = self.create_cli(["manage-work-request", "1"])
+
+        stderr, stdout = self.capture_output(
+            cli.execute, assert_system_exit_code=3
+        )
+
+        self.assertEqual(stderr, "Error: no changes specified\n")
+        self.assertEqual(stdout, "")
+
+    def test_set_priority_adjustment(self) -> None:
+        """manage-work-request sets the priority adjustment."""
+        debusine = self.patch_build_debusine_object()
+        cli = self.create_cli(
+            ["manage-work-request", "--set-priority-adjustment", "100", "1"]
+        )
+
+        stderr, stdout = self.capture_output(cli.execute)
+
+        self.assertEqual(stderr, "")
+        self.assertEqual(stdout, "")
+        debusine.return_value.work_request_update.assert_called_once_with(
+            1, priority_adjustment=100
+        )
+
+
+class CliRetryWorkRequestTests(BaseCliTests):
+    """Tests for CLI retry-work-request subcommand."""
+
+    def test_retry(self) -> None:
+        """retry-work-request calls the retry method."""
+        debusine = self.patch_build_debusine_object()
+        cli = self.create_cli(["retry-work-request", "1"])
+
+        stderr, stdout = self.capture_output(cli.execute)
+
+        self.assertEqual(stderr, "")
+        self.assertEqual(stdout, "")
+        debusine.return_value.work_request_retry.assert_called_once_with(
+            1,
+        )
+
+
 class CliWorkRequestListTests(BaseCliTests):
     """Tests for CLI list-work-requests method."""
 
@@ -1689,8 +1778,12 @@ class CliWorkRequestListTests(BaseCliTests):
                         "status": work_request_response.status,
                         "result": work_request_response.result,
                         "worker": work_request_response.worker,
+                        "task_type": work_request_response.task_type,
                         "task_name": work_request_response.task_name,
                         "task_data": work_request_response.task_data,
+                        "dynamic_task_data": None,
+                        "priority_base": 0,
+                        "priority_adjustment": 0,
                         "artifacts": [],
                         "workspace": work_request_response.workspace,
                     }
@@ -1777,8 +1870,14 @@ class CliWorkRequestStatusTests(BaseCliTests):
                 'status': work_request_response.status,
                 'result': work_request_response.result,
                 'worker': work_request_response.worker,
+                "task_type": work_request_response.task_type,
                 'task_name': work_request_response.task_name,
                 'task_data': work_request_response.task_data,
+                "dynamic_task_data": None,
+                "priority_base": work_request_response.priority_base,
+                "priority_adjustment": (
+                    work_request_response.priority_adjustment
+                ),
                 "artifacts": expected_artifacts,
                 "workspace": work_request_response.workspace,
             },
@@ -1787,6 +1886,395 @@ class CliWorkRequestStatusTests(BaseCliTests):
         self.assertEqual(stdout, expected)
 
 
+class CliCreateWorkflowTemplateTests(BaseCliTests):
+    """Tests for the CLI `create-workflow-template` command."""
+
+    def test_invalid_task_name(self):
+        """CLI fails if the task name is bad."""
+        task_name = "task-name"
+
+        mocked_sys_stdin_read = self.patch_sys_stdin_read()
+        mocked_sys_stdin_read.return_value = "{}"
+
+        cli = self.create_cli(["create-workflow-template", "test", task_name])
+
+        with mock.patch.object(
+            Debusine,
+            "workflow_template_create",
+            autospec=True,
+            side_effect=DebusineError({"title": f"invalid {task_name}"}),
+        ):
+            stderr, stdout = self.capture_output(cli.execute)
+
+        expected = yaml.safe_dump(
+            {
+                "result": "failure",
+                "error": {"title": f"invalid {task_name}"},
+            },
+            sort_keys=False,
+        )
+        self.assertEqual(stdout, f"---\n{expected}")
+
+    def assert_create_workflow_template(
+        self,
+        cli: Cli,
+        mocked_workflow_template_create: MagicMock,
+        workflow_template_request: WorkflowTemplateRequest,
+        data_from_stdin: bool = False,
+    ):
+        """
+        Call cli.execute() and assert the workflow template is created.
+
+        Assert expected stderr/stdout and network call.
+
+        :param data_from_stdin: if True, input data came from stdin.
+        """
+        stderr, stdout = self.capture_output(cli.execute)
+
+        workflow_template_id = mocked_workflow_template_create.return_value.id
+
+        expected = yaml.safe_dump(
+            {
+                "result": "success",
+                "message": (
+                    f"Workflow template registered on "
+                    f"https://debusine.debian.org/api with id "
+                    f"{workflow_template_id}."
+                ),
+                "workflow_template_id": workflow_template_id,
+            },
+            sort_keys=False,
+        )
+        if data_from_stdin:
+            expected = f"---\n{expected}"
+        self.assertEqual(stdout, expected)
+        self.assertEqual(stderr, "")
+        mocked_workflow_template_create.assert_called_once_with(
+            mock.ANY, workflow_template_request
+        )
+
+    def verify_create_workflow_template_scenario(
+        self,
+        cli_args: list[str],
+        workspace: str | None = None,
+        priority: int = 0,
+        use_stdin: bool = False,
+    ):
+        """
+        Test create-workflow-template using workspace/stdin/stdin-data.
+
+        :param cli_args: first and second parameters are template_name and
+          task_name.  create-workflow-template is prepended to them.
+        :param workspace: workspace to use, or None if not specified in the
+          cli_args.
+        :param priority: priority to use, or 0 if not specified in the
+          cli_args.
+        :param use_stdin: to pass the generated data: use_stdin or a file.
+        """
+        template_name = cli_args[0]
+        task_name = cli_args[1]
+
+        workflow_template_id = 11
+        data = {"target_distribution": "bookworm"}
+
+        serialized_data = yaml.safe_dump(data)
+
+        if use_stdin:
+            mocked_sys_stdin_read = self.patch_sys_stdin_read()
+            mocked_sys_stdin_read.return_value = serialized_data
+            data_option = "-"
+        else:
+            data_file = self.create_temporary_file(
+                contents=serialized_data.encode("utf-8")
+            )
+            data_option = str(data_file)
+
+        cli = self.create_cli(
+            ["create-workflow-template"] + cli_args + (["--data", data_option])
+        )
+
+        expected_workflow_template_request = WorkflowTemplateRequest(
+            name=template_name,
+            task_name=task_name,
+            workspace=workspace,
+            task_data=data,
+            priority=priority,
+        )
+        with mock.patch.object(
+            Debusine,
+            "workflow_template_create",
+            autospec=True,
+            return_value=create_workflow_template_response(
+                id=workflow_template_id
+            ),
+        ) as mocked_workflow_template_create:
+            self.assert_create_workflow_template(
+                cli,
+                mocked_workflow_template_create,
+                expected_workflow_template_request,
+                data_from_stdin=use_stdin,
+            )
+
+    def test_specific_workspace(self):
+        """CLI parses the command line and uses --workspace."""
+        workspace_name = "Testing"
+        args = ["sbuild-test", "sbuild", "--workspace", workspace_name]
+        self.verify_create_workflow_template_scenario(
+            args, workspace=workspace_name
+        )
+
+    def test_success_data_from_file(self):
+        """CLI creates a workflow template with data from a file."""
+        args = ["sbuild-test", "sbuild"]
+        self.verify_create_workflow_template_scenario(args)
+
+    def test_success_data_from_stdin(self):
+        """CLI creates a workflow template with data from stdin."""
+        args = ["sbuild-test", "sbuild"]
+        self.verify_create_workflow_template_scenario(args, use_stdin=True)
+
+    def test_data_is_empty(self):
+        """CLI rejects a workflow template with empty data."""
+        empty_file = self.create_temporary_file()
+        cli = self.create_cli(
+            [
+                "create-workflow-template",
+                "sbuild-test",
+                "sbuild",
+                "--data",
+                str(empty_file),
+            ]
+        )
+
+        stderr, stdout = self.capture_output(
+            cli.execute, assert_system_exit_code=3
+        )
+
+        self.assertEqual(
+            stderr, "Error: data must be a dictionary. It is empty\n"
+        )
+        self.assertEqual(stdout, "")
+
+    def test_yaml_errors_failed(self):
+        """cli.execute() deals with different invalid task_data."""
+        workflow_templates = [
+            {
+                "task_data": (
+                    "test:\n"
+                    "  name: a-name\n"
+                    "    first-name: some first name"
+                ),
+                "comment": "yaml.safe_load raises ScannerError",
+            },
+            {
+                "task_data": "input:\n  source_url: https://example.com\n )",
+                "comment": "yaml.safe_load raises ParserError",
+            },
+        ]
+
+        mocked_sys_stdin = self.patch_sys_stdin_read()
+
+        for workflow_template in workflow_templates:
+            task_data = workflow_template["task_data"]
+            with self.subTest(task_data):
+                mocked_sys_stdin.return_value = task_data
+
+                cli = self.create_cli(
+                    ["create-workflow-template", "template-name", "task-name"]
+                )
+                stderr, stdout = self.capture_output(
+                    cli.execute, assert_system_exit_code=3
+                )
+
+                self.assertRegex(stderr, "^Error parsing YAML:")
+                self.assertRegex(stderr, "Fix the YAML data\n$")
+
+
+class CliCreateWorkflowTests(BaseCliTests):
+    """Tests for the CLI `create-workflow` command."""
+
+    def test_invalid_template_name(self):
+        """CLI fails if the template name is bad."""
+        template_name = "template-name"
+
+        mocked_sys_stdin_read = self.patch_sys_stdin_read()
+        mocked_sys_stdin_read.return_value = "{}"
+
+        cli = self.create_cli(["create-workflow", template_name])
+
+        with mock.patch.object(
+            Debusine,
+            "workflow_create",
+            autospec=True,
+            side_effect=DebusineError({"title": f"invalid {template_name}"}),
+        ):
+            stderr, stdout = self.capture_output(cli.execute)
+
+        expected = yaml.safe_dump(
+            {
+                "result": "failure",
+                "error": {"title": f"invalid {template_name}"},
+            },
+            sort_keys=False,
+        )
+        self.assertEqual(stdout, f"---\n{expected}")
+
+    def assert_create_workflow(
+        self,
+        cli: Cli,
+        mocked_workflow_create: MagicMock,
+        workflow_request: CreateWorkflowRequest,
+        data_from_stdin: bool = False,
+    ):
+        """
+        Call cli.execute() and assert the workflow template is created.
+
+        Assert expected stderr/stdout and network call.
+
+        :param data_from_stdin: if True, input data came from stdin.
+        """
+        stderr, stdout = self.capture_output(cli.execute)
+
+        workflow_id = mocked_workflow_create.return_value.id
+
+        expected = yaml.safe_dump(
+            {
+                "result": "success",
+                "message": (
+                    f"Workflow created on https://debusine.debian.org/api "
+                    f"with id {workflow_id}."
+                ),
+                "workflow_id": workflow_id,
+            },
+            sort_keys=False,
+        )
+        if data_from_stdin:
+            expected = f"---\n{expected}"
+        self.assertEqual(stdout, expected)
+        self.assertEqual(stderr, "")
+
+        actual_workflow_request = mocked_workflow_create.mock_calls[0][1][1]
+
+        # Verify CreateWorkflowRequest is as expected
+        self.assertEqual(actual_workflow_request, workflow_request)
+
+    def verify_create_workflow_scenario(
+        self,
+        cli_args: list[str],
+        workspace: str | None = None,
+        use_stdin: bool = False,
+    ):
+        """
+        Test create-workflow using workspace/stdin/stdin-data.
+
+        :param cli_args: first parameter is template_name.  create-workflow
+          is prepended to them.
+        :param workspace: workspace to use, or None if not specified in the
+          cli_args.
+        :param use_stdin: to pass the generated data: use_stdin or a file.
+        """
+        template_name = cli_args[0]
+
+        workflow_id = 11
+        data = {"input": {"source_artifact_id": 1}}
+
+        serialized_data = yaml.safe_dump(data)
+
+        if use_stdin:
+            mocked_sys_stdin_read = self.patch_sys_stdin_read()
+            mocked_sys_stdin_read.return_value = serialized_data
+            data_option = "-"
+        else:
+            data_file = self.create_temporary_file(
+                contents=serialized_data.encode("utf-8")
+            )
+            data_option = str(data_file)
+
+        cli = self.create_cli(
+            ["create-workflow"] + cli_args + (["--data", data_option])
+        )
+
+        expected_workflow_request = CreateWorkflowRequest(
+            template_name=template_name, workspace=workspace, task_data=data
+        )
+        with mock.patch.object(
+            Debusine,
+            "workflow_create",
+            autospec=True,
+            return_value=create_work_request_response(id=workflow_id),
+        ) as mocked_workflow_create:
+            self.assert_create_workflow(
+                cli,
+                mocked_workflow_create,
+                expected_workflow_request,
+                data_from_stdin=use_stdin,
+            )
+
+    def test_specific_workspace(self):
+        """CLI parses the command line and uses --workspace."""
+        workspace_name = "Testing"
+        args = ["sbuild-amd64-arm64", "--workspace", workspace_name]
+        self.verify_create_workflow_scenario(args, workspace=workspace_name)
+
+    def test_success_data_from_file(self):
+        """CLI creates a workflow with data from a file."""
+        args = ["sbuild-amd64-arm64"]
+        self.verify_create_workflow_scenario(args)
+
+    def test_success_data_from_stdin(self):
+        """CLI creates a workflow with data from stdin."""
+        args = ["sbuild-amd64-arm64"]
+        self.verify_create_workflow_scenario(args, use_stdin=True)
+
+    def test_data_is_empty(self):
+        """CLI creates a workflow with empty data."""
+        empty_file = self.create_temporary_file()
+        cli = self.create_cli(
+            ["create-workflow", "sbuild-amd64-arm64", "--data", str(empty_file)]
+        )
+
+        stderr, stdout = self.capture_output(
+            cli.execute, assert_system_exit_code=3
+        )
+
+        self.assertEqual(
+            stderr, "Error: data must be a dictionary. It is empty\n"
+        )
+        self.assertEqual(stdout, "")
+
+    def test_yaml_errors_failed(self):
+        """cli.execute() deals with different invalid task_data."""
+        workflows = [
+            {
+                "task_data": (
+                    "test:\n"
+                    "  name: a-name\n"
+                    "    first-name: some first name"
+                ),
+                "comment": "yaml.safe_load raises ScannerError",
+            },
+            {
+                "task_data": "input:\n  source_url: https://example.com\n )",
+                "comment": "yaml.safe_load raises ParserError",
+            },
+        ]
+
+        mocked_sys_stdin = self.patch_sys_stdin_read()
+
+        for workflow in workflows:
+            task_data = workflow["task_data"]
+            with self.subTest(task_data):
+                mocked_sys_stdin.return_value = task_data
+
+                cli = self.create_cli(["create-workflow", "workflow-name"])
+                stderr, stdout = self.capture_output(
+                    cli.execute, assert_system_exit_code=3
+                )
+
+                self.assertRegex(stderr, "^Error parsing YAML:")
+                self.assertRegex(stderr, "Fix the YAML data\n$")
+
+
 class CliDownloadArtifact(BaseCliTests):
     """Tests for the Cli functionality related to downloading an artifact."""
 
@@ -1926,9 +2414,9 @@ class CliOnWorkRequestTests(BaseCliTests):
     def debusine_on_work_request_completed_is_called(
         self,
         *,
-        workspaces: Optional[list[str]] = None,
-        last_completed_at: Optional[Path] = None,
-        silent: Optional[bool] = False,
+        workspaces: list[str] | None = None,
+        last_completed_at: Path | None = None,
+        silent: bool | None = False,
     ):
         """Cli call debusine.on_work_request_completed."""
         command = self.create_temporary_file()
