@@ -1,4 +1,4 @@
-# Copyright 2022-2023 The Debusine developers
+# Copyright 2022-2023 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -20,14 +20,14 @@ import stat
 import tarfile
 import textwrap
 import urllib
+from collections.abc import Callable
 from pathlib import Path
-from typing import Any, Callable, Optional
-from unittest import TestCase, mock
+from typing import Any
+from unittest import mock
 from unittest.mock import MagicMock, call
 from urllib.parse import urlencode
 
 import aiohttp
-from aiohttp.test_utils import unittest_run_loop
 
 try:
     import pydantic.v1 as pydantic
@@ -39,30 +39,36 @@ import requests
 import responses
 
 from debusine.artifacts import WorkRequestDebugLogs
+from debusine.artifacts.models import ArtifactCategory
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
+    CollectionItemType,
+    CreateWorkflowRequest,
     FileResponse,
     FilesResponseType,
-    NonNegativeInt,
+    LookupMultipleResponse,
+    LookupSingleResponse,
     RelationCreateRequest,
     RelationResponse,
     RemoteArtifact,
-    StrMaxLength255,
     WorkRequestRequest,
+    WorkflowTemplateRequest,
     model_to_json_serializable_dict,
 )
 from debusine.client.tests import server
 from debusine.client.tests.server import DebusineAioHTTPTestCase
-from debusine.test import TestHelpersMixin
+from debusine.tasks.models import LookupMultiple
+from debusine.test import TestCase
 from debusine.test.utils import (
     create_artifact_response,
     create_listing_response,
     create_work_request_response,
+    create_workflow_template_response,
 )
 
 
-class DebusineTests(TestHelpersMixin, TestCase):
+class DebusineTests(TestCase):
     """Tests for the Debusine class."""
 
     def setUp(self):
@@ -109,9 +115,49 @@ class DebusineTests(TestHelpersMixin, TestCase):
         )
         self.assert_token_key_included_in_all_requests('token')
 
-    def verify_work_request_create_success(
-        self, workspace: Optional[str] = None
-    ):
+    @responses.activate
+    def test_work_request_update(self) -> None:
+        """Debusine.work_request_update patches the work request."""
+        work_request_id = 10
+        work_request_response = create_work_request_response(
+            id=work_request_id, priority_adjustment=100
+        )
+
+        responses.add(
+            responses.PATCH,
+            f"{self.debusine.api_url}/work-request/{work_request_id}/",
+            json=model_to_json_serializable_dict(work_request_response),
+        )
+
+        self.assertEqual(
+            self.debusine.work_request_update(
+                work_request_id, priority_adjustment=100
+            ),
+            work_request_response,
+        )
+        self.assert_token_key_included_in_all_requests("token")
+
+    @responses.activate
+    def test_work_request_retry(self) -> None:
+        """Debusine.work_request_retry triggers a retry."""
+        work_request_id = 10
+        work_request_response = create_work_request_response(
+            id=work_request_id + 1,
+        )
+
+        responses.add(
+            responses.POST,
+            f"{self.debusine.api_url}/work-request/{work_request_id}" "/retry/",
+            json=model_to_json_serializable_dict(work_request_response),
+        )
+
+        self.assertEqual(
+            self.debusine.work_request_retry(work_request_id),
+            work_request_response,
+        )
+        self.assert_token_key_included_in_all_requests("token")
+
+    def verify_work_request_create_success(self, workspace: str | None = None):
         """
         Verify Debusine.work_request_create().
 
@@ -141,10 +187,11 @@ class DebusineTests(TestHelpersMixin, TestCase):
         work_request_fields: dict[str, Any] = {
             "task_name": "sbuild",
             "task_data": {},
+            "event_reactions": {},
             "workspace": workspace,
         }
-        # "task_name" and "task_data" are always posted
-        fields_to_post = {"task_name", "task_data"}
+        # these fields are always posted
+        fields_to_post = {"task_name", "task_data", "event_reactions"}
 
         # "workspace" is posted only if it's not None
         if workspace is not None:
@@ -212,6 +259,146 @@ class DebusineTests(TestHelpersMixin, TestCase):
         """
         self.verify_work_request_create_success(workspace=None)
 
+    def verify_workflow_template_create_success(
+        self, workspace: str | None = None
+    ):
+        """
+        Verify Debusine.workflow_template_create().
+
+        :param workspace: if it's None: assert that the client's request
+          does not contain "workspace" (it's assigned by the server). The
+          server response contains workspace="System".
+
+          If it's not None: the request contains "workspace" with the
+          correct value.
+        """
+        # Response always contains "workspace".  It's "system" if it was not
+        # in the request.
+        expected_workflow_template_response = create_workflow_template_response(
+            workspace=(workspace if workspace is not None else "System")
+        )
+
+        responses.add(
+            responses.POST,
+            f'{self.debusine.api_url}/workflow-template/',
+            json=model_to_json_serializable_dict(
+                expected_workflow_template_response
+            ),
+            status=requests.codes.created,
+        )
+
+        workflow_template_fields: dict[str, Any] = {
+            "name": "test",
+            "task_name": "noop",
+            "task_data": {},
+            "workspace": workspace,
+            "priority": 0,
+        }
+        # these fields are always posted
+        fields_to_post = {"name", "task_name", "task_data", "priority"}
+
+        # "workspace" is posted only if it's not None
+        if workspace is not None:
+            fields_to_post.add("workspace")
+
+        # workflow_template_to_post can contain "workspace=None". The method
+        # Debusine.workflow_template_create will remove this key if it's
+        # None.
+        workflow_template_to_post = WorkflowTemplateRequest(
+            **workflow_template_fields
+        )
+
+        actual_workflow_template_response = (
+            self.debusine.workflow_template_create(workflow_template_to_post)
+        )
+        self.assertEqual(
+            actual_workflow_template_response,
+            expected_workflow_template_response,
+        )
+
+        self.assert_token_key_included_in_all_requests(self.token)
+
+        # The posted (over the wire) WorkflowTemplateRequest does not
+        # contain "workspace" if it was None.
+        self.assertEqual(
+            json.loads(responses.calls[0].request.body or "null"),
+            workflow_template_to_post.dict(include=fields_to_post),
+        )
+
+    @responses.activate
+    def test_workflow_template_create_success(self):
+        """Post a new workflow template."""
+        self.verify_workflow_template_create_success(workspace="Testing")
+
+    @responses.activate
+    def test_workflow_template_create_no_workspace_success(self):
+        """Post a new workflow template."""
+        self.verify_workflow_template_create_success(workspace=None)
+
+    def verify_workflow_create_success(self, workspace: str | None = None):
+        """
+        Verify Debusine.workflow_create().
+
+        :param workspace: if it's None: assert that the client's request
+          does not contain "workspace" (it's assigned by the server). The
+          server response contains workspace="System".
+
+          If it's not None: the request contains "workspace" with the
+          correct value.
+        """
+        # Response always contains "workspace".  It's "system" if it was not
+        # in the request.
+        expected_workflow_response = create_work_request_response(
+            workspace=(workspace if workspace is not None else "System")
+        )
+
+        responses.add(
+            responses.POST,
+            f"{self.debusine.api_url}/workflow/",
+            json=model_to_json_serializable_dict(expected_workflow_response),
+            status=requests.codes.created,
+        )
+
+        workflow_fields: dict[str, Any] = {
+            "template_name": "test",
+            "task_data": {},
+            "workspace": workspace,
+        }
+        # these fields are always posted
+        fields_to_post = {"template_name", "task_data"}
+
+        # "workspace" is posted only if it's not None
+        if workspace is not None:
+            fields_to_post.add("workspace")
+
+        # workflow_to_post can contain "workspace=None". The method
+        # Debusine.workflow_create will remove this key if it's None.
+        workflow_to_post = CreateWorkflowRequest(**workflow_fields)
+
+        actual_workflow_response = self.debusine.workflow_create(
+            workflow_to_post
+        )
+        self.assertEqual(actual_workflow_response, expected_workflow_response)
+
+        self.assert_token_key_included_in_all_requests(self.token)
+
+        # The posted (over the wire) CreateWorkflowRequest does not contain
+        # "workspace" if it was None.
+        self.assertEqual(
+            json.loads(responses.calls[0].request.body or "null"),
+            workflow_to_post.dict(include=fields_to_post),
+        )
+
+    @responses.activate
+    def test_workflow_create_success(self):
+        """Post a new workflow."""
+        self.verify_workflow_create_success(workspace="Testing")
+
+    @responses.activate
+    def test_workflow_create_no_workspace_success(self):
+        """Post a new workflow with no workspace."""
+        self.verify_workflow_create_success(workspace=None)
+
     def files_for_artifact(self) -> dict[str, Path]:
         """Return dictionary with files to upload in an artifact."""
         return {
@@ -286,7 +473,9 @@ class DebusineTests(TestHelpersMixin, TestCase):
         )
 
         artifact_to_post = WorkRequestDebugLogs(
-            category="debian:source-package", files=files, data={}
+            category=ArtifactCategory.WORK_REQUEST_DEBUG_LOGS,
+            files=files,
+            data={},
         )
 
         actual_artifact_response = self.debusine.artifact_create(
@@ -364,7 +553,7 @@ class DebusineTests(TestHelpersMixin, TestCase):
     @responses.activate
     def test_upload_artifact(self):
         """upload_artifact() uploads the artifact and associated files."""
-        category = "debusine:work-request-debug-logs"
+        category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         workspace = "the-workspace"
         work_request = 6
 
@@ -595,14 +784,9 @@ class DebusineTests(TestHelpersMixin, TestCase):
             files=FilesResponseType(
                 {
                     "system.tar.xz": FileResponse(
-                        size=pydantic.parse_obj_as(
-                            NonNegativeInt, len(contents)
-                        ),
+                        size=len(contents),
                         checksums={
-                            "sha256": pydantic.parse_obj_as(
-                                StrMaxLength255,
-                                hashlib.sha256(contents).hexdigest(),
-                            ),
+                            "sha256": hashlib.sha256(contents).hexdigest()
                         },
                         type="file",
                         url=pydantic.parse_obj_as(
@@ -672,9 +856,102 @@ class DebusineTests(TestHelpersMixin, TestCase):
             "Artifact file downloaded: %s", destination
         )
 
+    @responses.activate
+    def test_lookup_single(self):
+        """lookup_single() performs the requested lookup."""
+        responses.add(
+            responses.POST,
+            f"{self.debusine.api_url}/lookup/single/",
+            json={
+                "result_type": CollectionItemType.ARTIFACT,
+                "collection_item": None,
+                "artifact": 1,
+                "collection": None,
+            },
+        )
+
+        response = self.debusine.lookup_single(
+            lookup="collection/artifact",
+            work_request=1,
+            expect_type=CollectionItemType.ARTIFACT,
+            default_category="test",
+        )
+
+        self.assertEqual(
+            response,
+            LookupSingleResponse(
+                result_type=CollectionItemType.ARTIFACT, artifact=1
+            ),
+        )
+        self.assertEqual(
+            json.loads(responses.calls[0].request.body),
+            {
+                "lookup": "collection/artifact",
+                "work_request": 1,
+                "default_category": "test",
+                "expect_type": CollectionItemType.ARTIFACT,
+            },
+        )
+
+    @responses.activate
+    def test_lookup_multiple(self):
+        """lookup_multiple() performs the requested lookup."""
+        responses.add(
+            responses.POST,
+            f"{self.debusine.api_url}/lookup/multiple/",
+            json=[
+                {
+                    "result_type": CollectionItemType.ARTIFACT,
+                    "collection_item": artifact_id,
+                    "artifact": artifact_id,
+                    "collection": None,
+                }
+                for artifact_id in (1, 2)
+            ],
+        )
+
+        response = self.debusine.lookup_multiple(
+            lookup=LookupMultiple.parse_obj({"collection": "collection"}),
+            work_request=1,
+            expect_type=CollectionItemType.ARTIFACT,
+            default_category="test",
+        )
+
+        self.assertEqual(
+            response,
+            LookupMultipleResponse.parse_obj(
+                [
+                    LookupSingleResponse(
+                        result_type=CollectionItemType.ARTIFACT,
+                        collection_item=artifact_id,
+                        artifact=artifact_id,
+                    )
+                    for artifact_id in (1, 2)
+                ]
+            ),
+        )
+        self.assertEqual(
+            json.loads(responses.calls[0].request.body),
+            {
+                # The lookup is normalized.
+                "lookup": [
+                    {
+                        "category": None,
+                        "child_type": "artifact",
+                        "collection": "collection",
+                        "data_matchers": [],
+                        "name_matcher": None,
+                    }
+                ],
+                "work_request": 1,
+                "default_category": "test",
+                "expect_type": CollectionItemType.ARTIFACT,
+            },
+        )
+
     def test_get_fileobj_streaming(self):
         """Test requests.get() is called as expected."""
-        patcher = mock.patch("requests.sessions.Session.get", autospec=True)
+        patcher = mock.patch("requests.get", autospec=True)
         mocker = patcher.start()
 
         expected_raw = b"test"
@@ -690,10 +967,10 @@ class DebusineTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(r.raw, expected_raw)
 
-        mocker.assert_called_with(mock.ANY, url, headers=headers, stream=True)
+        mocker.assert_called_with(url, headers=headers, stream=True)
 
 
-class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
+class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
     """
     Tests for Debusine client asynchronous functionality.
 
@@ -711,9 +988,7 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         self.logger = mock.create_autospec(spec=logging.Logger)
         self.debusine = Debusine(self.api_url, self.token, logger=self.logger)
 
-    def on_complete_url(
-        self, *, params: Optional[dict[str, str]] = None
-    ) -> str:
+    def on_complete_url(self, *, params: dict[str, str] | None = None) -> str:
         """Return the WebSocket on-completed-url."""
         url = self.api_url + "/ws/1.0/work-request/on-completed/"
         url = url.replace("http", "ws", 1)
@@ -745,7 +1020,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         return command, output_file
 
-    @unittest_run_loop
     async def test_wait_and_execute_command(self):
         """A WorkRequest finished and command is executed."""
         await self.execute_wait_and_execute(
@@ -755,7 +1029,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         )
         self.assertEqual(self.requests[-1].headers["Token"], self.token)
 
-    @unittest_run_loop
     async def test_wait_and_execute_two_commands(self):
         """Two WorkRequests finished: two commands are executed."""
         await self.execute_wait_and_execute(
@@ -764,7 +1037,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             on_completed_work_requests=2,
         )
 
-    @unittest_run_loop
     async def test_wait_and_execute_command_relative_to_working_directory(self):
         """A WorkRequest finished and command is executed."""
         await self.execute_wait_and_execute(
@@ -833,7 +1105,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             ],
         )
 
-    @unittest_run_loop
     async def test_wait_and_execute_command_write_last_completed_file(self):
         """wait_and_execute_command update last_completed_at file."""
         last_completed_at = self.create_temporary_file()
@@ -867,7 +1138,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         return mocked
 
-    @unittest_run_loop
     async def test_wait_and_execute_command_retry(self):
         """Client get disconnected: try again."""
         attempts = 2
@@ -907,7 +1177,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.logger.error.assert_has_calls([call("  Error: %s", exc)] * 3)
 
-    @unittest_run_loop
     async def test_wait_and_execute_command_fail(self):
         """Command cannot be executed: raise FileNotFoundError."""
         with self.assertRaises(FileNotFoundError):
@@ -918,7 +1187,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
                 last_completed_at=None,
             )
 
-    @unittest_run_loop
     async def test_wait_and_execute_no_token(self):
         """_wait_and_execute works even if it has no token."""
         self.debusine = Debusine(self.api_url, None, logger=self.logger)
@@ -929,7 +1197,6 @@ class DebusineAsyncTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         )
         self.assertNotIn("Token", self.requests[-1].headers)
 
-    @unittest_run_loop
     async def test_tenacity_sleep(self):
         """Debusine._tenacity_sleep call asyncio.sleep."""
         patched = mock.patch("asyncio.sleep", autospec=True)
