@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine developers
+# Copyright 2023 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -12,7 +12,7 @@
 import json
 from collections import deque
 from itertools import count
-from typing import Any, Optional
+from typing import Any
 from unittest import TestCase, mock
 
 import requests
@@ -28,8 +28,10 @@ from debusine.test import TestHelpersMixin
 class TestResponse(StrictBaseModel):
     """A dummy model for test cases."""
 
+    __test__ = False
+
     id: int
-    page: Optional[int] = None
+    page: int | None = None
 
 
 class DebusineHttpClientTests(TestHelpersMixin, TestCase):
@@ -101,25 +103,16 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
             if pageno > 1:
                 body["previous"] = base_url + f"?page={pageno - 1}"
 
-            try:
-                # bookworm
-                from responses.matchers import query_string_matcher  # type: ignore # noqa: E501
-            except ModuleNotFoundError:  # pragma: no cover
-                # bullseye
-                responses.add(
-                    responses.GET, url, json=body, match_querystring=True
-                )
+            from responses.matchers import query_string_matcher  # type: ignore # noqa: E501
 
-            else:
-                # bookworm
-                url_without_query_string, _, query_string = url.partition("?")
+            url_without_query_string, _, query_string = url.partition("?")
 
-                responses.add(
-                    responses.GET,
-                    url_without_query_string,
-                    json=body,
-                    match=[query_string_matcher(query_string)],
-                )
+            responses.add(
+                responses.GET,
+                url_without_query_string,
+                json=body,
+                match=[query_string_matcher(query_string)],
+            )
 
             if not objects:
                 break
@@ -161,7 +154,7 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
         """Raise ValueError if the method is not recognized."""
         debusine_client = DebusineHttpClient(self.api_url, self.token)
         with self.assertRaisesRegex(
-            ValueError, "^Method must be one of: GET, POST, PUT$"
+            ValueError, "^Method must be one of: GET, POST, PUT, PATCH$"
         ):
             debusine_client._api_request("BAD-METHOD", "", TestResponse)
 
@@ -214,6 +207,31 @@ class DebusineHttpClientTests(TestHelpersMixin, TestCase):
         ):
             debusine_client._api_request("GET", self.api_url, TestResponse)
 
+    @responses.activate
+    def test_api_request_raise_403_error_with_detail(self):
+        """Raise TokenDisabledError: server returned HTTP 403 with detail."""
+        title = "Error"
+        detail = "You do not have permission to perform this action."
+
+        self.responses_add_problem(
+            responses.GET,
+            self.api_url,
+            title,
+            detail=detail,
+            status=requests.codes.forbidden,
+        )
+
+        debusine_client = DebusineHttpClient(self.api_url, self.token)
+        with self.assertRaises(exceptions.DebusineError) as debusine_error:
+            debusine_client._api_request("GET", self.api_url, TestResponse)
+
+        exception = debusine_error.exception
+
+        self.assertEqual(
+            exception.problem,
+            {"title": title, "detail": detail, "validation_errors": None},
+        )
+
     @responses.activate
     def test_api_request_raise_unexpected_error(self):
         """Raise UnexpectedResponseError for unexpected status code."""
