@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine developers
+# Copyright 2022 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -10,11 +10,8 @@
 """Unit tests for the FileUploader class."""
 
 import mmap
-from importlib.metadata import version
 from unittest import TestCase, mock
 
-from debian import debian_support  # type: ignore
-
 import requests
 
 import responses
@@ -34,36 +31,21 @@ def body_matcher(expected_body):  # pragma: no cover
 
     def match(request):
         """Return True if request_body matches expected_body."""
-        responses_version = version("responses")
-
-        if debian_support.version_compare(responses_version, "0.18.0") >= 0:
-            # bookworm
-            if isinstance(request.body, mmap.mmap):
-                position = request.body.tell()
-                contents = request.body.read()
-                request.body.seek(position)
-            else:
-                contents = request.body
-
-            matched = expected_body == contents
-
-            if matched:
-                reason = "Bodies matched"
-            else:
-                reason = "Bodies are different"
-
-            return matched, reason
-
+        if isinstance(request.body, mmap.mmap):
+            position = request.body.tell()
+            contents = request.body.read()
+            request.body.seek(position)
         else:
-            # bullseye
-            if isinstance(request, mmap.mmap):
-                position = request.tell()
-                contents = request.read()
-                request.seek(position)
-            else:
-                contents = request
+            contents = request.body
 
-            return expected_body == contents
+        matched = expected_body == contents
+
+        if matched:
+            reason = "Bodies matched"
+        else:
+            reason = "Bodies are different"
+
+        return matched, reason
 
     return match
 
@@ -401,7 +383,7 @@ class FileUploaderTests(TestHelpersMixin, TestCase):
             uploaded_size=2, status_code_upload=requests.codes.conflict
         )
 
-        error_msg = r"Error uploading .* to %s \(%s retries\)" % (
+        error_msg = r"Error uploading .* to {} \({} retries\)".format(
             self.url,
             self.file_uploader._MAX_FILE_UPLOAD_RETRIES,
         )
