@@ -19,7 +19,10 @@ except ImportError:
 
 from debusine.client.models import (
     ArtifactCreateRequest,
+    CollectionItemType,
     FileRequest,
+    LookupMultipleResponse,
+    LookupSingleResponse,
     RelationCreateRequest,
     StrictBaseModel,
     model_to_json_serializable_dict,
@@ -204,6 +207,26 @@ class RelationCreateRequestTests(TestCase):
             RelationCreateRequest(artifact=1, target=2, type="does-not-exist")
 
 
+class LookupMultipleResponseTests(TestCase):
+    """Tests for LookupMultipleResponse."""
+
+    def test_iter(self):
+        """`__iter__` iterates over individual lookup results."""
+        response = LookupMultipleResponse.parse_obj(
+            [{"result_type": "a", "artifact": artifact} for artifact in (1, 2)]
+        )
+
+        self.assertEqual(
+            list(response),
+            [
+                LookupSingleResponse(
+                    result_type=CollectionItemType.ARTIFACT, artifact=artifact
+                )
+                for artifact in (1, 2)
+            ],
+        )
+
+
 class StrictBaseClass(StrictBaseModel):
     """Class to instantiate in StrictBaseModelTests."""
 
