@@ -16,7 +16,7 @@ from collections.abc import Generator, Iterable
 from contextlib import contextmanager
 from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Union
+from typing import TypeAlias, Union
 from urllib.parse import urljoin, urlparse, urlunparse
 
 from debian.deb822 import Changes, Deb822, Dsc
@@ -27,7 +27,7 @@ from debusine.client import exceptions
 
 SOURCE_PACKAGE_HASHES = ("sha256", "sha1", "md5")
 log = logging.getLogger(__name__)
-DownloadedFileStats = dict[str, Union[str, int]]  # py < 3.10 TypeAlias
+DownloadedFileStats: TypeAlias = dict[str, Union[str, int]]
 
 
 def requests_put_or_connection_error(
@@ -69,7 +69,7 @@ def download_file(
                 for hasher in hashers.values():
                     hasher.update(chunk)
             size = f.tell()
-    stats: dict[str, Union[str, int]] = {"size": size}
+    stats: dict[str, str | int] = {"size": size}
     for hash_name, hasher in hashers.items():
         stats[hash_name] = hasher.hexdigest()
     return stats
@@ -161,7 +161,7 @@ class DGet:
         source_name: str,
         filename: str,
         stat_name: str,
-        value: Union[str, int],
+        value: str | int,
     ):
         stats = self.known_files[filename]
         if stat_name in stats:
