@@ -89,11 +89,18 @@ More settings:
 """
 import os.path
 import socket
+from datetime import timedelta
 from os.path import dirname
+from typing import TYPE_CHECKING
 
 import django
 from django.core.exceptions import ImproperlyConfigured
 
+if TYPE_CHECKING:
+    from nacl.public import PrivateKey
+else:
+    PrivateKey = object
+
 if django.VERSION < (3, 2):
     raise ImproperlyConfigured("Debusine needs Django >= 3.2")
 
@@ -365,9 +372,21 @@ LOGGING = {
 
 AUTH_USER_MODEL = "db.User"
 
+CELERY_BEAT_SCHEDULE = {
+    # Run the scheduler once every five minutes, just in case we have a bug
+    # where something forgot to request a run of the scheduler when data
+    # changed.
+    "fallback-scheduler": {
+        "task": "debusine.server.scheduler.schedule_task",
+        "schedule": timedelta(minutes=5),
+    }
+}
 CELERY_BROKER_URL = "redis://localhost:6379"
 CELERY_RESULT_BACKEND = "django-db"
 CELERY_RESULT_BACKEND_TRANSPORT_OPTIONS = {"global_keyprefix": "celery"}
+CELERY_TASK_ROUTES = {
+    "debusine.server.scheduler.schedule_task": {"queue": "scheduler"}
+}
 CELERY_WORKER_PREFETCH_MULTIPLIER = 1
 
 # === Debusine specific settings ===
@@ -375,17 +394,27 @@ CELERY_WORKER_PREFETCH_MULTIPLIER = 1
 # The fully qualified domain name for the Debusine deployment
 DEBUSINE_FQDN = socket.getfqdn()
 
-# Default CHANNELS_LAYERS pointing at a Redis local configuration
+# Default CHANNEL_LAYERS pointing at a Redis local configuration
 CHANNEL_LAYERS = {
     "default": {
-        "BACKEND": "channels_redis.core.RedisChannelLayer",
-        "CONFIG": {"hosts": [("127.0.0.1", 6379)]},
+        "BACKEND": "channels_redis.pubsub.RedisPubSubChannelLayer",
+        "CONFIG": {"hosts": ["redis://localhost:6379"]},
     }
 }
 
 # A default contact email
 DEBUSINE_CONTACT_EMAIL = f'owner@{DEBUSINE_FQDN}'
 
+# Encryption keys used by the signing service to encrypt private keys; the
+# first in the list is used for newly-encrypted private keys, while the
+# others may also be used for decryption
+DEBUSINE_SIGNING_PRIVATE_KEYS: list[PrivateKey] = []
+
+# Allow signing objects that declare a trust chain to these certificate
+# fingerprints.  See
+# https://wiki.debian.org/SecureBoot/Discussion#Describing_the_trust_chain.
+DEBUSINE_SIGNING_TRUSTED_CERTS: list[str] = []
+
 # The lambda functions are evaluated at the end of the settings import
 # logic. They provide default values to settings which have not yet been
 # set (neither above nor in local.py).
