@@ -1,4 +1,4 @@
-# Copyright 2022 The Debusine developers
+# Copyright 2022 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -22,6 +22,7 @@ from django.core.management import call_command
 from django.test import TestCase, override_settings
 
 from debusine.db.models import FileStore
+from debusine.django import checks as common_checks
 from debusine.project import checks
 from debusine.test import TestHelpersMixin
 
@@ -29,6 +30,8 @@ from debusine.test import TestHelpersMixin
 class ChecksTests(TestHelpersMixin, TestCase):
     """Tests for functions in checks.py."""
 
+    playground_memory_file_store = False
+
     @override_settings()
     def test_check_directories_owners_mandatory_dirs_ok(self):
         """_check_directories_owners with mandatory dirs return empty list."""
@@ -157,7 +160,7 @@ class ChecksTests(TestHelpersMixin, TestCase):
         # Ensure that the original one is restored.
         self.addCleanup(self.restore_argv, sys.argv[:])
 
-        for command in checks._migration_commands:
+        for command in common_checks._migration_commands:
             with self.subTest(command=command):
                 sys.argv[1] = command
                 self.assertEqual(
@@ -170,7 +173,7 @@ class ChecksTests(TestHelpersMixin, TestCase):
         failing_result = [
             Error(
                 'Default SECRET_KEY cannot be used in DEBUG=False. Make sure '
-                'to use "SECRET_kEY = read_secret_key(path)" from selected.py '
+                'to use "SECRET_KEY = read_secret_key(path)" from selected.py '
                 'settings file',
                 hint="Generate a secret key using the command: "
                 "$ python3 -c 'from django.core.management.utils import "
@@ -304,7 +307,10 @@ class ChecksTests(TestHelpersMixin, TestCase):
 
     def test_all_migrations_applied(self):
         """all_migrations_applied() do not return any error."""
-        self.assertEqual(checks.all_migrations_applied(app_configs=None), [])
+        self.assertEqual(
+            checks.all_migrations_applied("debusine-admin", app_configs=None),
+            [],
+        )
 
     def _create_empty_migration(self, app_name: str) -> str:
         """Create empty migration file, return name of the migration."""
@@ -347,7 +353,8 @@ class ChecksTests(TestHelpersMixin, TestCase):
         )
 
         self.assertEqual(
-            checks.all_migrations_applied(app_configs=None), [expected_error]
+            checks.all_migrations_applied("debusine-admin", app_configs=None),
+            [expected_error],
         )
 
     @staticmethod
@@ -373,11 +380,14 @@ class ChecksTests(TestHelpersMixin, TestCase):
         # Ensure that the original one is restored.
         self.addCleanup(self.restore_argv, sys.argv[:])
 
-        for command in checks._migration_commands:
+        for command in common_checks._migration_commands:
             with self.subTest(command=command):
                 sys.argv[1] = command
                 self.assertEqual(
-                    checks.all_migrations_applied(app_configs=None), []
+                    checks.all_migrations_applied(
+                        "debusine-admin", app_configs=None
+                    ),
+                    [],
                 )
 
     def test_local_file_store_directories_exist(self):
@@ -453,7 +463,7 @@ class ChecksTests(TestHelpersMixin, TestCase):
 
         self.addCleanup(self.restore_argv, sys.argv[:])
 
-        for command in checks._migration_commands:
+        for command in common_checks._migration_commands:
             with self.subTest(command=command):
                 sys.argv[1] = command
                 self.assertEqual(
