@@ -10,24 +10,27 @@
 """Celery integration for debusine tasks."""
 
 import logging
+from contextlib import ExitStack
 
 from celery import shared_task
 
 from django.db import transaction
 
 from debusine.db.models import WorkRequest
+from debusine.server.tasks import BaseServerTask
 from debusine.tasks import BaseTask, TaskConfigError
+from debusine.tasks.models import TaskTypes
 
 
 logger = logging.getLogger(__name__)
 
 
-class CeleryWorkerRequiresInternalTask(Exception):
-    """Only internal tasks can run on Celery workers."""
+class CeleryWorkerRequiresServerTask(Exception):
+    """Only server tasks can run on Celery workers."""
 
 
 @shared_task
-def run_internal_task(work_request_id: int) -> bool:
+def run_server_task(work_request_id: int) -> bool:
     """Run a :class:`BaseTask` via Celery."""
     try:
         work_request = WorkRequest.objects.get(pk=work_request_id)
@@ -38,8 +41,13 @@ def run_internal_task(work_request_id: int) -> bool:
     task_data = work_request.task_data
     work_request.mark_running()
 
+    if work_request.task_type != TaskTypes.SERVER:
+        logger.error("Task: %s cannot run on a Celery worker", task_name)
+        work_request.mark_completed(WorkRequest.Results.ERROR)
+        raise CeleryWorkerRequiresServerTask
+
     try:
-        task = BaseTask.class_from_name(task_name)(task_data)
+        task = BaseTask.class_from_name(TaskTypes.SERVER, task_name)(task_data)
     except ValueError:
         logger.error("Task: %s does not exist", task_name)
         work_request.mark_completed(WorkRequest.Results.ERROR)
@@ -48,17 +56,14 @@ def run_internal_task(work_request_id: int) -> bool:
         logger.error("Task: %s failed to configure", task_name)
         work_request.mark_completed(WorkRequest.Results.ERROR)
         raise
+    assert isinstance(task, BaseServerTask)
 
-    if not task.TASK_INTERNAL:
-        logger.error("Task: %s cannot run on a Celery worker", task_name)
-        work_request.mark_completed(WorkRequest.Results.ERROR)
-        raise CeleryWorkerRequiresInternalTask
-
-    task.work_request = work_request.id
-    task.workspace = work_request.workspace.name
+    task.set_work_request(work_request)
 
     try:
-        with transaction.atomic():
+        with ExitStack() as stack:
+            if not task.TASK_MANAGES_TRANSACTIONS:
+                stack.enter_context(transaction.atomic())
             result = task.execute_logging_exceptions()
     except Exception:
         logger.error("Task: %s failed to execute", task_name)
