@@ -13,7 +13,7 @@ import asyncio
 import json
 import logging
 from datetime import datetime
-from typing import Any, Optional
+from typing import Any
 from urllib.parse import parse_qsl
 
 from channels.db import database_sync_to_async
@@ -23,9 +23,9 @@ from rest_framework.fields import DateTimeField
 from rest_framework.permissions import BasePermission
 from rest_framework.renderers import JSONRenderer
 
-from debusine.db.models import WorkRequest, Worker, Workspace
+from debusine.db.models import Token, WorkRequest, Worker, Workspace
 from debusine.server.serializers import OnWorkRequestCompleted
-from debusine.server.views import (
+from debusine.server.views.base import (
     IsTokenUserAuthenticated,
     IsWorkerAuthenticated,
 )
@@ -36,7 +36,7 @@ logger = logging.getLogger(__name__)
 class RequestAdaptor:
     """Minimal class used when helper classes expect a Request."""
 
-    def __init__(self, token: Optional[str]):
+    def __init__(self, token: str | None):
         """Initialize RequestAdaptor."""
         self.headers = {}
 
@@ -49,7 +49,7 @@ class ConsumerMixin:
 
     def get_header_value(
         self: AsyncWebsocketConsumer, header_field_name: str
-    ) -> Optional[str]:
+    ) -> str | None:
         """
         Return the value of header_field_name from self.scope['headers'].
 
@@ -69,7 +69,7 @@ class ConsumerMixin:
         self: AsyncWebsocketConsumer,
         reason,
         *,
-        reason_code: Optional[str] = None,
+        reason_code: str | None = None,
     ) -> None:
         """Send JSON rejecting the connection and logs it."""
         msg = {
@@ -87,7 +87,7 @@ class ConsumerMixin:
         self, permission_class: type[BasePermission]
     ) -> bool:
         """Return bool checking if the request has permission to connect."""
-        token_header: Optional[str] = self.get_header_value("token")
+        token_header: str | None = self.get_header_value("token")
         request_adaptor = RequestAdaptor(token_header)
 
         return await database_sync_to_async(permission_class().has_permission)(
@@ -108,13 +108,15 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
     REQUEST_DYNAMIC_METADATA_SECONDS = 3600
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
-        """Initialise WorkerConsumer member variables."""
+        """Initialize WorkerConsumer member variables."""
         super().__init__(*args, **kwargs)
-        self._worker: Optional[Worker] = None
-        self._request_dynamic_metadata_task: Optional[asyncio.Task[None]] = None
+        self._worker: Worker | None = None
+        self._request_dynamic_metadata_task: asyncio.Task[None] | None = None
 
-    async def _channel_layer_group_add(self, token_key: str, channel_name: str):
-        await self.channel_layer.group_add(token_key, channel_name)
+    async def _channel_layer_group_add(
+        self, token_hash: str, channel_name: str
+    ):
+        await self.channel_layer.group_add(token_hash, channel_name)
 
     async def connect(self) -> None:
         """Worker client is connecting."""
@@ -128,18 +130,21 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
             )
             return
 
-        token_header: Optional[str] = self.get_header_value("token")
+        token_header: str | None = self.get_header_value("token")
+        token_hash: str | None = (
+            Token._generate_hash(token_header) if token_header else None
+        )
 
         self._worker = await database_sync_to_async(
             Worker.objects.select_related("token").get
-        )(token__key=token_header)
+        )(token__hash=token_hash)
         # Must be true, since we already checked IsWorkerAuthenticated.
         assert self._worker is not None
         assert self._worker.token is not None
 
         try:
             await self._channel_layer_group_add(
-                self._worker.token.key, self.channel_name
+                self._worker.token.hash, self.channel_name
             )
         except Exception as exc:
             logger.error(  # noqa: G200
@@ -157,6 +162,12 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
             name='dynamic_metadata_refresher',
         )
 
+        await self.send(
+            text_data=json.dumps(
+                {"type": "websocket.send", "text": "connected"}
+            )
+        )
+
         # If there are WorkRequests running or pending: send to the worker
         # "work-request-available" message
         if await database_sync_to_async(self._worker.is_busy)():
@@ -194,9 +205,10 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
 
         if self._worker:
             await self.channel_layer.group_discard(
-                self._worker.token.key, self.channel_name
+                self._worker.token.hash, self.channel_name
             )
 
+            await database_sync_to_async(self._worker.refresh_from_db)()
             await database_sync_to_async(self._worker.mark_disconnected)()
             logger.info(
                 "Worker disconnected: %s (code: %s)",
@@ -211,7 +223,7 @@ class WorkerConsumer(ConsumerMixin, AsyncWebsocketConsumer):
         msg = {
             "type": "connection_closed",
             "reason": 'Token has been disabled '
-            f'(token: "{self._worker.token.key}")',
+            f'(token hash: "{self._worker.token.hash}")',
             "reason_code": "TOKEN_DISABLED",
         }
         await self.send(text_data=json.dumps(msg), close=True)
@@ -225,15 +237,20 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
     """Send work_request_completed notifications."""
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
-        """Initialise OnWorkRequestCompletedConsumer."""
+        """Initialize OnWorkRequestCompletedConsumer."""
         super().__init__(*args, **kwargs)
 
         # If _workspaces is not None: list of the workspaces that are being
         # monitored, otherwise all the workspaces that the client has access to
-        self._workspaces_only: Optional[list[int]] = None
+        self._workspaces_only: list[int] | None = None
 
         self._query_string: dict[str, str] = {}
 
+        # RedisPubSubChannelLayer in channels_redis 4.0.0 has a bug in
+        # group_discard where it fails if group_add hadn't been called.
+        # This was fixed in 4.1.0, but work around it.
+        self._channel_added_to_group: bool = False
+
     async def _get_workspaces_from_query_params(self):
         workspace_names = self._query_string.get("workspaces")
 
@@ -242,20 +259,25 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
 
         workspace_names = workspace_names.split(",")
 
-        workspace_ids = set()
-
-        for workspace_name in workspace_names:
-            # TODO: check that the current user has access to this
+        def get_workspace_ids(workspace_names: list[str]) -> set[int]:
+            # TODO: check that the current user has access to each
             # workspace (need implementing #80)
-            try:
-                workspace = await database_sync_to_async(Workspace.objects.get)(
-                    name=workspace_name
+            names_to_ids = {
+                workspace.name: workspace.id
+                for workspace in Workspace.objects.filter(
+                    name__in=workspace_names
                 )
-            except Workspace.DoesNotExist:
-                raise Workspace.DoesNotExist(
-                    f'Workspace "{workspace_name}" not found'
-                )
-            workspace_ids.add(workspace.id)
+            }
+            for workspace_name in workspace_names:
+                if workspace_name not in names_to_ids:
+                    raise Workspace.DoesNotExist(
+                        f'Workspace "{workspace_name}" not found'
+                    )
+            return set(names_to_ids.values())
+
+        workspace_ids = await database_sync_to_async(get_workspace_ids)(
+            workspace_names
+        )
 
         return workspace_ids
 
@@ -285,14 +307,23 @@ class OnWorkRequestCompletedConsumer(ConsumerMixin, AsyncWebsocketConsumer):
         await self.channel_layer.group_add(
             "work_request_completed", self.channel_name
         )
+        self._channel_added_to_group = True
+
+        await self.send(
+            text_data=json.dumps(
+                {"type": "websocket.send", "text": "connected"}
+            )
+        )
 
         await self._send_pending_work_requests_since_last_completed()
 
     async def disconnect(self, close_code):  # noqa: U100
         """Client is disconnecting, clean-up."""
-        await self.channel_layer.group_discard(
-            "work_request_completed", self.channel_name
-        )
+        if self._channel_added_to_group:
+            await self.channel_layer.group_discard(
+                "work_request_completed", self.channel_name
+            )
+        self._channel_added_to_group = False
 
     @database_sync_to_async
     def _get_pending_work_requests(
