@@ -10,32 +10,31 @@
 """Implementation of external file backends."""
 
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 
 from debusine.db.models import File, FileInStore, FileStore
 from debusine.server.file_backend.interface import FileBackendInterface
+from debusine.server.file_backend.models import (
+    ExternalDebianSuiteBackendConfiguration,
+)
 from debusine.utils import NotSupportedError
 
 
-class ExternalDebianSuiteFileBackend(FileBackendInterface):
+class ExternalDebianSuiteFileBackend(
+    FileBackendInterface[ExternalDebianSuiteBackendConfiguration]
+):
     """Mirror an external repository in debusine."""
 
-    def __init__(
-        self,
-        file_store: FileStore,
-        archive_root_url: str,
-        dist: str,
-        components: str,
-    ):
+    def __init__(self, file_store: FileStore):
         """Initialize instance."""
         super().__init__()
         self.db_store = file_store
         # All files are relative to the archive root
         # https://wiki.debian.org/DebianRepository/Format
-        self.archive_root_url = archive_root_url
+        self.archive_root_url = self.configuration.archive_root_url
         # For Collections to grab and parse the appropriate Packages files
-        self.dist = dist
-        self.components = components
+        self.suite = self.configuration.suite
+        self.components = self.configuration.components
 
     def remove_file(self, db_file: File):  # noqa: U100
         """
@@ -56,7 +55,7 @@ class ExternalDebianSuiteFileBackend(FileBackendInterface):
         raise NotImplementedError()
 
     def add_file(
-        self, local_path: Path, db_file: Optional[File] = None  # noqa: U100
+        self, local_path: Path, db_file: File | None = None  # noqa: U100
     ) -> File:
         """
         Copy local_path to underlying storage and register it in the FileStore.
@@ -68,7 +67,7 @@ class ExternalDebianSuiteFileBackend(FileBackendInterface):
             "ExternalDebianSuiteFileBackend is a read-only file backend"
         )
 
-    def get_local_path(self, db_file: File) -> Optional[Path]:  # noqa: U100
+    def get_local_path(self, db_file: File) -> Path | None:  # noqa: U100
         """
         Return None: no local path for now.
 
@@ -76,7 +75,7 @@ class ExternalDebianSuiteFileBackend(FileBackendInterface):
         """
         return None
 
-    def get_url(self, db_file: File) -> Optional[str]:
+    def get_url(self, db_file: File) -> str | None:
         """Return a URL pointing to the content when possible or None."""
         file_in_store = FileInStore.objects.get(
             file=db_file, store=self.db_store
