@@ -12,32 +12,53 @@
 import contextlib
 import logging
 from abc import ABC, abstractmethod
+from collections.abc import Iterator
+from functools import cached_property
 from pathlib import Path
-from typing import Optional
+from typing import Generic, IO, TypeVar
 
 import requests
 
-from debusine.db.models import File, FileInStore
-from debusine.utils import NotSupportedError
+from debusine.db.models import File, FileInStore, FileStore
+from debusine.server.file_backend.models import FileBackendConfiguration
+from debusine.utils import NotSupportedError, extract_generic_type_arguments
 
 logger = logging.getLogger(__name__)
 
+FBC = TypeVar("FBC", bound=FileBackendConfiguration)
 
-class FileBackendInterface(ABC):
+
+class FileBackendInterface(Generic[FBC], ABC):
     """Interface to operate files in a storage."""
 
-    def __init__(self):
-        """Initialize object."""
-        # self.db_store holds the FileStore in use
-        self.db_store = None
+    db_store: FileStore
+
+    # Class used as the in-memory representation of file backend
+    # configuration.
+    _configuration_type: type[FBC]
+
+    def __init_subclass__(cls, **kwargs):
+        """Register configuration type for a subclass."""
+        super().__init_subclass__(**kwargs)
+
+        # The configuration type, computed by introspecting the type
+        # argument used to specialize this generic class.
+        [cls._configuration_type] = extract_generic_type_arguments(
+            cls, FileBackendInterface
+        )
+
+    @cached_property
+    def configuration(self) -> FBC:
+        """Return the configuration for this file backend."""
+        return self._configuration_type(**self.db_store.configuration)
 
     @abstractmethod
-    def get_url(self, fileobj: File) -> Optional[str]:
+    def get_url(self, fileobj: File) -> str | None:
         """Return a URL pointing to the content when possible or None."""
         raise NotImplementedError()
 
     @abstractmethod
-    def get_local_path(self, fileobj: File) -> Optional[Path]:
+    def get_local_path(self, fileobj: File) -> Path | None:
         """Return the path to a local copy of the file when possible or None."""
         raise NotImplementedError()
 
@@ -62,9 +83,7 @@ class FileBackendInterface(ABC):
         raise NotImplementedError()
 
     @abstractmethod
-    def add_file(
-        self, local_path: Path, fileobj: Optional[File] = None
-    ) -> File:
+    def add_file(self, local_path: Path, fileobj: File | None = None) -> File:
         """
         Copy local_path to underlying storage and register it in the FileStore.
 
@@ -74,7 +93,7 @@ class FileBackendInterface(ABC):
         raise NotImplementedError()
 
     @contextlib.contextmanager
-    def get_stream(self, fileobj: File):
+    def get_stream(self, fileobj: File) -> Iterator[IO[bytes]]:
         """
         Return a file-like object that can be read.
 
