@@ -10,7 +10,7 @@
 """debusine-admin command to list notification channels."""
 
 from debusine.db.models import NotificationChannel
-from debusine.server.management.debusine_base_command import DebusineBaseCommand
+from debusine.django.management.debusine_base_command import DebusineBaseCommand
 from debusine.server.management.utils import print_notification_channels
 
 
