@@ -11,7 +11,7 @@
 
 from django.contrib.auth import get_user_model
 
-from debusine.server.management.debusine_base_command import DebusineBaseCommand
+from debusine.django.management.debusine_base_command import DebusineBaseCommand
 from debusine.server.management.utils import print_users
 
 
@@ -22,6 +22,6 @@ class Command(DebusineBaseCommand):
 
     def handle(self, *args, **options):
         """List users."""
-        users = get_user_model().objects.all()
+        users = get_user_model().objects.filter(is_system=False)
         print_users(self.stdout, users)
         raise SystemExit(0)
