@@ -13,13 +13,17 @@ Server-side Debusine tasks.
 The tasks in this package run in a Celery worker.
 """
 
-from debusine.server.tasks.base import BaseInternalTask
+from debusine.server.tasks.base import BaseServerTask
 
 # Sub-tasks need to be imported in order to be available to BaseTask
 # (e.g. for BaseTask.is_valid_task_name). They are registered via
 # BaseTask.__init_subclass__.
-from debusine.server.tasks.noop import InternalNoop  # noqa: F401, I202
+from debusine.server.tasks.aptmirror import APTMirror  # noqa: F401, I100, I202
+from debusine.server.tasks.noop import ServerNoop  # noqa: F401, I202
+from debusine.server.tasks.update_suite_lintian_collection import (  # noqa: E501, F401, I202
+    UpdateSuiteLintianCollection,
+)
 
 __all__ = [
-    "BaseInternalTask",
+    "BaseServerTask",
 ]
