@@ -7,10 +7,11 @@
 # modified, propagated, or distributed except according to the terms
 # contained in the LICENSE file.
 
-"""Base support for tasks that run on internal workers."""
+"""Base support for tasks that run on server-side Celery workers."""
 
+from abc import ABCMeta
 from pathlib import Path
-from typing import Generic, TypeVar
+from typing import TypeVar
 
 from debusine.artifacts import WorkRequestDebugLogs
 from debusine.db.models import (
@@ -20,16 +21,34 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.tasks import BaseTask
-from debusine.tasks.models import BaseTaskData
+from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, TaskTypes
 
 
 TD = TypeVar("TD", bound=BaseTaskData)
 
 
-class BaseInternalTask(BaseTask[TD], Generic[TD]):
-    """Base class for tasks that run on internal workers."""
+class BaseServerTask(BaseTask[TD, BaseDynamicTaskData], metaclass=ABCMeta):
+    """Base class for tasks that run on server-side Celery workers."""
 
-    TASK_INTERNAL = True
+    TASK_TYPE = TaskTypes.SERVER
+
+    # If True, the task manages its own transactions.  If False, the task is
+    # automatically run within a single transaction.
+    TASK_MANAGES_TRANSACTIONS = False
+
+    # Set by the Celery worker via :py:meth:`set_work_request`.
+    work_request: WorkRequest | None = None
+    workspace: Workspace | None = None
+
+    def set_work_request(self, work_request: WorkRequest) -> None:
+        """Set this task's work request and workspace."""
+        self.work_request = work_request
+        self.workspace = work_request.workspace
+
+        # BaseTask attributes; less useful for server tasks since we have
+        # the model instances, but we set them anyway for compatibility.
+        self.work_request_id = work_request.id
+        self.workspace_name = work_request.workspace.name
 
     def _upload_work_request_debug_logs(self):
         """
@@ -44,17 +63,14 @@ class BaseInternalTask(BaseTask[TD], Generic[TD]):
         if self._debug_log_files_directory is None:
             return
 
-        work_request = WorkRequest.objects.get(pk=self.work_request)
-        workspace = Workspace.objects.get(name=self.workspace)
-
         work_request_debug_logs = WorkRequestDebugLogs.create(
             files=Path(self._debug_log_files_directory.name).glob("*")
         )
 
         artifact = Artifact.objects.create_from_local_artifact(
             work_request_debug_logs,
-            workspace,
-            created_by_work_request=work_request,
+            self.workspace,
+            created_by_work_request=self.work_request,
         )
 
         for source_artifact_id in self._source_artifacts_ids:
