@@ -9,16 +9,16 @@
 
 """Server-side no-operation task."""
 
-from debusine.server.tasks import BaseInternalTask
-from debusine.server.tasks.models import InternalNoopData
+from debusine.server.tasks import BaseServerTask
+from debusine.server.tasks.models import ServerNoopData
 
 
-class InternalNoop(BaseInternalTask[InternalNoopData]):
-    """Task that runs on internal workers and returns a boolean."""
+class ServerNoop(BaseServerTask[ServerNoopData]):
+    """Task that runs on server-side Celery workers and returns a boolean."""
 
     TASK_VERSION = 1
 
-    def execute(self) -> bool:
+    def _execute(self) -> bool:
         """Act as specified by the client."""
         if self.data.exception:
             raise RuntimeError("Client requested an exception")
