@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine developers
+# Copyright 2024 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -11,26 +11,26 @@
 
 from unittest import TestCase
 
-from debusine.server.tasks.noop import InternalNoop
+from debusine.server.tasks.noop import ServerNoop
 from debusine.tasks import TaskConfigError
 
 
-class InternalNoopTaskTests(TestCase):
-    """Test the internalnoop task."""
+class ServerNoopTaskTests(TestCase):
+    """Test the servernoop task."""
 
     def test_execute_result_true(self):
         """If result=True, the task returns True."""
-        noop = InternalNoop({"result": True})
+        noop = ServerNoop({"result": True})
         self.assertTrue(noop.execute())
 
     def test_execute_result_false(self):
         """If result=False, the task returns False."""
-        noop = InternalNoop({"result": False})
+        noop = ServerNoop({"result": False})
         self.assertFalse(noop.execute())
 
     def test_execute_exception(self):
         """If exception=True, the task raises an exception."""
-        noop = InternalNoop({"exception": True})
+        noop = ServerNoop({"exception": True})
         self.assertRaisesRegex(
             RuntimeError, "Client requested an exception", noop.execute
         )
@@ -39,4 +39,4 @@ class InternalNoopTaskTests(TestCase):
         """Assert no additional properties."""
         error_msg = "extra fields not permitted"
         with self.assertRaisesRegex(TaskConfigError, error_msg):
-            InternalNoop({"result": "True", "input": {}})
+            ServerNoop({"result": "True", "input": {}})
