@@ -11,7 +11,7 @@
 
 import asyncio
 from datetime import timedelta
-from typing import Optional, cast
+from typing import cast
 from unittest import mock
 from unittest.mock import PropertyMock
 from urllib.parse import urlencode
@@ -20,7 +20,6 @@ from channels.db import database_sync_to_async
 from channels.routing import URLRouter
 from channels.testing import WebsocketCommunicator
 
-from django.test import TestCase, TransactionTestCase
 from django.utils import timezone
 
 from rest_framework.fields import DateTimeField
@@ -35,7 +34,7 @@ from debusine.db.models import (
 from debusine.server import consumers
 from debusine.server.consumers import WorkerConsumer
 from debusine.server.routing import websocket_urlpatterns
-from debusine.test import TestHelpersMixin
+from debusine.test.django import TestCase, TransactionTestCase
 from debusine.test.utils import date_time_to_isoformat_rest_framework
 
 
@@ -47,9 +46,14 @@ class WebSocketURLTests(TestCase):
         URLRouter(websocket_urlpatterns)
 
 
-class WorkerConsumerMixin(TestHelpersMixin):
+class WorkerConsumerMixin:
     """Methods used by different WorkerConsumer tests classes."""
 
+    CONNECTED_PAYLOAD = {
+        "type": "websocket.send",
+        "text": "connected",
+    }
+
     REQUEST_DYNAMIC_METADATA_PAYLOAD = {
         "type": "websocket.send",
         "text": "request_dynamic_metadata",
@@ -63,11 +67,11 @@ class WorkerConsumerMixin(TestHelpersMixin):
     @database_sync_to_async
     def acreate_worker(self):
         """Async version to return a new Worker."""
-        return self.create_worker()
+        return self._create_worker()
 
-    def create_worker(self):
+    def _create_worker(self):
         """Return a new Worker."""
-        token = self.create_token_enabled()
+        token = self.playground.create_token_enabled()
 
         worker = Worker.objects.create_with_fqdn("computer.lan", token)
 
@@ -82,7 +86,12 @@ class WorkerConsumerMixin(TestHelpersMixin):
 
         return worker
 
-    async def connect(self, *, token_key: Optional[str] = None):
+    async def connect(
+        self,
+        *,
+        token_key: str | None = None,
+        wait_for_fully_connected: bool = True,
+    ):
         """Return WebsocketCommunicator to api/ws/1.0/worker/connect/."""
         headers = [(b'User-Agent', b'A user agent')]
 
@@ -98,6 +107,12 @@ class WorkerConsumerMixin(TestHelpersMixin):
         connected, subprotocol = await communicator.connect()
         cast(TestCase, self).assertTrue(connected)
 
+        if wait_for_fully_connected:
+            cast(TestCase, self).assertEqual(
+                await communicator.receive_json_from(timeout=3),
+                self.CONNECTED_PAYLOAD,
+            )
+
         return communicator
 
     async def connect_to_new_worker(self):
@@ -151,12 +166,14 @@ class WorkerConsumerMixin(TestHelpersMixin):
 
     async def assert_client_rejected(
         self,
-        token_key: Optional[str],
+        token_key: str | None,
         reason: str,
-        reason_code: Optional[str] = None,
+        reason_code: str | None = None,
     ):
         """Assert that the client is rejected with a reason."""
-        communicator = await self.connect(token_key=token_key)
+        communicator = await self.connect(
+            token_key=token_key, wait_for_fully_connected=False
+        )
 
         msg = {
             'type': 'connection_rejected',
@@ -231,7 +248,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         """Connect succeeds and consumer send expected two messages."""
         worker = await self.acreate_worker()
 
-        await database_sync_to_async(self.create_work_request)(
+        await database_sync_to_async(self.playground.create_work_request)(
             status=WorkRequest.Statuses.PENDING, worker=worker
         )
 
@@ -247,7 +264,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         """Connect succeeds and consumer send expected two messages."""
         worker = await self.acreate_worker()
 
-        await database_sync_to_async(self.create_work_request)(
+        await database_sync_to_async(self.playground.create_work_request)(
             status=WorkRequest.Statuses.RUNNING, worker=worker
         )
 
@@ -310,7 +327,7 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         communicator, worker = await self.connect_to_new_worker()
 
         work_request: WorkRequest = await database_sync_to_async(
-            self.create_work_request
+            self.playground.create_work_request
         )(task_name="sbuild")
 
         await database_sync_to_async(work_request.assign_worker)(worker)
@@ -342,6 +359,30 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         # a work-request cannot transition from PENDING to COMPLETED.
         self.assertEqual(work_request.status, work_request.Statuses.RUNNING)
 
+    async def test_disconnect_refreshes_worker_from_db(self) -> None:
+        """
+        Worker disconnect refreshes worker from DB.
+
+        Otherwise asynchronous changes to the worker (such as editing static
+        metadata) may be overwritten.
+        """
+        communicator, worker = await self.connect_to_new_worker()
+
+        def update_static_metadata(worker: Worker) -> None:
+            worker.static_metadata = {"system:architectures": ["amd64", "i386"]}
+            worker.save()
+
+        await database_sync_to_async(update_static_metadata)(worker)
+
+        await communicator.disconnect()
+
+        await database_sync_to_async(worker.refresh_from_db)()
+
+        self.assertEqual(
+            worker.static_metadata, {"system:architectures": ["amd64", "i386"]}
+        )
+        self.assertFalse(worker.connected())
+
     async def test_request_dynamic_metadata_after_connect(self):
         """Debusine sends request_dynamic_metadata_updated after connect."""
         worker = await self.acreate_worker()
@@ -381,7 +422,9 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         mocked = patcher.start()
         self.addCleanup(patcher.stop)
 
-        token = await database_sync_to_async(self.create_token_enabled)()
+        token = await database_sync_to_async(
+            self.playground.create_token_enabled
+        )()
         await database_sync_to_async(Worker.objects.create)(
             registered_at=timezone.now(), token=token
         )
@@ -395,13 +438,13 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
 
         await communicator.send_input({"type": "worker.disabled"})
 
-        key = await database_sync_to_async(lambda: worker.token.key)()
+        hash_ = await database_sync_to_async(lambda: worker.token.hash)()
 
         self.assertEqual(
             await communicator.receive_json_from(),
             {
                 "type": "connection_closed",
-                "reason": f'Token has been disabled (token: "{key}")',
+                "reason": f'Token has been disabled (token hash: "{hash_}")',
                 "reason_code": "TOKEN_DISABLED",
             },
         )
@@ -413,7 +456,9 @@ class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
         communicator, worker = await self.connect_to_new_worker()
 
         def assign_work_request(worker):
-            work_request = self.create_work_request(task_name="sbuild")
+            work_request = self.playground.create_work_request(
+                task_name="sbuild"
+            )
             work_request.assign_worker(worker)
 
         await database_sync_to_async(assign_work_request)(worker)
@@ -482,18 +527,25 @@ class WorkerConsumerTests(WorkerConsumerMixin, TestCase):
         await worker.disconnect(0)
 
 
-class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
+class WorkRequestCompletedConsumerTests(TransactionTestCase):
     """Tests for OnWorkRequestCompletedConsumer class."""
 
+    CONNECTED_PAYLOAD = {
+        "type": "websocket.send",
+        "text": "connected",
+    }
+
     def setUp(self):
         """Initialize test."""
-        self.token = self.create_token_enabled(with_user=True)
+        super().setUp()
+        self.token = self.playground.create_token_enabled(with_user=True)
 
     async def connect(
         self,
         *,
-        token_key: Optional[str] = None,
+        token_key: str | None = None,
         query_params: dict[str, str] = {},
+        wait_for_fully_connected: bool = True,
     ):
         """Connect to the WorkRequestCompleted endpoint."""
         headers = [(b"User-Agent", b"A user agent")]
@@ -513,6 +565,11 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
         self.assertTrue(connected)
 
+        if wait_for_fully_connected:
+            self.assertEqual(
+                await communicator.receive_json_from(), self.CONNECTED_PAYLOAD
+            )
+
         return communicator
 
     async def test_connect(self):
@@ -529,6 +586,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
         consumer.channel_layer = mock.AsyncMock()
         consumer.channel_name = "some-channel-name"
+        consumer._channel_added_to_group = True
         await consumer.disconnect("close_code")
 
         consumer.channel_layer.group_discard.assert_awaited_once_with(
@@ -537,7 +595,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
     async def test_connect_rejected_no_token_header(self):
         """Connect is rejected: no token was provided."""
-        communicator = await self.connect()
+        communicator = await self.connect(wait_for_fully_connected=False)
 
         self.assertEqual(
             await communicator.receive_json_from(),
@@ -555,6 +613,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
         communicator = await self.connect(
             token_key=self.token.key,
             query_params={"workspaces": "does-not-exist"},
+            wait_for_fully_connected=False,
         )
 
         self.assertEqual(
@@ -583,7 +642,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
         now = timezone.now()
 
         work_request_notified = await database_sync_to_async(
-            self.create_work_request
+            self.playground.create_work_request
         )(
             task_name="sbuild",
             assign_new_worker=True,
@@ -595,7 +654,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
         await database_sync_to_async(work_request_notified.save)()
 
         work_request_to_notify = await database_sync_to_async(
-            self.create_work_request
+            self.playground.create_work_request
         )(
             task_name="sbuild",
             assign_new_worker=True,
@@ -638,7 +697,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
         now = timezone.now()
 
-        await database_sync_to_async(self.create_work_request)(
+        await database_sync_to_async(self.playground.create_work_request)(
             task_name="sbuild",
             assign_new_worker=True,
             mark_running=True,
@@ -671,7 +730,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
         now = timezone.now()
 
         work_request_last_created_1 = await database_sync_to_async(
-            self.create_work_request
+            self.playground.create_work_request
         )(
             task_name="sbuild",
             assign_new_worker=True,
@@ -683,7 +742,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
         await database_sync_to_async(work_request_last_created_1.save)()
 
         work_request_last_created_2 = await database_sync_to_async(
-            self.create_work_request
+            self.playground.create_work_request
         )(
             task_name="sbuild",
             assign_new_worker=True,
@@ -731,7 +790,9 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
     async def create_sbuild_work_request(self, workspace: Workspace):
         """."""
-        return await database_sync_to_async(self.create_work_request)(
+        return await database_sync_to_async(
+            self.playground.create_work_request
+        )(
             task_name="sbuild",
             workspace=workspace,
             assign_new_worker=True,
@@ -739,7 +800,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
             result=WorkRequest.Results.SUCCESS,
             task_data={
                 "input": {
-                    "source_artifact_id": 5,
+                    "source_artifact": 5,
                 },
                 "distribution": "stable",
                 "backend": "schroot",
@@ -751,9 +812,9 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
     async def test_work_request_completed_selected_workspace(self):
         """Notification is received: from a monitored workspace."""
         workspace_name = "lts"
-        workspace = await database_sync_to_async(self.create_workspace)(
-            name=workspace_name
-        )
+        workspace = await database_sync_to_async(
+            self.playground.create_workspace
+        )(name=workspace_name)
 
         workspace_names = f"{workspace_name},{workspace_name}"
         communicator = await self.connect(
@@ -786,13 +847,13 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
     async def test_work_request_completed_workspace_not_monitored(self):
         """Notification not received: from a non-monitored workspace."""
         # Need to exist when connecting
-        await database_sync_to_async(self.create_workspace)(
+        await database_sync_to_async(self.playground.create_workspace)(
             name=DEFAULT_WORKSPACE_NAME
         )
 
-        workspace = await database_sync_to_async(self.create_workspace)(
-            name="lts"
-        )
+        workspace = await database_sync_to_async(
+            self.playground.create_workspace
+        )(name="lts")
 
         communicator = await self.connect(
             token_key=self.token.key,
@@ -805,7 +866,7 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
         result = WorkRequest.Results.SUCCESS
 
-        await database_sync_to_async(self.create_work_request)(
+        await database_sync_to_async(self.playground.create_work_request)(
             task_name="sbuild",
             workspace=workspace,
             assign_new_worker=True,
@@ -822,7 +883,9 @@ class WorkRequestCompletedConsumerTests(TestHelpersMixin, TransactionTestCase):
 
         result = WorkRequest.Results.SUCCESS
 
-        workspace = await database_sync_to_async(self.create_workspace)()
+        workspace = await database_sync_to_async(
+            self.playground.create_workspace
+        )()
 
         work_request = await self.create_sbuild_work_request(workspace)
 
