@@ -10,7 +10,6 @@
 """Tests for generic notifications."""
 
 import logging
-from typing import Any, Optional
 from unittest import mock
 
 from asgiref.sync import sync_to_async
@@ -19,6 +18,11 @@ from django.core import mail
 from django.core.mail import EmailMessage
 from django.test import TestCase
 
+try:
+    import pydantic.v1 as pydantic
+except ImportError:
+    import pydantic as pydantic  # type: ignore
+
 from debusine.db.models import (
     FileStore,
     NotificationChannel,
@@ -28,7 +32,11 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.server import notifications
-from debusine.tasks.models import NotificationDataEmail, TaskNotification
+from debusine.tasks.models import (
+    ActionSendNotification,
+    EventReactions,
+    NotificationDataEmail,
+)
 from debusine.test import TestHelpersMixin
 from debusine.test.django import ChannelsHelpersMixin
 
@@ -37,12 +45,12 @@ class NotifyWorkerTokenDisabledTests(ChannelsHelpersMixin, TestCase):
     """Tests for notify_worker_token_disabled."""
 
     def setUp(self):
-        """Initialise test."""
+        """Initialize test."""
         self.token = Token.objects.create()
 
     async def test_message_is_not_sent(self):
         """Test Token with an associated worker: message is sent."""
-        channel = await self.create_channel(self.token.key)
+        channel = await self.create_channel(self.token.hash)
 
         await sync_to_async(self.token.disable)()
 
@@ -54,7 +62,7 @@ class NotifyWorkerTokenDisabledTests(ChannelsHelpersMixin, TestCase):
             fqdn="debusine", token=self.token
         )
 
-        channel = await self.create_channel(self.token.key)
+        channel = await self.create_channel(self.token.hash)
 
         await sync_to_async(self.token.disable)()
 
@@ -67,7 +75,7 @@ class NotifyWorkRequestAssigned(
     """Tests for notify_work_request_assigned."""
 
     def setUp(self):
-        """Initialise test."""
+        """Initialize test."""
         workspace, _ = Workspace.objects.get_or_create(
             name="Tests", default_file_store=FileStore.default()
         )
@@ -90,7 +98,7 @@ class NotifyWorkRequestAssigned(
             "debusine", token=token
         )
 
-        channel = await self.create_channel(token.key)
+        channel = await self.create_channel(token.hash)
 
         await sync_to_async(self.work_request.assign_worker)(worker)
 
@@ -98,9 +106,9 @@ class NotifyWorkRequestAssigned(
             channel, {"type": "work_request.assigned"}
         )
 
-    def test_internal_worker(self):
+    def test_celery_worker(self):
         """
-        WorkRequest.assign_worker sends no message for internal workers.
+        WorkRequest.assign_worker sends no message for Celery workers.
 
         These are assumed to notify the worker of work to do via some other
         mechanism (e.g. a Celery queue).
@@ -119,29 +127,31 @@ class NotifyWorkRequestCompletedTests(TestHelpersMixin, TestCase):
         status: WorkRequest.Results,
         expected_subject: str,
         *,
-        expected_to: Optional[list[str]] = None,
-        notification_data: Optional[NotificationDataEmail] = None,
-        expected_cc: Optional[list[str]] = None,
+        expected_to: list[str] | None = None,
+        event_name='on_failure',
+        notification_data: NotificationDataEmail | None = None,
+        expected_cc: list[str] | None = None,
     ):
         """Assert WorkRequest completed in failure or error."""
         notification_channel_name = "deblts-email"
 
-        task_data: dict[str, Any] = {
-            "notifications": {
-                "on_failure": [
+        event_reactions = EventReactions.parse_obj(
+            {
+                event_name: [
                     {
+                        "action": "send-notification",
                         "channel": notification_channel_name,
                     }
                 ]
             }
-        }
+        )
         if notification_data is not None:
-            task_data["notifications"]["on_failure"][0][
-                "data"
-            ] = notification_data.dict()
+            getattr(event_reactions, event_name)[
+                0
+            ].data = notification_data.dict()
 
         work_request: WorkRequest = self.create_work_request(
-            status=WorkRequest.Statuses.RUNNING, task_data=task_data
+            status=WorkRequest.Statuses.RUNNING, event_reactions=event_reactions
         )
 
         if expected_to is None:
@@ -179,45 +189,36 @@ class NotifyWorkRequestCompletedTests(TestHelpersMixin, TestCase):
 
         self.assertEqual(sent_mail.cc, expected_cc)
 
-    def test_send_notification_on_success(self) -> None:
-        """No notification emails are sent in case of success."""
-        notification_channel_name = "deblts-email"
+    def test_send_notification_on_success(self):
+        """One notification email is sent."""
         to = ["a@example.com"]
+        cc = ["cc@example.com"]
 
-        task_data = {
-            "notifications": {
-                "on_failure": [
-                    {
-                        "channel": notification_channel_name,
-                        "data": {
-                            "to": to,
-                            "subject": "Bad news: $work_request_id completed "
-                            "in $work_request_result $invalid_variable",
-                        },
-                    }
-                ]
-            }
-        }
-
-        work_request: WorkRequest = self.create_work_request(
-            status=WorkRequest.Statuses.RUNNING, task_data=task_data
+        self.assert_send_notification(
+            WorkRequest.Results.SUCCESS,
+            event_name='on_success',
+            notification_data=NotificationDataEmail(to=to, cc=cc),
+            expected_to=to,
+            expected_cc=cc,
+            expected_subject=(
+                "WorkRequest {work_request_id} " "completed in success"
+            ),
         )
 
-        notification_channel_data = {
-            "from": "sender@debusine.example.org",
-            "to": to,
-        }
+    def test_send_notification_no_to(self):
+        """One notification email is sent."""
+        cc = ["cc@example.com"]
 
-        NotificationChannel.objects.create(
-            name=notification_channel_name,
-            method=NotificationChannel.Methods.EMAIL,
-            data=notification_channel_data,
+        self.assert_send_notification(
+            WorkRequest.Results.SUCCESS,
+            event_name='on_success',
+            notification_data=NotificationDataEmail(cc=cc),
+            expected_cc=cc,
+            expected_subject=(
+                "WorkRequest {work_request_id} " "completed in success"
+            ),
         )
 
-        work_request.mark_completed(WorkRequest.Results.SUCCESS)
-
-        self.assertEqual(len(mail.outbox), 0)
-
     def test_send_notification_on_failure(self):
         """
         One notification email is sent.
@@ -261,33 +262,43 @@ class NotifyWorkRequestCompletedTests(TestHelpersMixin, TestCase):
 
     def test_no_notification_is_sent(self) -> None:
         """No mail notification is sent."""
-        work_request: WorkRequest = self.create_work_request(
-            status=WorkRequest.Statuses.RUNNING,
-            task_data={"notifications": None},
-        )
-        work_request.mark_completed(WorkRequest.Results.FAILURE)
-        self.assertEqual(len(mail.outbox), 0)
+        for result in (
+            WorkRequest.Results.SUCCESS,
+            WorkRequest.Results.FAILURE,
+        ):
+            work_request: WorkRequest = self.create_work_request(
+                status=WorkRequest.Statuses.RUNNING,
+            )
+            work_request.event_reactions = EventReactions()
+            with mock.patch(
+                'debusine.db.models.WorkRequest'
+                '.process_update_collection_with_artifacts'
+            ):
+                work_request.mark_completed(result)
+                self.assertEqual(len(mail.outbox), 0)
 
     def test_log_notification_channel_does_not_exist(self) -> None:
         """Log contains debug error that notification channel does not exist."""
         channel_name = "does-not-exist"
-        task_data = {
-            "notifications": {
-                "on_failure": [
-                    {
-                        "channel": channel_name,
-                        "data": {"to": ["a@example.com"]},
-                    }
-                ]
-            }
-        }
+        event_reactions = EventReactions(
+            on_failure=[
+                ActionSendNotification(
+                    channel=channel_name,
+                    data=NotificationDataEmail(
+                        to=[pydantic.EmailStr("a@example.com")]
+                    ),
+                )
+            ]
+        )
         work_request: WorkRequest = self.create_work_request(
-            status=WorkRequest.Statuses.RUNNING, task_data=task_data
+            status=WorkRequest.Statuses.RUNNING,
+            event_reactions_json=event_reactions.dict(),
         )
 
         expected_log = (
-            f"WorkRequest {work_request.id}: on_failure notification cannot be "
-            f"sent: NotificationChannel {channel_name} does not exist"
+            f"WorkRequest {work_request.id}: work-request-completed "
+            "notification cannot be sent: "
+            f"NotificationChannel {channel_name} does not exist"
         )
         with self.assertLogsContains(
             expected_log, logger="debusine", level=logging.DEBUG
@@ -305,7 +316,8 @@ class NotifyWorkRequestCompletedTests(TestHelpersMixin, TestCase):
                 "to": ["rcpt@example.com"],
             },
         )
-        work_request_notification = TaskNotification(channel="test")
+
+        work_request_notification = ActionSendNotification(channel="test")
         body = "Not used"
 
         email_send_patch = mock.patch.object(
