@@ -11,15 +11,28 @@
 
 from django.urls import path
 
-from debusine.server.views import (
+from debusine.server.views.artifacts import (
     ArtifactRelationsView,
     ArtifactView,
+    UploadFileView,
+)
+from debusine.server.views.lookups import (
+    LookupMultipleView,
+    LookupSingleView,
+)
+from debusine.server.views.work_requests import (
+    WorkRequestRetryView,
+    WorkRequestView,
+)
+from debusine.server.views.workers import (
     GetNextWorkRequestView,
     RegisterView,
     UpdateWorkRequestAsCompletedView,
     UpdateWorkerDynamicMetadataView,
-    UploadFileView,
-    WorkRequestView,
+)
+from debusine.server.views.workflows import (
+    WorkflowTemplateView,
+    WorkflowView,
 )
 
 app_name = 'server'
@@ -46,6 +59,11 @@ urlpatterns = [
         UpdateWorkRequestAsCompletedView.as_view(),
         name='work-request-completed',
     ),
+    path(
+        '1.0/work-request/<int:work_request_id>/retry/',
+        WorkRequestRetryView.as_view(),
+        name='work-requests-retry',
+    ),
     path(
         "1.0/artifact/",
         ArtifactView.as_view(),
@@ -61,6 +79,14 @@ urlpatterns = [
         UploadFileView.as_view(),
         name="upload-file",
     ),
+    path(
+        "1.0/lookup/single/", LookupSingleView.as_view(), name="lookup-single"
+    ),
+    path(
+        "1.0/lookup/multiple/",
+        LookupMultipleView.as_view(),
+        name="lookup-multiple",
+    ),
     path('1.0/worker/register/', RegisterView.as_view(), name='register'),
     path(
         '1.0/worker/dynamic-metadata/',
@@ -77,4 +103,15 @@ urlpatterns = [
         ArtifactRelationsView.as_view(),
         name="artifact-relation-detail",
     ),
+    path(
+        "1.0/workflow-template/",
+        WorkflowTemplateView.as_view(),
+        name="workflow-templates",
+    ),
+    path(
+        "1.0/workflow-template/<int:pk>/",
+        WorkflowTemplateView.as_view(),
+        name="workflow-template-detail",
+    ),
+    path("1.0/workflow/", WorkflowView.as_view(), name="workflows"),
 ]
