@@ -14,10 +14,11 @@ Provides containment for tasks using a variety of backends.
 """
 
 from debusine.tasks.executors.base import (  # noqa: F401
+    ExecutorImageCategory,
     ExecutorInterface,
     InstanceInterface,
     analyze_worker_all_executors,
-    executor,
+    executor_class,
 )
 from debusine.tasks.executors.incus import (  # noqa: F401
     IncusInstance,
