@@ -10,8 +10,7 @@
 """Unit tests for the Executor basic interface."""
 from pathlib import Path, PurePath
 from subprocess import CalledProcessError, CompletedProcess, PIPE
-from typing import Any, AnyStr, Literal, Union
-from unittest import TestCase
+from typing import Any, AnyStr, Literal
 from unittest.mock import Mock, call, patch
 
 from debusine.tasks.executors.base import (
@@ -21,12 +20,14 @@ from debusine.tasks.executors.base import (
     _backends,
     analyze_worker_all_executors,
 )
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
 class TestInstance(InstanceInterface):
     """Instance used for testing."""
 
+    __test__ = False
+
     started: bool = False
 
     def is_started(self) -> bool:
@@ -58,7 +59,7 @@ class TestInstance(InstanceInterface):
     def do_run(
         self,
         args: list[str],
-        text: Union[Literal[True], Literal[False], None] = None,
+        text: Literal[True] | Literal[False] | None = None,
         run_as_root: bool = False,
         **kwargs: Any,
     ) -> CompletedProcess[AnyStr]:
@@ -66,7 +67,7 @@ class TestInstance(InstanceInterface):
         raise NotImplementedError()
 
 
-class ExecutorTests(TestHelpersMixin, TestCase):
+class ExecutorTests(TestCase):
     """Unit tests for the Executor basic interface."""
 
     def test_backends_populated(self):
@@ -89,7 +90,7 @@ class ExecutorTests(TestHelpersMixin, TestCase):
         self.assertTrue(executors_metadata["executor:unshare:available"])
 
 
-class InstanceTests(TestCase, TestHelpersMixin):
+class InstanceTests(TestCase):
     """Unit tests for the Instance basic interface."""
 
     def setUp(self):
@@ -222,8 +223,11 @@ class InstanceTests(TestCase, TestHelpersMixin):
 
         temp_directory = self.create_temporary_directory()
         (source_directory := temp_directory / "source").mkdir()
+        source_directory.chmod(0o755)
         (sub_directory := source_directory / "sub-directory").mkdir()
-        (sub_directory / "sub-sub-directory").mkdir()
+        sub_directory.chmod(0o755)
+        (sub_sub_directory := sub_directory / "sub-sub-directory").mkdir()
+        sub_sub_directory.chmod(0o755)
 
         (file1 := source_directory / "file1.txt").touch()
         file1.chmod(0o640)
