@@ -14,13 +14,17 @@ from datetime import datetime, timezone
 from pathlib import Path
 from shutil import rmtree
 from tempfile import mkdtemp
-from typing import Optional
 from unittest import TestCase
 from unittest.mock import MagicMock, create_autospec, patch
 
+from debusine.artifacts.models import ArtifactCategory
 from debusine.client.debusine import Debusine
 from debusine.tasks import Noop
-from debusine.tasks.executors.base import ExecutorInterface, _backends
+from debusine.tasks.executors.base import (
+    ExecutorImageCategory,
+    ExecutorInterface,
+    _backends,
+)
 from debusine.tasks.executors.images import (
     CACHE_SIZE,
     ImageCache,
@@ -30,18 +34,22 @@ from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
 from debusine.test import TestHelpersMixin
 
 
-class ImageCacheMetadataTests(ExternalTaskHelperMixin[Noop], TestCase):
-    """Unit tests for ImageCache class that don't work with images."""
+class ImageCacheTestMixin(ExternalTaskHelperMixin[Noop]):
+    """Common setup for unit tests of ImageCache class."""
 
     def setUp(self):
         """Mock the Debusine API for tests."""
         self.debusine_api = create_autospec(Debusine)
         self.image_cache = ImageCache(
-            self.debusine_api, "debian:system-tarball"
+            self.debusine_api, ExecutorImageCategory.TARBALL
         )
         self.artifact = self.fake_system_tarball_artifact()
         self.debusine_api.artifact_get.return_value = self.artifact
 
+
+class ImageCacheMetadataTests(ImageCacheTestMixin, TestCase):
+    """Unit tests for ImageCache class that don't work with images."""
+
     def test_cache_artifact_path(self):
         """Test that cache_artifact_path builds the correct path."""
         self.assertEqual(
@@ -73,17 +81,21 @@ class ImageCacheMetadataTests(ExternalTaskHelperMixin[Noop], TestCase):
         """Test that image_metadata fetches the Artifact."""
         self.assertEqual(self.image_cache.image_artifact(42), self.artifact)
 
+        self.debusine_api.artifact_get.assert_called_once_with(42)
+
     def test_image_artifact_incorrect_category(self):
         """Test that image_artifact rejects the wrong kind of artifact."""
-        self.artifact.category = "debian:system-image"
+        self.artifact.category = ArtifactCategory.SYSTEM_IMAGE
 
         with self.assertRaisesRegex(
-            ValueError, r"^Unexpected artifact type: debian:system-image$"
+            ValueError,
+            r"^Unexpected artifact type debian:system-image; expected "
+            r"debian:system-tarball$",
         ):
             self.image_cache.image_artifact(42)
 
 
-class ImageCacheTests(TestHelpersMixin, ImageCacheMetadataTests):
+class ImageCacheTests(TestHelpersMixin, ImageCacheTestMixin, TestCase):
     """Unit tests for ImageCache class that work with images."""
 
     def setUp(self):
@@ -292,8 +304,8 @@ class ImageCacheTests(TestHelpersMixin, ImageCacheMetadataTests):
     def write_image_usage_log(
         self,
         artifact_id: int,
-        timestamp: Optional[datetime] = None,
-        backends: Optional[set[str]] = None,
+        timestamp: datetime | None = None,
+        backends: set[str] | None = None,
     ):
         """Create an image usage log for artifact_id, with timestamp."""
         if timestamp is None:
