@@ -15,19 +15,19 @@ from pathlib import Path
 from shutil import rmtree
 from subprocess import CalledProcessError, CompletedProcess, DEVNULL, PIPE
 from tempfile import NamedTemporaryFile, mkdtemp
-from typing import Optional, cast
-from unittest import TestCase
+from typing import cast
 from unittest.mock import ANY, MagicMock, call, create_autospec, patch
 
 import yaml
 
+from debusine.artifacts.models import ArtifactCategory
 from debusine.client.debusine import Debusine
 from debusine.tasks import Noop
 from debusine.tasks.executors.base import (
     ExecutorInterface,
     ImageImportError,
     InstanceNotRunning,
-    executor,
+    executor_class,
 )
 from debusine.tasks.executors.incus import (
     IncusImageCache,
@@ -39,7 +39,7 @@ from debusine.tasks.executors.incus import (
     run_incus_cmd,
 )
 from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 
 
 class RunIncusCmdTest(TestCase):
@@ -73,7 +73,7 @@ class IncusImageCacheTestMixin(ExternalTaskHelperMixin[Noop]):
 
     def create_tar(
         self,
-        path: Optional[Path] = None,
+        path: Path | None = None,
         dirs: tuple[str, ...] = ("./", "./sbin"),
         files: tuple[str, ...] = ("./test-file", "./sbin/init"),
         special_files: bool = False,
@@ -146,7 +146,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
         super().setUp()
         self.debusine_api = create_autospec(Debusine)
         self.image_cache = IncusImageCache(
-            self.debusine_api, "debian:system-tarball"
+            self.debusine_api, ArtifactCategory.SYSTEM_TARBALL
         )
         self.artifact = self.fake_system_tarball_artifact()
         self.image_cache.image_cache_path = Path("/tmp/not-used")
@@ -288,9 +288,7 @@ class IncusImageCacheTests(IncusImageCacheTestMixin, TestCase):
             self.assertEqual(member.linkname, "bar")
 
 
-class IncusLXCImageCacheTests(
-    IncusImageCacheTestMixin, TestHelpersMixin, TestCase
-):
+class IncusLXCImageCacheTests(IncusImageCacheTestMixin, TestCase):
     """Unit tests for IncusLXCImageCache."""
 
     def setUp(self):
@@ -298,7 +296,7 @@ class IncusLXCImageCacheTests(
         super().setUp()
         self.debusine_api = create_autospec(Debusine)
         self.image_cache = IncusLXCImageCache(
-            self.debusine_api, "debian:system-tarball"
+            self.debusine_api, ArtifactCategory.SYSTEM_TARBALL
         )
         self.artifact = self.fake_system_tarball_artifact()
         self.image_cache.image_cache_path = Path("/tmp/not-used")
@@ -408,7 +406,7 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
         super().setUp()
         self.debusine_api = create_autospec(Debusine)
         self.image_cache = IncusVMImageCache(
-            self.debusine_api, "debian:system-tarball"
+            self.debusine_api, ArtifactCategory.SYSTEM_TARBALL
         )
         self.artifact = self.fake_system_image_artifact()
         self.image_cache.image_cache_path = Path("/tmp/not-used")
@@ -512,7 +510,7 @@ class IncusVMImageCacheTests(IncusImageCacheTestMixin, TestCase):
         )
 
 
-class IncusExecutorCommon(TestHelpersMixin, ExternalTaskHelperMixin[Noop]):
+class IncusExecutorCommon(ExternalTaskHelperMixin[Noop]):
     """Unit tests that apply to both Incus Executors."""
 
     executor_cls: type[ExecutorInterface]
@@ -646,10 +644,10 @@ class IncusLXCExecutorTests(IncusExecutorCommon, TestCase):
         """Test that the backend_name attribute was set."""
         self.assertEqual(self.executor_cls.backend_name, "incus-lxc")
 
-    def test_executor_instantiates_incus_lxc(self):
-        """Test that executor() supports incus-lxc."""
-        instance = executor(self.debusine_api, "incus-lxc", 42)
-        self.assertIsInstance(instance, self.executor_cls)
+    def test_executor_class_finds_incus_lxc(self):
+        """Test that executor_class() supports incus-lxc."""
+        instance = executor_class("incus-lxc")
+        self.assertEqual(instance, self.executor_cls)
 
     def test_available_no_driver(self):
         """Test available() returns False if the lxc driver isn't there."""
@@ -691,10 +689,10 @@ class IncusVMExecutorTests(IncusExecutorCommon, TestCase):
         """Test that the backend_name attribute was set."""
         self.assertEqual(self.executor_cls.backend_name, "incus-vm")
 
-    def test_executor_instantiates_incus_vm(self):
-        """Test that executor() supports incus-vm."""
-        instance = executor(self.debusine_api, "incus-vm", 42)
-        self.assertIsInstance(instance, self.executor_cls)
+    def test_executor_class_finds_incus_vm(self):
+        """Test that executor_class() supports incus-vm."""
+        instance = executor_class("incus-vm")
+        self.assertEqual(instance, self.executor_cls)
 
     def test_available_no_driver(self):
         """Test available() returns False if the qemu driver isn't there."""
@@ -721,7 +719,7 @@ class IncusVMExecutorTests(IncusExecutorCommon, TestCase):
         )
 
 
-class IncusInstanceTests(TestHelpersMixin, TestCase):
+class IncusInstanceTests(TestCase):
     """Tests for IncusInstance class."""
 
     def setUp(self):
