@@ -21,7 +21,7 @@ from debusine.client.debusine import Debusine
 from debusine.tasks import Noop
 from debusine.tasks.executors.base import (
     ImageNotDownloadedError,
-    executor,
+    executor_class,
 )
 from debusine.tasks.executors.unshare import (
     InstanceNotRunning,
@@ -105,10 +105,10 @@ class UnshareExecutorTests(
             ImageNotDownloadedError, self.executor.autopkgtest_virt_args
         )
 
-    def test_executor_instantiates_unshare(self):
-        """Test that executor() supports unshare."""
-        instance = executor(self.debusine_api, "unshare", 42)
-        self.assertIsInstance(instance, UnshareExecutor)
+    def test_executor_class_finds_unshare(self):
+        """Test that executor_class() supports unshare."""
+        instance = executor_class("unshare")
+        self.assertEqual(instance, UnshareExecutor)
 
     def test_create(self):
         """Test create() return UnshareInstance instance."""
