@@ -14,12 +14,13 @@ import subprocess
 import tempfile
 from pathlib import Path, PurePath
 from subprocess import CompletedProcess
-from typing import Any, AnyStr, Literal, Optional, Union, overload
+from typing import Any, AnyStr, Literal, overload
 
 from debusine import utils
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse
 from debusine.tasks.executors.base import (
+    ExecutorImageCategory,
     ExecutorInterface,
     ImageNotDownloadedError,
     InstanceInterface,
@@ -33,21 +34,19 @@ class UnshareExecutor(ExecutorInterface, backend_name="unshare"):
 
     system_image: ArtifactResponse
     _image_cache: ImageCache
-    _local_path: Optional[Path] = None
+    _local_path: Path | None = None
 
-    def __init__(
-        self,
-        debusine_api: Debusine,
-        system_image_id: int,
-    ):
+    image_category = ExecutorImageCategory.TARBALL
+
+    def __init__(self, debusine_api: Debusine, system_image_id: int):
         """
         Instantiate an UnshareExecutor.
 
         :param debusine_api: The object to use the debusine client API.
         :param system_image_id: An artifact id pointing to the system tarball.
         """
-        self._image_cache = ImageCache(debusine_api, "debian:system-tarball")
-        self._extracted: Optional[Path] = None
+        self._image_cache = ImageCache(debusine_api, self.image_category)
+        self._extracted: Path | None = None
         self.system_image = self._image_cache.image_artifact(system_image_id)
 
     @classmethod
@@ -115,7 +114,7 @@ class UnshareInstance(InstanceInterface):
         """Initialize the object."""
         super().__init__()
         self._image = image
-        self._extracted: Optional[Path] = None
+        self._extracted: Path | None = None
 
     @property
     def _root(self) -> Path:
@@ -266,7 +265,7 @@ class UnshareInstance(InstanceInterface):
     def do_run(
         self,
         args: list[str],
-        text: Union[Literal[False], None] = None,
+        text: Literal[False] | None = None,
         run_as_root: bool = False,
         **kwargs,
     ) -> CompletedProcess[bytes]: ...
@@ -274,7 +273,7 @@ class UnshareInstance(InstanceInterface):
     def do_run(
         self,
         args: list[str],
-        text: Union[Literal[False], Literal[True], None] = None,
+        text: Literal[False] | Literal[True] | None = None,
         run_as_root: bool = False,
         **kwargs,
     ) -> CompletedProcess[AnyStr]:
@@ -318,7 +317,7 @@ class UnshareInstance(InstanceInterface):
         cls,
         args: list[str],
         *,
-        stdin_path: Optional[Path] = None,
+        stdin_path: Path | None = None,
         text: Literal[True],
         **kwargs,
     ) -> CompletedProcess[str]: ...
@@ -329,8 +328,8 @@ class UnshareInstance(InstanceInterface):
         cls,
         args: list[str],
         *,
-        stdin_path: Optional[Path] = None,
-        text: Union[Literal[False], None] = None,
+        stdin_path: Path | None = None,
+        text: Literal[False] | None = None,
         **kwargs,
     ) -> CompletedProcess[bytes]: ...
 
