@@ -11,24 +11,33 @@
 import json
 import re
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 
 from debian import debfile
 
 from debusine import utils
 from debusine.artifacts import LintianArtifact
+from debusine.artifacts.models import CollectionCategory, DebianLintianSummary
 from debusine.tasks import (
     BaseTaskWithExecutor,
     RunCommandTask,
 )
-from debusine.tasks.models import LintianData, LintianPackageType
+from debusine.tasks.models import (
+    LintianData,
+    LintianDynamicData,
+    LintianPackageType,
+)
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class LintianVersionParseError(Exception):
     """The output of "lintian --version" cannot be parsed."""
 
 
-class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
+class Lintian(
+    RunCommandTask[LintianData, LintianDynamicData],
+    BaseTaskWithExecutor[LintianData, LintianDynamicData],
+):
     """Task to use lintian in debusine."""
 
     TASK_VERSION = 1
@@ -50,13 +59,17 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
 
     CAPTURE_OUTPUT_FILENAME = "lintian.txt"
 
-    def __init__(self, task_data: dict[str, Any]) -> None:
+    def __init__(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> None:
         """Initialize object."""
-        super().__init__(task_data)
+        super().__init__(task_data, dynamic_task_data)
 
         # In the lintian cmd: the target that is tested (local file, .dsc
         # or .changes)
-        self._lintian_targets: Optional[list[Path]] = None
+        self._lintian_targets: list[Path] | None = None
 
         # Package type to package names that are being analysed by
         # this Lintian invocation. In other words: packages belonging
@@ -76,6 +89,32 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
         ):
             self._severities_to_level[severity] = ranking
 
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface
+    ) -> LintianDynamicData:
+        """Resolve artifact lookups for this task."""
+        environment = self.get_environment_lookup()
+        return LintianDynamicData(
+            environment_id=(
+                None
+                if environment is None
+                else task_database.lookup_single_artifact(
+                    environment,
+                    default_category=CollectionCategory.ENVIRONMENTS,
+                )
+            ),
+            input_source_artifact_id=(
+                None
+                if self.data.input.source_artifact is None
+                else task_database.lookup_single_artifact(
+                    self.data.input.source_artifact
+                )
+            ),
+            input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
+                self.data.input.binary_artifacts
+            ),
+        )
+
     @classmethod
     def generate_severity_count_zero(cls) -> dict[str, int]:
         """Return dictionary. Keys: severities. Value: 0."""
@@ -150,7 +189,7 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
         }
 
     @classmethod
-    def _parse_tag_line(cls, line: str) -> Optional[dict[str, str]]:
+    def _parse_tag_line(cls, line: str) -> dict[str, str] | None:
         """
         Return dictionary with the tag information or None.
 
@@ -218,8 +257,8 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
         """
         tags = []
 
-        tag: Optional[dict[str, str]] = None
-        previous_line: Optional[str] = None
+        tag: dict[str, str] | None = None
+        previous_line: str | None = None
 
         in_comment = False
         explanation: str = ""
@@ -313,7 +352,7 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
     def _extract_package_name_type(
         self,
         file: Path,
-    ) -> tuple[Optional[str], Optional[LintianPackageType]]:
+    ) -> tuple[str | None, LintianPackageType | None]:
         """
         Return package name for file.
 
@@ -354,12 +393,14 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
 
     def fetch_input(self, destination: Path) -> bool:
         """Download the required artifacts."""
-        data_input = self.data.input
+        assert self.dynamic_data
 
-        if (artifact_id := data_input.source_artifact_id) is not None:
+        if (
+            artifact_id := self.dynamic_data.input_source_artifact_id
+        ) is not None:
             self.fetch_artifact(artifact_id, destination)
 
-        for artifact_id in data_input.binary_artifacts_ids:
+        for artifact_id in self.dynamic_data.input_binary_artifacts_ids:
             self.fetch_artifact(artifact_id, destination)
 
         return True
@@ -455,13 +496,13 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
             lintian_artifact = LintianArtifact.create(
                 analysis=analysis_file,
                 lintian_output=lintian_file,
-                summary=analysis["summary"],
+                summary=DebianLintianSummary(**analysis["summary"]),
             )
 
             uploaded = self.debusine.upload_artifact(
                 lintian_artifact,
-                workspace=self.workspace,
-                work_request=self.work_request,
+                workspace=self.workspace_name,
+                work_request=self.work_request_id,
             )
 
             for source_artifact_id in self._source_artifacts_ids:
@@ -514,7 +555,7 @@ class Lintian(RunCommandTask[LintianData], BaseTaskWithExecutor[LintianData]):
         return package_type_to_analysis
 
     def task_succeeded(
-        self, returncode: Optional[int], execute_directory: Path  # noqa: U100
+        self, returncode: int | None, execute_directory: Path  # noqa: U100
     ) -> bool:
         """
         Evaluate task output and return success.
