@@ -11,7 +11,7 @@
 import shlex
 import subprocess
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 
 from debusine import utils
 from debusine.artifacts.local_artifact import DebianSystemTarballArtifact
@@ -29,12 +29,16 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
     _VAR_LIB_DPKG = "var_lib_dpkg"
     _TEST_SBIN_INIT_RETURN_CODE_FILE = "test-sbin-init"
 
-    def __init__(self, task_data: dict[str, Any]) -> None:
+    def __init__(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> None:
         """Initialize MmDebstrap."""
-        super().__init__(task_data)
+        super().__init__(task_data, dynamic_task_data)
 
-        self._host_sources_file: Optional[Path] = None
-        self._chroot_sources_file: Optional[Path] = None
+        self._host_sources_file: Path | None = None
+        self._chroot_sources_file: Path | None = None
 
         # keyrings that will be uploaded into the chroot
         self._upload_keyrings: list[Path] = []
@@ -43,7 +47,7 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
         self._keyrings: list[Path] = []
 
         # customization_script file
-        self._customization_script: Optional[Path] = None
+        self._customization_script: Path | None = None
 
     @classmethod
     def analyze_worker(cls):
@@ -61,7 +65,11 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
             return False
 
         available_key = self.prefix_with_task_name("available")
-        return worker_metadata.get(available_key, False)
+        if not worker_metadata.get(available_key, False):
+            return False
+
+        request_arch = self.data.bootstrap_options.architecture
+        return request_arch in worker_metadata.get("system:architectures", [])
 
     def _cmdline(self) -> list[str]:
         """
@@ -74,6 +82,7 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
             "mmdebstrap",
             "--mode=unshare",
             "--format=tar",
+            f"--architectures={self.data.bootstrap_options.architecture}",
             "--verbose",
             "--hook-dir=/usr/share/mmdebstrap/hooks/maybe-jessie-or-older",
             # Set "find"'s cwd to "$1". Otherwise, find cwd is the
@@ -122,6 +131,15 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
             ]
         )
 
+        # Remove network-related files that mmdebstrap copies from the host
+        # (/etc/resolv.conf symlinks are OK, since those don't break
+        # reproducibility)
+        cmd.append('--customize-hook=rm -f "$1/etc/hostname"')
+        cmd.append(
+            '--customize-hook='
+            'test -h "$1/etc/resolv.conf" || rm -f "$1/etc/resolv.conf"'
+        )
+
         # customization_script
         if script := self._customization_script:
             script_name = script.name
@@ -253,9 +271,19 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
             vendor = self._get_value_os_release(
                 execute_dir / self._OS_RELEASE_FILE, "ID"
             )
-            codename = self._get_value_os_release(
-                execute_dir / self._OS_RELEASE_FILE, "VERSION_CODENAME"
-            )
+
+            main_bootstrap_repository = self.data.bootstrap_repositories[0]
+            try:
+                codename = self._get_value_os_release(
+                    execute_dir / self._OS_RELEASE_FILE, "VERSION_CODENAME"
+                )
+            except KeyError:
+                # jessie doesn't provide VERSION_CODENAME
+                codename = main_bootstrap_repository.suite
+
+            # /etc/os-release reports the testing release for unstable (#341)
+            if main_bootstrap_repository.suite in ("unstable", "sid"):
+                codename = "sid"
 
             pkglist = self._get_pkglist(execute_dir / self._VAR_LIB_DPKG)
             with_init = self._get_with_init(
@@ -277,14 +305,14 @@ class MmDebstrap(SystemBootstrap[MmDebstrapData]):
                     # TODO / XXX: is "mirror" meant to be the first repository?
                     # or "mirrors" and list all of them?
                     # Or we could duplicate all the bootstrap_repositories...
-                    "mirror": self.data.bootstrap_repositories[0].mirror,
+                    "mirror": main_bootstrap_repository.mirror,
                 },
             )
 
             self.debusine.upload_artifact(
                 artifact,
-                workspace=self.workspace,
-                work_request=self.work_request,
+                workspace=self.workspace_name,
+                work_request=self.work_request_id,
             )
 
     @staticmethod
