@@ -9,11 +9,13 @@
 
 """no operation task. Waits and returns. Used in the integration tests."""
 
+from pathlib import Path
+
 from debusine.tasks import BaseExternalTask
-from debusine.tasks.models import NoopData
+from debusine.tasks.models import BaseDynamicTaskData, NoopData
 
 
-class Noop(BaseExternalTask[NoopData]):
+class Noop(BaseExternalTask[NoopData, BaseDynamicTaskData]):
     """
     Task that returns a boolean (execute() depending on the result field).
 
@@ -22,6 +24,6 @@ class Noop(BaseExternalTask[NoopData]):
 
     TASK_VERSION = 1
 
-    def execute(self) -> bool:
+    def run(self, execute_directory: Path) -> bool:  # noqa: U100
         """Return self.data.result (was sent by the client)."""
         return self.data.result
