@@ -16,19 +16,21 @@ https://freexian-team.pages.debian.net/debusine/reference/tasks.html#piuparts-ta
 import tarfile
 from io import BytesIO
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 
 import debusine.utils
+from debusine.artifacts.models import CollectionCategory
 from debusine.tasks import (
     BaseTaskWithExecutor,
     RunCommandTask,
 )
-from debusine.tasks.models import PiupartsData
+from debusine.tasks.models import PiupartsData, PiupartsDynamicData
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class Piuparts(
-    RunCommandTask[PiupartsData],
-    BaseTaskWithExecutor[PiupartsData],
+    RunCommandTask[PiupartsData, PiupartsDynamicData],
+    BaseTaskWithExecutor[PiupartsData, PiupartsDynamicData],
 ):
     """Test Debian binary packages using piuparts."""
 
@@ -36,24 +38,49 @@ class Piuparts(
 
     _piuparts_tmpdir = "/var/cache/piuparts/tmp"
 
-    def __init__(self, task_data: dict[str, Any]) -> None:
+    def __init__(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> None:
         """Initialize (constructor)."""
-        super().__init__(task_data)
+        super().__init__(task_data, dynamic_task_data)
         # *.deb paths. Set by self.configure_for_execution()
         self._deb_files: list[Path] = []
 
         # --basetgz=... option for piuparts cmdline.
         # Set by self.configure_for_execution()
-        self._base_tar: Optional[Path] = None
+        self._base_tar: Path | None = None
 
         # --distribution... option for piuparts cmdline.
         # Set by self.configure_for_execution()
-        self._distribution_codename: Optional[str] = None
+        self._distribution_codename: str | None = None
+
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface
+    ) -> PiupartsDynamicData:
+        """Resolve artifact lookups for this task."""
+        environment = self.get_environment_lookup()
+        assert environment is not None
+        return PiupartsDynamicData(
+            environment_id=task_database.lookup_single_artifact(
+                environment, default_category=CollectionCategory.ENVIRONMENTS
+            ),
+            input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
+                self.data.input.binary_artifacts
+            ),
+            base_tgz_id=task_database.lookup_single_artifact(
+                self.data.base_tgz,
+                default_category=CollectionCategory.ENVIRONMENTS,
+            ),
+        )
 
     def fetch_input(self, destination: Path) -> bool:
         # FetchBuildUploadMixin method
         """Populate work directory with user-specified binary artifact(s)."""
-        for artifact_id in self.data.input.binary_artifacts_ids:
+        assert self.dynamic_data
+
+        for artifact_id in self.dynamic_data.input_binary_artifacts_ids:
             self.fetch_artifact(artifact_id, destination)
         return True
 
@@ -76,8 +103,8 @@ class Piuparts(
             download_directory, [".deb"]
         )
         # Ensure we've got >=1 .deb file(s)
-        # Note: make BaseTask.find_file_by_suffix() more generic and factor
-        # code
+        # Note: This could be moved into a more generic function shared across
+        # tasks.
         if len(self._deb_files) == 0:
             list_of_files = sorted(map(str, download_directory.iterdir()))
             self.append_to_log_file(
@@ -121,11 +148,12 @@ class Piuparts(
         """
         if self.executor_instance is None:
             raise AssertionError("self.executor_instance cannot be None")
+        assert self.dynamic_data
 
         (base_tar_dir := download_directory / "base_tar").mkdir()
 
         base_tgz_response = self.fetch_artifact(
-            self.data.base_tgz_id, base_tar_dir
+            self.dynamic_data.base_tgz_id, base_tar_dir
         )
 
         self._distribution_codename = base_tgz_response.data["codename"]
