@@ -14,12 +14,20 @@ This task implements the PackageBuild generic task for its task_data:
 https://freexian-team.pages.debian.net/debusine/reference/tasks.html#task-packagebuild
 """
 
+import email.utils
 import subprocess
 from pathlib import Path
-from typing import Any, Optional, Union, cast
+from typing import Any, cast
+
+try:
+    from pydantic.v1 import ValidationError
+except ImportError:
+    from pydantic import ValidationError  # type: ignore
 
 import debian.deb822 as deb822
 
+import yaml
+
 import debusine.utils
 from debusine.artifacts import (
     BinaryPackage,
@@ -27,39 +35,74 @@ from debusine.artifacts import (
     PackageBuildLog,
     Upload,
 )
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DoseDistCheck,
+)
 from debusine.client.models import RemoteArtifact
 from debusine.tasks import (
     BaseTaskWithExecutor,
     RunCommandTask,
     TaskConfigError,
 )
-from debusine.tasks.models import SbuildData
+from debusine.tasks.models import SbuildData, SbuildDynamicData
 from debusine.tasks.sbuild_validator_mixin import SbuildValidatorMixin
+from debusine.tasks.server import TaskDatabaseInterface
 from debusine.utils import read_dsc
 
 
 class Sbuild(
     SbuildValidatorMixin,
-    RunCommandTask[SbuildData],
-    BaseTaskWithExecutor[SbuildData],
+    RunCommandTask[SbuildData, SbuildDynamicData],
+    BaseTaskWithExecutor[SbuildData, SbuildDynamicData],
 ):
     """Task implementing a Debian package build with sbuild."""
 
     TASK_VERSION = 1
 
-    def __init__(self, task_data: dict[str, Any]) -> None:
+    def __init__(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> None:
         """Initialize the sbuild task."""
-        super().__init__(task_data)
+        super().__init__(task_data, dynamic_task_data)
         self.chroots = None
         self.builder = "sbuild"
 
         # dsc_file Path. Set by self.configure_for_execution()
-        self._dsc_file: Optional[Path] = None
+        self._dsc_file: Path | None = None
+        self._extra_packages: list[Path] = []
+
+    def compute_dynamic_data(
+        self, task_database: TaskDatabaseInterface
+    ) -> SbuildDynamicData:
+        """Resolve artifact lookups for this task."""
+        environment = self.get_environment_lookup()
+        return SbuildDynamicData(
+            environment_id=(
+                None
+                if environment is None
+                else task_database.lookup_single_artifact(
+                    environment,
+                    default_category=CollectionCategory.ENVIRONMENTS,
+                )
+            ),
+            input_source_artifact_id=task_database.lookup_single_artifact(
+                self.data.input.source_artifact
+            ),
+            input_extra_binary_artifacts_ids=(
+                task_database.lookup_multiple_artifacts(
+                    self.data.input.extra_binary_artifacts
+                )
+            ),
+        )
 
     @property
     def chroot_name(self) -> str:
         """Build name of required chroot."""
-        return "%s-%s" % (
+        return "{}-{}".format(
             self.data.distribution,
             self.data.host_architecture,
         )
@@ -163,9 +206,51 @@ class Sbuild(
 
     def fetch_input(self, destination: Path) -> bool:
         """Download the source artifact."""
-        source_artifact_id = self.data.input.source_artifact_id
+        if not self.debusine:
+            raise AssertionError("self.debusine not set")
+        assert self.dynamic_data
+
+        source_artifact_id = self.dynamic_data.input_source_artifact_id
+        artifact = self.debusine.artifact_get(source_artifact_id)
+        if artifact.category != ArtifactCategory.SOURCE_PACKAGE:
+            self.append_to_log_file(
+                "fetch_input.log",
+                [
+                    f"input.source_artifact points to a "
+                    f"{artifact.category}, not the expected "
+                    f"{ArtifactCategory.SOURCE_PACKAGE}."
+                ],
+            )
+            return False
+        for file in artifact.files:
+            if file.endswith(".dsc"):
+                self._dsc_file = destination / file
+                break
+        else:
+            raise AssertionError("No .dsc file found in source package.")
         self.fetch_artifact(source_artifact_id, destination)
 
+        for artifact_id in self.dynamic_data.input_extra_binary_artifacts_ids:
+            artifact = self.debusine.artifact_get(artifact_id)
+            if artifact.category in (
+                ArtifactCategory.BINARY_PACKAGE,
+                ArtifactCategory.BINARY_PACKAGES,
+            ):
+                for file in artifact.files:
+                    self._extra_packages.append(destination / file)
+            else:
+                self.append_to_log_file(
+                    "fetch_input.log",
+                    [
+                        f"input.extra_binary_artifacts includes artifact "
+                        f"{artifact_id}, a {artifact.category}, not the "
+                        f"expected {ArtifactCategory.BINARY_PACKAGE} or "
+                        f"{ArtifactCategory.BINARY_PACKAGES}."
+                    ],
+                )
+                return False
+            self.fetch_artifact(artifact_id, destination)
+
         return True
 
     def _cmdline(self) -> list[str]:
@@ -177,6 +262,7 @@ class Sbuild(
         cmd = [
             self.builder,
             "--no-clean",
+            "--purge-deps=never",
         ]
         if "any" in self.data.build_components:
             cmd.append("--arch-any")
@@ -207,6 +293,9 @@ class Sbuild(
                 cmd += [
                     "--chroot-mode=unshare",
                     f"--chroot={self.executor.image_name()}",
+                    # Remove any dangling resolv.conf symlink (from
+                    # systemd-resolved installed in the environment, #1071736)
+                    "--chroot-setup-commands=rm -f /etc/resolv.conf",
                 ]
             else:
                 virt_server = self.executor.autopkgtest_virt_server()
@@ -218,7 +307,25 @@ class Sbuild(
                     for opt in self.executor.autopkgtest_virt_args()
                 ]
 
-        cmd.extend(self.data.sbuild_options)
+        # binNMU
+        if self.data.binnmu is not None:
+            cmd.append(f"--make-binNMU={self.data.binnmu.changelog}")
+            cmd.append(f"--append-to-version={self.data.binnmu.suffix}")
+            cmd.append("--binNMU=0")
+            if self.data.binnmu.timestamp is not None:
+                rfc5322date = email.utils.format_datetime(
+                    self.data.binnmu.timestamp
+                )
+                cmd.append(f"--binNMU-timestamp={rfc5322date}")
+            if self.data.binnmu.maintainer is not None:
+                cmd.append(f"--maintainer={self.data.binnmu.maintainer}")
+
+        # BD-Uninstallable
+        cmd.append("--bd-uninstallable-explainer=dose3")
+
+        for package in self._extra_packages:
+            cmd.append(f"--extra-package={package}")
+
         cmd.append(str(self._dsc_file))
 
         return cmd
@@ -237,9 +344,43 @@ class Sbuild(
 
         return super().execute()
 
+    @staticmethod
+    def _extract_dose3_explanation(
+        build_log_path: Path,
+    ) -> DoseDistCheck | None:
+        """Isolate and parse dose3 output in BD-Uninstallable scenario."""
+        output = ''
+        f = open(build_log_path)
+
+        # find start of dose-debcheck output
+        while (line := f.readline()) != '':
+            if line == '(I)Dose_applications: Solving...\n':
+                break
+
+        # exit if no/invalid output
+        if not (line := f.readline()).startswith('output-version'):
+            return None
+        output += line
+
+        # grab until next section
+        while (line := f.readline()) != '':
+            if line.startswith('+----'):
+                break
+            output += line
+
+        # convert & validate to Pydantic structure
+        # Use yaml.CBaseLoad to work-around malformed yaml
+        # https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=834059#22
+        # https://gitlab.com/irill/dose3/-/issues/18
+        parsed_dict = yaml.load(output, yaml.CBaseLoader)
+        try:
+            return DoseDistCheck.parse_obj(parsed_dict)
+        except ValidationError:
+            return None
+
     def _upload_package_build_log(
         self, build_directory: Path, source: str, version: str
-    ) -> Optional[RemoteArtifact]:
+    ) -> RemoteArtifact | None:
         if not self.debusine:
             raise AssertionError("self.debusine not set")
 
@@ -249,18 +390,25 @@ class Sbuild(
         if build_log_path is None:
             return None
 
+        # BD-Uninstallable: look for the dose3 output
+        explanation = self._extract_dose3_explanation(build_log_path)
+
         package_build_log = PackageBuildLog.create(
-            source=source, version=version, file=build_log_path
+            source=source,
+            version=version,
+            file=build_log_path,
+            bd_uninstallable=explanation,
         )
+
         return self.debusine.upload_artifact(
             package_build_log,
-            workspace=self.workspace,
-            work_request=self.work_request,
+            workspace=self.workspace_name,
+            work_request=self.work_request_id,
         )
 
     def _upload_binary_upload(
         self, build_directory: Path
-    ) -> Optional[RemoteArtifact]:
+    ) -> RemoteArtifact | None:
         if not self.debusine:
             raise AssertionError("self.debusine not set")
 
@@ -275,8 +423,8 @@ class Sbuild(
         )
         return self.debusine.upload_artifact(
             artifact_binary_upload,
-            workspace=self.workspace,
-            work_request=self.work_request,
+            workspace=self.workspace_name,
+            work_request=self.work_request_id,
         )
 
     def _create_binary_package_local_artifacts(
@@ -285,12 +433,12 @@ class Sbuild(
         dsc: deb822.Dsc,
         architecture: str,
         suffixes: list[str],
-    ) -> list[Union[BinaryPackage, BinaryPackages]]:
+    ) -> list[BinaryPackage | BinaryPackages]:
         deb_paths = debusine.utils.find_files_suffixes(
             build_directory, suffixes
         )
 
-        artifacts: list[Union[BinaryPackage, BinaryPackages]] = []
+        artifacts: list[BinaryPackage | BinaryPackages] = []
         for deb_path in deb_paths:
             artifacts.append(BinaryPackage.create(file=deb_path))
         artifacts.append(
@@ -343,8 +491,8 @@ class Sbuild(
                 remote_artifacts.append(
                     self.debusine.upload_artifact(
                         package,
-                        workspace=self.workspace,
-                        work_request=self.work_request,
+                        workspace=self.workspace_name,
+                        work_request=self.work_request_id,
                     )
                 )
 
@@ -352,8 +500,8 @@ class Sbuild(
 
     def _create_remote_binary_packages_relations(
         self,
-        remote_build_log: Optional[RemoteArtifact],
-        remote_binary_upload: Optional[RemoteArtifact],
+        remote_build_log: RemoteArtifact | None,
+        remote_binary_upload: RemoteArtifact | None,
         remote_binary_packages: list[RemoteArtifact],
     ):
         if not self.debusine:
@@ -384,18 +532,19 @@ class Sbuild(
                     "relates-to",
                 )
 
-    def configure_for_execution(self, download_directory: Path) -> bool:
+    def configure_for_execution(
+        self, download_directory: Path  # noqa: U100
+    ) -> bool:
         """
         Configure Task: set variables needed for the build() step.
 
         Return True if configuration worked, False, if there was a problem.
-
-        Note: self.find_file_by_suffix() write to a log file to be uploaded as
-        artifact.
         """
-        self._dsc_file = self.find_file_by_suffix(download_directory, ".dsc")
-
-        if self._dsc_file is None:
+        if self._dsc_file is None or not self._dsc_file.exists():
+            self.append_to_log_file(
+                "configure_for_execution.log",
+                ["Input source package not found."],
+            )
             return False
 
         if self.backend != "schroot":
