@@ -11,9 +11,8 @@
 import importlib.resources
 import os
 import re
-import subprocess
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 
 import yaml
 
@@ -93,11 +92,15 @@ class SimpleSystemImageBuild(SystemImageBuild):
         "riscv64": "riscv64-efi",
     }
 
-    def __init__(self, task_data: dict[str, Any]) -> None:
+    def __init__(
+        self,
+        task_data: dict[str, Any],
+        dynamic_task_data: dict[str, Any] | None = None,
+    ) -> None:
         """Initialize SimpleSystemImageBuild."""
-        super().__init__(task_data)
+        super().__init__(task_data, dynamic_task_data)
 
-        self._debos_recipe: Optional[Path] = None
+        self._debos_recipe: Path | None = None
         if self.data.disk_image.format == DiskImageFormat.QCOW2:
             self.filename = f"{self.data.disk_image.filename}.qcow2"
         else:
@@ -116,15 +119,6 @@ class SimpleSystemImageBuild(SystemImageBuild):
         available_key = cls.prefix_with_task_name("available")
         metadata[available_key] = debos and resolved and (kvm_access or uml)
 
-        arch_key = cls.prefix_with_task_name("architecture")
-        metadata[arch_key] = (
-            subprocess.run(
-                ["dpkg-architecture", "-qDEB_HOST_ARCH"], capture_output=True
-            )
-            .stdout.decode()
-            .strip()
-        )
-
         return metadata
 
     def can_run_on(self, worker_metadata: dict[str, Any]) -> bool:
@@ -133,15 +127,11 @@ class SimpleSystemImageBuild(SystemImageBuild):
             return False
 
         available_key = self.prefix_with_task_name("available")
-        arch_key = self.prefix_with_task_name("architecture")
+        if not worker_metadata.get(available_key, False):
+            return False
+
         request_arch = self.data.bootstrap_options.architecture
-        host_arch = worker_metadata.get(arch_key)
-        # TODO move the architecture comparison to a general place in Debusin
-        return worker_metadata.get(available_key, False) and (
-            request_arch == host_arch
-            or (host_arch == "amd64" and request_arch == "i386")
-            or (host_arch == "arm64" and request_arch in ("armel", "armhf"))
-        )
+        return request_arch in worker_metadata.get("system:architectures", [])
 
     def _cmdline(self) -> list[str]:
         """
@@ -669,6 +659,9 @@ class SimpleSystemImageBuild(SystemImageBuild):
         if codename is None:
             # jessie doesn't provide VERSION_CODENAME
             codename = self.data.bootstrap_repositories[0].suite
+        # /etc/os-release reports the testing release for unstable (#341)
+        if self.data.bootstrap_repositories[0].suite in ("unstable", "sid"):
+            codename = "sid"
 
         system_file = execute_dir / self.filename
         artifact = DebianSystemImageArtifact.create(
@@ -694,6 +687,6 @@ class SimpleSystemImageBuild(SystemImageBuild):
 
         self.debusine.upload_artifact(
             artifact,
-            workspace=self.workspace,
-            work_request=self.work_request,
+            workspace=self.workspace_name,
+            work_request=self.work_request_id,
         )
