@@ -11,10 +11,12 @@
 import abc
 
 from debusine.tasks import RunCommandTask
-from debusine.tasks.models import SystemImageBuildData
+from debusine.tasks.models import BaseDynamicTaskData, SystemImageBuildData
 
 
-class SystemImageBuild(abc.ABC, RunCommandTask[SystemImageBuildData]):
+class SystemImageBuild(
+    abc.ABC, RunCommandTask[SystemImageBuildData, BaseDynamicTaskData]
+):
     """Implement ontology SystemImageBuild."""
 
     TASK_VERSION = 1
