@@ -1,4 +1,4 @@
-# Copyright 2022-2023 The Debusine developers
+# Copyright 2022-2023 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,7 +14,7 @@ from io import BytesIO
 from pathlib import Path
 from shutil import rmtree
 from tempfile import mkdtemp
-from typing import Any, Generic, Optional, Protocol, TypeVar, cast
+from typing import Any, Generic, Protocol, TypeVar, cast
 from unittest import TestCase, mock
 from unittest.mock import MagicMock
 
@@ -23,21 +23,24 @@ try:
 except ImportError:
     import pydantic as pydantic  # type: ignore
 
+from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
     FileResponse,
     FilesResponseType,
-    NonNegativeInt,
-    StrMaxLength255,
 )
 from debusine.tasks import BaseExternalTask, BaseTask
 from debusine.tasks.executors import ExecutorInterface, UnshareInstance
-from debusine.utils import extract_generic_type_argument
+from debusine.tasks.models import LookupMultiple, LookupSingle
+from debusine.tasks.server import TaskDatabaseInterface
+from debusine.utils import extract_generic_type_arguments
 
 
-TaskClass = TypeVar("TaskClass", bound=BaseTask[Any])
-ExternalTaskClass = TypeVar("ExternalTaskClass", bound=BaseExternalTask[Any])
+TaskClass = TypeVar("TaskClass", bound=BaseTask[Any, Any])
+ExternalTaskClass = TypeVar(
+    "ExternalTaskClass", bound=BaseExternalTask[Any, Any]
+)
 
 
 class TaskTestProtocol(Protocol, Generic[TaskClass]):
@@ -48,6 +51,41 @@ class TaskTestProtocol(Protocol, Generic[TaskClass]):
     task: TaskClass
 
 
+class FakeTaskDatabase(TaskDatabaseInterface):
+    """Fake implementation of database interaction in worker tests."""
+
+    def __init__(
+        self,
+        single_lookups: (
+            dict[tuple[LookupSingle, str | None], int] | None
+        ) = None,
+        multiple_lookups: (
+            dict[tuple[LookupMultiple, str | None], list[int]] | None
+        ) = None,
+    ) -> None:
+        """Construct a :py:class:`FakeTaskDatabase`."""
+        self.single_lookups = single_lookups or {}
+        self.multiple_lookups = multiple_lookups or {}
+
+    def lookup_single_artifact(
+        self,
+        lookup: LookupSingle,
+        default_category: CollectionCategory | None = None,  # noqa: U100
+    ) -> int:
+        """Pretend to look up a single artifact."""
+        return self.single_lookups[(lookup, default_category)]
+
+    def lookup_multiple_artifacts(
+        self,
+        lookup: LookupMultiple,
+        default_category: CollectionCategory | None = None,  # noqa: U100
+    ) -> list[int]:
+        """Pretend to look up multiple artifacts."""
+        if not lookup.__root__:
+            return []
+        return self.multiple_lookups[(lookup, default_category)]
+
+
 class TaskHelperMixin(Generic[TaskClass]):
     """Helper mixin for Task tests."""
 
@@ -60,15 +98,13 @@ class TaskHelperMixin(Generic[TaskClass]):
 
         # The task class used for this test suite, computed by introspecting
         # the type argument used to specialize this generic class.
-        cls.task_class = extract_generic_type_argument(
-            cls, TaskHelperMixin, BaseTask
-        )
+        [cls.task_class] = extract_generic_type_arguments(cls, TaskHelperMixin)
 
     def configure_task(
         self: TaskTestProtocol[TaskClass],
-        task_data: Optional[dict[str, Any]] = None,
-        override: Optional[dict[str, Any]] = None,
-        remove: Optional[list[str]] = None,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+        remove: list[str] | None = None,
     ):
         """
         Run self.task.configure(task_data) with different inputs.
@@ -206,9 +242,9 @@ class ExternalTaskHelperMixin(
         return ArtifactResponse(
             id=42,
             workspace="Testing",
-            category="debian:system-tarball",
+            category=ArtifactCategory.SYSTEM_TARBALL,
             created_at=datetime.datetime(
-                2024, 1, 1, 0, 0, 0, tzinfo=datetime.timezone.utc
+                2024, 1, 1, 0, 0, 0, tzinfo=datetime.UTC
             ),
             data={
                 "architecture": "amd64",
@@ -223,12 +259,8 @@ class ExternalTaskHelperMixin(
             files=FilesResponseType(
                 {
                     "system.tar.xz": FileResponse(
-                        size=pydantic.parse_obj_as(NonNegativeInt, 4001),
-                        checksums={
-                            "sha256": pydantic.parse_obj_as(
-                                StrMaxLength255, "abc123"
-                            ),
-                        },
+                        size=4001,
+                        checksums={"sha256": "abc123"},
                         type="file",
                         url=pydantic.parse_obj_as(
                             pydantic.AnyUrl,
@@ -245,9 +277,9 @@ class ExternalTaskHelperMixin(
         return ArtifactResponse(
             id=42,
             workspace="Testing",
-            category="debian:system-image",
+            category=ArtifactCategory.SYSTEM_IMAGE,
             created_at=datetime.datetime(
-                2024, 1, 1, 0, 0, 0, tzinfo=datetime.timezone.utc
+                2024, 1, 1, 0, 0, 0, tzinfo=datetime.UTC
             ),
             data={
                 "architecture": "amd64",
@@ -266,12 +298,8 @@ class ExternalTaskHelperMixin(
             files=FilesResponseType(
                 {
                     "image.qcow2": FileResponse(
-                        size=pydantic.parse_obj_as(NonNegativeInt, 4001),
-                        checksums={
-                            "sha256": pydantic.parse_obj_as(
-                                StrMaxLength255, "abc123"
-                            ),
-                        },
+                        size=4001,
+                        checksums={"sha256": "abc123"},
                         type="file",
                         url=pydantic.parse_obj_as(
                             pydantic.AnyUrl,
@@ -282,3 +310,72 @@ class ExternalTaskHelperMixin(
             ),
             files_to_upload=[],
         )
+
+    def fake_debian_source_package_artifact(self) -> ArtifactResponse:
+        """Create a fake ArtifactResponse for a debian:source-package."""
+        return ArtifactResponse(
+            id=6,
+            workspace="Testing",
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            created_at=datetime.datetime(
+                2024, 1, 1, 0, 0, 0, tzinfo=datetime.UTC
+            ),
+            data={
+                "name": "foo",
+                "version": "1.0-1",
+                "type": "dpkg",
+                "dsc_fields": {},
+            },
+            download_tar_gz_url=pydantic.parse_obj_as(
+                pydantic.AnyUrl, "https://example.com/download-6/"
+            ),
+            files=FilesResponseType(
+                {
+                    "foo_1.0-1.dsc": FileResponse(
+                        size=10,
+                        checksums={"sha256": "abc123"},
+                        type="file",
+                        url=pydantic.parse_obj_as(
+                            pydantic.AnyUrl,
+                            "https://example.com/foo_1.0-1.dsc",
+                        ),
+                    ),
+                }
+            ),
+            files_to_upload=[],
+        )
+
+    def fake_debian_binary_package_artifact(self) -> ArtifactResponse:
+        """Create a fake ArtifactResponse for a debian:binary-package."""
+        return ArtifactResponse(
+            id=7,
+            workspace="Testing",
+            category=ArtifactCategory.BINARY_PACKAGE,
+            created_at=datetime.datetime(
+                2024, 1, 1, 0, 0, 0, tzinfo=datetime.UTC
+            ),
+            data={
+                "srcpkg_name": "foo",
+                "srcpkg_version": "1.0-1",
+                "type": "dpkg",
+                "dsc_fields": {},
+                "dsc_control_files": [],
+            },
+            download_tar_gz_url=pydantic.parse_obj_as(
+                pydantic.AnyUrl, "https://example.com/download-7/"
+            ),
+            files=FilesResponseType(
+                {
+                    "foo_1.0-1_all.deb": FileResponse(
+                        size=10,
+                        checksums={"sha256": "abc123"},
+                        type="file",
+                        url=pydantic.parse_obj_as(
+                            pydantic.AnyUrl,
+                            "https://example.com/foo_1.0-1_all.deb",
+                        ),
+                    ),
+                }
+            ),
+            files_to_upload=[],
+        )
