@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine developers
+# Copyright 2023 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,10 +14,14 @@ import shlex
 import textwrap
 from pathlib import Path
 from unittest import TestCase
-from unittest.mock import MagicMock, call, create_autospec
+from unittest.mock import MagicMock, call, create_autospec, patch
 
 from debusine.artifacts.local_artifact import AutopkgtestArtifact
-from debusine.artifacts.models import DebianAutopkgtest, DebianAutopkgtestResult
+from debusine.artifacts.models import (
+    CollectionCategory,
+    DebianAutopkgtest,
+    DebianAutopkgtestResult,
+)
 from debusine.client.debusine import Debusine
 from debusine.client.models import ArtifactResponse
 from debusine.tasks import TaskConfigError
@@ -25,8 +29,15 @@ from debusine.tasks.autopkgtest import (
     Autopkgtest,
 )
 from debusine.tasks.executors import UnshareExecutor
-from debusine.tasks.models import WorkerType
-from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
+from debusine.tasks.models import (
+    AutopkgtestDynamicData,
+    LookupMultiple,
+    WorkerType,
+)
+from debusine.tasks.tests.helper_mixin import (
+    ExternalTaskHelperMixin,
+    FakeTaskDatabase,
+)
 from debusine.test import TestHelpersMixin
 
 
@@ -37,17 +48,27 @@ class AutopkgtestTests(
 
     SAMPLE_TASK_DATA = {
         "input": {
-            "source_artifact_id": 5,
-            "binary_artifacts_ids": [10, 11],
+            "source_artifact": 5,
+            "binary_artifacts": [10, 11],
         },
         "host_architecture": "amd64",
-        "environment_id": 42,
+        "environment": "debian/match:codename=bookworm",
     }
 
     def setUp(self):
         """Initialize test."""
         self.configure_task()
 
+        # self.task.workspace_name and self.task.work_request_id are set by
+        # the Worker.
+        self.task.workspace_name = "testing"
+        self.task.work_request_id = 170
+        self.task.dynamic_data = AutopkgtestDynamicData(
+            environment_id=1,
+            input_source_artifact_id=5,
+            input_binary_artifacts_ids=[10, 11],
+        )
+
     def tearDown(self):
         """Delete debug log files directory if it exists."""
         if self.task._debug_log_files_directory:
@@ -197,26 +218,26 @@ class AutopkgtestTests(
         """Valid task data do not raise any exception."""
         self.configure_task()
 
-    def test_configure_missing_required_source_artifact_id(self):
-        """Missing required field "source_artifact_id": raise exception."""
+    def test_configure_missing_required_source_artifact(self):
+        """Missing required field "source_artifact": raise exception."""
         task_no_source_artifact = copy.deepcopy(self.SAMPLE_TASK_DATA)
-        del task_no_source_artifact["input"]["source_artifact_id"]
+        del task_no_source_artifact["input"]["source_artifact"]
 
         with self.assertRaises(TaskConfigError):
             self.configure_task(task_data=task_no_source_artifact)
 
-    def test_configure_missing_required_binary_artifacts_ids(self):
-        """Missing required field "binary_artifacts_ids": raise exception."""
+    def test_configure_missing_required_binary_artifacts(self):
+        """Missing required field "binary_artifacts": raise exception."""
         task_no_binary_artifacts = copy.deepcopy(self.SAMPLE_TASK_DATA)
-        del task_no_binary_artifacts["input"]["binary_artifacts_ids"]
+        del task_no_binary_artifacts["input"]["binary_artifacts"]
 
         with self.assertRaises(TaskConfigError):
             self.configure_task(task_data=task_no_binary_artifacts)
 
-    def test_configure_accept_context_artifacts_ids(self):
-        """Optional field "context_artifacts_ids": do not raise exception."""
+    def test_configure_accept_context_artifacts(self):
+        """Optional field "context_artifacts": do not raise exception."""
         task_context_artifacts_ids = copy.deepcopy(self.SAMPLE_TASK_DATA)
-        task_context_artifacts_ids["input"]["context_artifacts_ids"] = [20, 21]
+        task_context_artifacts_ids["input"]["context_artifacts"] = [20, 21]
 
         self.configure_task(task_data=task_context_artifacts_ids)
 
@@ -238,10 +259,10 @@ class AutopkgtestTests(
         with self.assertRaises(TaskConfigError):
             self.configure_task(remove=["host_architecture"])
 
-    def test_configure_environment_id_is_required(self):
-        """Missing required field "environment_id": raise exception."""
+    def test_configure_environment_is_required(self):
+        """Missing required field "environment": raise exception."""
         with self.assertRaises(TaskConfigError):
-            self.configure_task(remove=["environment_id"])
+            self.configure_task(remove=["environment"])
 
     def test_configure_backend_choices(self):
         """Field "backend" accept correct values."""
@@ -314,10 +335,10 @@ class AutopkgtestTests(
         self.configure_task()
         self.assertEqual(self.task.data.needs_internet, "run")
 
-    def test_configure_environment(self):
-        """Field "environment" is accepted."""
+    def test_configure_extra_environment(self):
+        """Field "extra_environment" is accepted."""
         self.configure_task(
-            override={"environment": {"var1": "foo1", "var2": "foo2"}}
+            override={"extra_environment": {"var1": "foo1", "var2": "foo2"}}
         )
 
     def test_configure_fail_on_defaults(self):
@@ -452,6 +473,71 @@ class AutopkgtestTests(
             )
         )
 
+    def test_compute_dynamic_data(self) -> None:
+        """Dynamic data receives relevant artifact IDs."""
+        binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): 1,
+                # input.source_artifact
+                (5, None): 5,
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [10, 11]
+            },
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            AutopkgtestDynamicData(
+                environment_id=1,
+                input_source_artifact_id=5,
+                input_binary_artifacts_ids=[10, 11],
+                input_context_artifacts_ids=[],
+            ),
+        )
+
+    def test_compute_dynamic_data_context(self) -> None:
+        """Dynamic data includes input context artifact IDs if relevant."""
+        binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
+        self.task.data.input.context_artifacts = LookupMultiple.parse_obj(
+            {"collection": "bookworm@debian:suite"}
+        )
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): 1,
+                # input.source_artifact
+                (5, None): 5,
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (binary_artifacts_lookup, None): [10, 11],
+                # input.context_artifacts
+                (self.task.data.input.context_artifacts, None): [2, 3],
+            },
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            AutopkgtestDynamicData(
+                environment_id=1,
+                input_source_artifact_id=5,
+                input_binary_artifacts_ids=[10, 11],
+                input_context_artifacts_ids=[2, 3],
+            ),
+        )
+
     def test_cmdline_executor_not_set(self):
         """Raise AssertionError: self.executor is not set."""
         msg = (
@@ -539,12 +625,12 @@ class AutopkgtestTests(
     def test_cmdline_environment(self):
         """Command line include "--env=VAR=VALUE" for environment variables."""
         self.mock_executor()
-        environment = {"DEBUG": "1", "LOG": "NONE"}
-        self.task.data.environment = environment
+        extra_environment = {"DEBUG": "1", "LOG": "NONE"}
+        self.task.data.extra_environment = extra_environment
 
         cmdline = self.task._cmdline()
 
-        for var, value in environment.items():
+        for var, value in extra_environment.items():
             self.assertIn(f"--env={var}={value}", cmdline)
 
     def test_cmdline_includes_executor(self):
@@ -612,6 +698,36 @@ class AutopkgtestTests(
                 f"--timeout-{key}", [arg.split("=")[0] for arg in cmdline]
             )
 
+    def test_cmdline_unshare_copy_resolv_conf(self):
+        """In unshare mode, we copy /etc/resolv.conf if it exists."""
+        self.mock_executor()
+        self.task.data.backend = "unshare"
+
+        with patch("pathlib.Path.is_file", return_value=True):
+            cmdline = self.task._cmdline()
+
+        self.assertIn("--copy=/etc/resolv.conf:/etc/resolv.conf", cmdline)
+
+    def test_cmdline_unshare_missing_resolv_conf(self):
+        """In unshare mode, we don't copy /etc/resolv.conf if it's missing."""
+        self.mock_executor()
+        self.task.data.backend = "unshare"
+
+        with patch("pathlib.Path.is_file", return_value=False):
+            cmdline = self.task._cmdline()
+
+        self.assertNotIn("--copy=/etc/resolv.conf:/etc/resolv.conf", cmdline)
+
+    def test_cmdline_not_unshare_does_not_copy_resolv_conf(self):
+        """For non-unshare backends, we don't copy /etc/resolv.conf."""
+        self.mock_executor()
+        self.task.data.backend = "incus-lxc"
+
+        with patch("pathlib.Path.is_file", return_value=True):
+            cmdline = self.task._cmdline()
+
+        self.assertNotIn("--copy=/etc/resolv.conf:/etc/resolv.conf", cmdline)
+
     def test_select_default_backend(self):
         """We select Autopkgtest.DEFAULT_BACKEND if backend is auto."""
         debusine_mock = self.mock_debusine()
@@ -638,8 +754,8 @@ class AutopkgtestTests(
 
         self.assertEqual(self.task.executor.backend_name, "unshare")
 
-    def test_fetch_input_no_context_artifacts_ids(self):
-        """Download "source_artifact_id" and "binary_artifacts_ids"."""
+    def test_fetch_input_no_context_artifacts(self):
+        """Download "source_artifact" and "binary_artifacts"."""
         debusine_mock = self.mock_debusine()
         workspace = "Testing"
         debusine_mock.download_artifact.return_value = ArtifactResponse(
@@ -671,6 +787,8 @@ class AutopkgtestTests(
             },
         }
 
+        data_input = self.task.data.input
+
         debusine_mock.download_artifact.return_value = ArtifactResponse(
             id=10,
             workspace=workspace,
@@ -686,13 +804,9 @@ class AutopkgtestTests(
 
         self.task.fetch_input(destination)
 
-        data_input = self.task.data.input
+        calls = [call(data_input.source_artifact, destination, tarball=False)]
 
-        calls = [
-            call(data_input.source_artifact_id, destination, tarball=False)
-        ]
-
-        for binary_artifact_id in data_input.binary_artifacts_ids:
+        for binary_artifact_id in data_input.binary_artifacts:
             calls.append(call(binary_artifact_id, destination, tarball=False))
 
         debusine_mock.download_artifact.assert_has_calls(calls)
@@ -700,20 +814,20 @@ class AutopkgtestTests(
         self.assertEqual(self.task._source_package_url, source_package_url)
         self.assertEqual(self.task._source_package_path, "hello.dsc")
 
-    def test_fetch_input_context_artifacts_ids(self):
-        """Download the "context_artifacts_ids"."""
+    def test_fetch_input_context_artifacts(self):
+        """Download the "context_artifacts"."""
         debusine_mock = self.mock_debusine()
 
         destination = self.create_temporary_directory()
 
         data_input = self.task.data.input
-
-        data_input.context_artifacts_ids = [50, 51]
+        data_input.context_artifacts = [50, 51]
+        self.task.dynamic_data.input_context_artifacts_ids = [50, 51]
 
         self.task.fetch_input(destination)
 
         calls = []
-        for context_artifact_id in data_input.context_artifacts_ids:
+        for context_artifact_id in data_input.context_artifacts:
             calls.append(call(context_artifact_id, destination, tarball=False))
 
         debusine_mock.download_artifact.assert_has_calls(calls)
@@ -765,7 +879,7 @@ class AutopkgtestTests(
         directory = self.create_temporary_directory()
         (directory / Autopkgtest.ARTIFACT_DIR).mkdir()
         summary_file = directory / Autopkgtest.SUMMARY_FILE
-        summary_file.write_text(f"test_name\t{test_result}")
+        summary_file.write_text(test_result)
 
         self.configure_task(override={"fail_on": fail_on_config})
 
@@ -773,34 +887,75 @@ class AutopkgtestTests(
 
     def test_task_succeeded_failed_test_fail_on_failed(self):
         """Test failed, fail on failed_test=True. Failure."""
-        directory = self.setup_task_succeeded("FAIL", {"failed_test": True})
+        directory = self.setup_task_succeeded(
+            "test_name\tFAIL", {"failed_test": True}
+        )
         self.assertFalse(self.task.task_succeeded(0, directory))
 
     def test_task_succeeded_failed_test_do_not_fail_on_failed(self):
         """Test failed, fail on failed_test=False. Success."""
-        directory = self.setup_task_succeeded("FAIL", {"failed_test": False})
+        directory = self.setup_task_succeeded(
+            "test_name\tFAIL", {"failed_test": False}
+        )
         self.assertTrue(self.task.task_succeeded(0, directory))
 
     def test_task_succeed_flaky_test_fail_on_flaky_failure(self):
         """Test is flaky, fail on flaky_test=True. Failure."""
-        directory = self.setup_task_succeeded("FLAKY", {"flaky_test": True})
+        directory = self.setup_task_succeeded(
+            "test_name\tFLAKY", {"flaky_test": True}
+        )
         self.assertFalse(self.task.task_succeeded(0, directory))
 
     def test_task_succeed_flaky_test_do_not_fail_on_flaky_success(self):
         """Test is flaky, fail on flaky_test=False. Success."""
-        directory = self.setup_task_succeeded("FLAKY", {"flaky_test": False})
+        directory = self.setup_task_succeeded(
+            "test_name\tFLAKY", {"flaky_test": False}
+        )
         self.assertTrue(self.task.task_succeeded(0, directory))
 
     def test_task_succeeded_skipped_test_fail_on_skipped_success(self):
         """Test is skipped, fail on skipped_test=True. Failure."""
-        directory = self.setup_task_succeeded("SKIP", {"skipped_test": True})
+        directory = self.setup_task_succeeded(
+            "test_name\tSKIP", {"skipped_test": True}
+        )
         self.assertFalse(self.task.task_succeeded(0, directory))
 
     def test_task_succeeded_skipped_test_do_not_fail_on_skipped_failure(self):
         """Test is skipped, fail on skipped_test=False. Success."""
-        directory = self.setup_task_succeeded("SKIP", {"skipped_test": False})
+        directory = self.setup_task_succeeded(
+            "test_name\tSKIP", {"skipped_test": False}
+        )
         self.assertTrue(self.task.task_succeeded(0, directory))
 
+    def test_task_succeeded_ignore_return_code(self):
+        """Summary file is inconclusive and ignore return value."""
+        directory = self.setup_task_succeeded(
+            "foo: nothing", {"failed_test": False}
+        )
+        self.assertTrue(self.task.task_succeeded(4, directory))
+
+    def test_task_succeeded_flaky_return_code(self):
+        """Summary file is inconclusive and return value 2."""
+        directory = self.setup_task_succeeded(
+            "foo: nothing", {"flaky_test": True}
+        )
+        self.assertFalse(self.task.task_succeeded(2, directory))
+
+    def test_task_succeeded_skip_return_code(self):
+        """Summary file is inconclusive and return value 8."""
+        directory = self.setup_task_succeeded(
+            "foo: nothing", {"skipped_test": True}
+        )
+        self.assertFalse(self.task.task_succeeded(8, directory))
+
+    def test_task_succeeded_fail_on_return_code(self):
+        """Summary file is inconclusive fail on return value."""
+        directory = self.setup_task_succeeded(
+            "erroneous package: Test dependencies are unsatisfiable.",
+            {"failed_test": True},
+        )
+        self.assertFalse(self.task.task_succeeded(12, directory))
+
     def test_upload_artifacts(self):
         """Upload the AutopkgtestArtifact."""
         exec_dir = self.create_temporary_directory()
@@ -812,12 +967,6 @@ class AutopkgtestTests(
         # Worker would set the server
         self.task.configure_server_access(debusine_mock)
 
-        # self.task.workspace and self.task.work_request are set by the Worker.
-        workspace_name = "testing"
-        self.task.workspace = workspace_name
-        work_request = 170
-        self.task.work_request = work_request
-
         results = {"testname": DebianAutopkgtestResult(status="PASS")}
         self.task._parsed = results
 
@@ -848,8 +997,8 @@ class AutopkgtestTests(
         calls.append(
             call(
                 expected_autopkgtest_artifact,
-                workspace=workspace_name,
-                work_request=work_request,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
             )
         )
 
@@ -902,10 +1051,3 @@ class AutopkgtestTests(
             self.task.check_directory_for_consistency_errors(build_directory),
             [],
         )
-
-    def test_run_cmd_succeeded(self):
-        """run_cmd_succeeded() return value is correct."""
-        self.assertTrue(self.task.run_cmd_succeeded(0))
-        self.assertTrue(self.task.run_cmd_succeeded(2))
-        self.assertFalse(self.task.run_cmd_succeeded(16))
-        self.assertFalse(self.task.run_cmd_succeeded(20))
