@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine developers
+# Copyright 2023 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,13 +24,12 @@ except ImportError:
     import pydantic as pydantic  # type: ignore
 
 from debusine.artifacts import LintianArtifact
+from debusine.artifacts.models import CollectionCategory
 from debusine.client.models import (
     ArtifactResponse,
     FileResponse,
     FilesResponseType,
-    NonNegativeInt,
     RemoteArtifact,
-    StrMaxLength255,
 )
 from debusine.tasks import TaskConfigError
 from debusine.tasks.executors import InstanceInterface
@@ -38,8 +37,11 @@ from debusine.tasks.lintian import (
     Lintian,
     LintianVersionParseError,
 )
-from debusine.tasks.models import LintianPackageType
-from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
+from debusine.tasks.models import LintianDynamicData, LintianPackageType
+from debusine.tasks.tests.helper_mixin import (
+    ExternalTaskHelperMixin,
+    FakeTaskDatabase,
+)
 from debusine.test import TestHelpersMixin
 
 
@@ -50,9 +52,9 @@ class LintianTests(
 
     SAMPLE_TASK_DATA = {
         "input": {
-            "source_artifact_id": 5,
+            "source_artifact": 5,
         },
-        "environment_id": 42,
+        "environment": "debian/match:codename=bookworm:architecture=amd64",
     }
 
     def setUp(self):
@@ -633,12 +635,30 @@ class LintianTests(
 
     def test_configure_valid_task_artifact_id(self):
         """Configuration valid: does not raise any exception."""
-        self.configure_task(override={"input": {"source_artifact_id": 5}})
+        self.configure_task(override={"input": {"source_artifact": 5}})
 
     def test_configure_valid_task_binary_ids(self):
         """Configure valid: does not raise any exception."""
-        self.configure_task(
-            override={"input": {"binary_artifacts_ids": [6, 7]}}
+        self.configure_task(override={"input": {"binary_artifacts": [6, 7]}})
+
+    def test_compute_dynamic_data(self) -> None:
+        """Dynamic data receives relevant artifact IDs."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare",
+                    CollectionCategory.ENVIRONMENTS,
+                ): 1,
+                # input.source_artifact
+                (5, None): 5,
+            }
+        )
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db),
+            LintianDynamicData(environment_id=1, input_source_artifact_id=5),
         )
 
     def test_cmdline_minimum_options(self):
@@ -856,8 +876,8 @@ class LintianTests(
             self.configure_task(
                 override={
                     "input": {
-                        "source_artifact_id": 5,
-                        "binary_artifacts_ids": [],
+                        "source_artifact": 5,
+                        "binary_artifacts": [],
                         "some_additional_property": 87,
                     },
                 }
@@ -890,10 +910,32 @@ class LintianTests(
 
         # Set the analysis that will be uploaded
 
+        summary_data = {
+            "tags_count_by_severity": {"error": 1},
+            "tags_found": ["invalid-file"],
+            "overridden_tags_found": [],
+            "lintian_version": "1.0.0",
+            "distribution": "debian:unstable",
+        }
         package_type_to_analysis = {
-            LintianPackageType.SOURCE: {"summary": {"key": "value"}},
-            LintianPackageType.BINARY_ALL: {"summary": {"key": "value2"}},
-            LintianPackageType.BINARY_ANY: {"summary": {"key": "value3"}},
+            LintianPackageType.SOURCE: {
+                "summary": {
+                    **summary_data,
+                    "package_filename": {"hello": "hello_1.0.dsc"},
+                }
+            },
+            LintianPackageType.BINARY_ALL: {
+                "summary": {
+                    **summary_data,
+                    "package_filename": {"hello-doc": "hello-doc_1.0_all.deb"},
+                }
+            },
+            LintianPackageType.BINARY_ANY: {
+                "summary": {
+                    **summary_data,
+                    "package_filename": {"hello": "hello_1.0_amd64.deb"},
+                }
+            },
         }
 
         create_package_type_to_analysis_patcher = mock.patch.object(
@@ -923,14 +965,14 @@ class LintianTests(
 
         debusine_mock.upload_artifact.side_effect = uploaded_artifacts
 
-        # self.task.workspace is set by the Worker
+        # self.task.workspace_name is set by the Worker
         # and is the workspace that downloads the artifact
         # containing the files needed for Lintian
-        self.task.workspace = workspace_name
+        self.task.workspace_name = workspace_name
 
-        # The worker set self.task.work_request of the task
-        work_request = 147
-        self.task.work_request = work_request
+        # The worker set self.task.work_request_id of the task
+        work_request_id = 147
+        self.task.work_request_id = work_request_id
 
         self.task.upload_artifacts(exec_dir, execution_success=True)
 
@@ -953,7 +995,7 @@ class LintianTests(
                 call(
                     lintian_artifact,
                     workspace=workspace_name,
-                    work_request=work_request,
+                    work_request=work_request_id,
                 )
             )
 
@@ -1100,12 +1142,17 @@ class LintianTests(
         self.assertEqual(actual, expected)
 
     def run_fetch_input(
-        self, input_override: dict[str, Any], expected_artifact_ids: list[int]
+        self,
+        input_override: dict[str, Any],
+        dynamic_data: LintianDynamicData,
+        expected_artifact_ids: list[int],
     ):
         """Verify that Lintian.fetch_input() downloads artifacts."""
         workspace = "System"
 
         self.configure_task(override={"input": input_override})
+        self.task.work_request_id = 147
+        self.task.dynamic_data = dynamic_data
         debusine_mock = self.mock_debusine()
         destination = self.create_temporary_directory()
 
@@ -1118,12 +1165,8 @@ class LintianTests(
 
             file_in_artifact = self.create_temporary_file()
             file_model = FileResponse(
-                size=pydantic.parse_obj_as(
-                    NonNegativeInt, file_in_artifact.stat().st_size
-                ),
-                checksums={
-                    "sha256": pydantic.parse_obj_as(StrMaxLength255, "not-used")
-                },
+                size=file_in_artifact.stat().st_size,
+                checksums={"sha256": "not-used"},
                 type="file",
                 url=pydantic.parse_obj_as(pydantic.AnyUrl, "https://not-used"),
             )
@@ -1160,7 +1203,8 @@ class LintianTests(
         """Lintian.fetch_input() download binary_artifacts_ids."""
         artifacts = [1, 2]
         self.run_fetch_input(
-            input_override={"binary_artifacts_ids": artifacts},
+            input_override={"binary_artifacts": artifacts},
+            dynamic_data=LintianDynamicData(input_binary_artifacts_ids=[1, 2]),
             expected_artifact_ids=artifacts,
         )
 
@@ -1168,7 +1212,10 @@ class LintianTests(
         """Lintian.fetch_input() download source_artifact_id."""
         artifact_id = 5
         self.run_fetch_input(
-            input_override={"source_artifact_id": artifact_id},
+            input_override={"source_artifact": artifact_id},
+            dynamic_data=LintianDynamicData(
+                input_source_artifact_id=artifact_id
+            ),
             expected_artifact_ids=[artifact_id],
         )
 
@@ -1178,9 +1225,13 @@ class LintianTests(
         binary_artifacts_ids = [6, 7]
         self.run_fetch_input(
             input_override={
-                "source_artifact_id": source_artifact_id,
-                "binary_artifacts_ids": binary_artifacts_ids,
+                "source_artifact": source_artifact_id,
+                "binary_artifacts": binary_artifacts_ids,
             },
+            dynamic_data=LintianDynamicData(
+                input_source_artifact_id=source_artifact_id,
+                input_binary_artifacts_ids=binary_artifacts_ids,
+            ),
             expected_artifact_ids=[source_artifact_id] + binary_artifacts_ids,
         )
 
