@@ -10,7 +10,6 @@
 """Unit tests for MmDebstrap class."""
 import copy
 import shlex
-import textwrap
 from pathlib import Path
 from unittest import TestCase, mock
 from unittest.mock import call
@@ -99,6 +98,7 @@ class MmDebstrapTests(
         self.assertTrue(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "mmdebstrap:available": True,
                     "mmdebstrap:version": self.task.TASK_VERSION,
@@ -111,6 +111,7 @@ class MmDebstrapTests(
         self.assertFalse(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "mmdebstrap:available": True,
                     "mmdebstrap:version": self.task.TASK_VERSION + 1,
@@ -123,6 +124,7 @@ class MmDebstrapTests(
         self.assertFalse(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "mmdebstrap:available": False,
                     "mmdebstrap:version": self.task.TASK_VERSION,
@@ -130,6 +132,19 @@ class MmDebstrapTests(
             )
         )
 
+    def test_can_run_on_wrong_architecture(self):
+        """can_run_on returns False if the task is for a different arch."""
+        self.assertFalse(
+            self.task.can_run_on(
+                {
+                    "system:architectures": ["arm64"],
+                    "system:worker_type": WorkerType.EXTERNAL,
+                    "mmdebstrap:available": True,
+                    "mmdebstrap:version": self.task.TASK_VERSION,
+                }
+            )
+        )
+
     def test_cmdline_add_keyrings(self):
         """Command line has the keyring files."""
         self.task._host_sources_file = Path("/somewhere/some-file.sources")
@@ -173,6 +188,7 @@ class MmDebstrapTests(
             "mmdebstrap",
             "--mode=unshare",
             "--format=tar",
+            "--architectures=amd64",
             "--verbose",
             "--hook-dir=/usr/share/mmdebstrap/hooks/maybe-jessie-or-older",
             (
@@ -186,6 +202,9 @@ class MmDebstrapTests(
             '--customize-hook=mkdir "$1/etc/apt/keyrings-debusine"',
             f"--customize-hook=download /etc/os-release {os_release_file}",
             f"--customize-hook=copy-out /var/lib/dpkg {var_lib_dpkg}",
+            '--customize-hook=rm -f "$1/etc/hostname"',
+            '--customize-hook='
+            'test -h "$1/etc/resolv.conf" || rm -f "$1/etc/resolv.conf"',
             '--customize-hook='
             '(test -x "$1/sbin/init" || test -h "$1/sbin/init") ; '
             'echo $? > "$1/test_sbin_init"',
@@ -505,34 +524,33 @@ class MmDebstrapTests(
         )
         self.assertEqual(self.task._customization_script.read_text(), script)
 
-    def write_os_release(self, directory: Path) -> dict[str, str]:
+    def write_os_release(
+        self, directory: Path, codename: str = "bookworm", version: int = 12
+    ) -> dict[str, str]:
         """
         Write in directory / self.task._OS_RELEASE_FILE a release file.
 
         :return: dictionary with keys to values.
         """
-        vendor = "debian"
-        codename = "bookworm"
-        pretty_name = "Debian GNU/Linux 12 (bookworm)"
-        name = "Debian GNU/Linux"
+        data = {
+            "ID": "debian",
+            "NAME": "Debian GNU/Linux",
+            "PRETTY_NAME": f"Debian GNU/Linux {version} ({codename})",
+            "VERSION": f"{version} ({codename})",
+            "VERSION_CODENAME": codename,
+            "VERSION_ID": str(version),
+        }
+
+        if codename == "jessie":
+            del data["VERSION_CODENAME"]
+        if version >= 13:
+            del data["VERSION"]
+            del data["VERSION_ID"]
 
         (directory / self.task._OS_RELEASE_FILE).write_text(
-            textwrap.dedent(
-                f"""\
-                ID={vendor}
-                VERSION_CODENAME={codename}
-                PRETTY_NAME="{pretty_name}"
-                NAME="{name}"
-            """
-            )
+            "\n".join(f"{k}={shlex.quote(v)}" for k, v in data.items()) + "\n"
         )
-
-        return {
-            "ID": vendor,
-            "VERSION_CODENAME": codename,
-            "PRETTY_NAME": pretty_name,
-            "NAME": name,
-        }
+        return data
 
     def test_upload_artifacts(self):
         """Test upload_artifacts()."""
@@ -574,8 +592,108 @@ class MmDebstrapTests(
         calls.append(
             call(
                 expected_system_artifact,
-                workspace=self.task.workspace,
-                work_request=self.task.work_request,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
+            )
+        )
+
+        debusine_mock.upload_artifact.assert_has_calls(calls)
+
+    def test_upload_artifacts_jessie(self):
+        """Test upload_artifacts() for a jessie tarball."""
+        directory = self.create_temporary_directory()
+
+        system_tarball = directory / MmDebstrap._OUTPUT_SYSTEM_FILE
+        system_tarball.write_bytes(b"Generated tarball")
+
+        # Debusine.upload_artifact is mocked to verify the call only
+        debusine_mock = self.mock_debusine()
+
+        os_release_data = self.write_os_release(
+            directory, codename="jessie", version=8
+        )
+        (directory / self.task._TEST_SBIN_INIT_RETURN_CODE_FILE).write_text("0")
+
+        packages = self.patch_subprocess_run_pkglist()
+
+        mirror = self.SAMPLE_TASK_DATA["bootstrap_repositories"][0]["mirror"]
+        self.task.data.bootstrap_repositories[0].suite = "jessie"
+
+        self.task.upload_artifacts(directory, execution_success=True)
+
+        calls = []
+
+        expected_system_artifact = DebianSystemTarballArtifact.create(
+            system_tarball,
+            data={
+                "variant": None,
+                "architecture": self.SAMPLE_TASK_DATA["bootstrap_options"][
+                    "architecture"
+                ],
+                "vendor": os_release_data["ID"],
+                "codename": "jessie",
+                "pkglist": packages,
+                "with_dev": True,
+                "with_init": True,
+                "mirror": mirror,
+            },
+        )
+
+        calls.append(
+            call(
+                expected_system_artifact,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
+            )
+        )
+
+        debusine_mock.upload_artifact.assert_has_calls(calls)
+
+    def test_upload_artifacts_sid(self):
+        """Test upload_artifacts() for a sid tarball."""
+        directory = self.create_temporary_directory()
+
+        system_tarball = directory / MmDebstrap._OUTPUT_SYSTEM_FILE
+        system_tarball.write_bytes(b"Generated tarball")
+
+        # Debusine.upload_artifact is mocked to verify the call only
+        debusine_mock = self.mock_debusine()
+
+        os_release_data = self.write_os_release(
+            directory, codename="trixie", version=13
+        )
+        (directory / self.task._TEST_SBIN_INIT_RETURN_CODE_FILE).write_text("0")
+
+        packages = self.patch_subprocess_run_pkglist()
+
+        mirror = self.SAMPLE_TASK_DATA["bootstrap_repositories"][0]["mirror"]
+        self.task.data.bootstrap_repositories[0].suite = "unstable"
+
+        self.task.upload_artifacts(directory, execution_success=True)
+
+        calls = []
+
+        expected_system_artifact = DebianSystemTarballArtifact.create(
+            system_tarball,
+            data={
+                "variant": None,
+                "architecture": self.SAMPLE_TASK_DATA["bootstrap_options"][
+                    "architecture"
+                ],
+                "vendor": os_release_data["ID"],
+                "codename": "sid",
+                "pkglist": packages,
+                "with_dev": True,
+                "with_init": True,
+                "mirror": mirror,
+            },
+        )
+
+        calls.append(
+            call(
+                expected_system_artifact,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
             )
         )
 
