@@ -11,9 +11,9 @@
 import importlib.resources
 from copy import deepcopy
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any
 from unittest import TestCase
-from unittest.mock import MagicMock, call, patch
+from unittest.mock import call, patch
 
 import responses
 
@@ -83,7 +83,7 @@ class SimpleSystemImageBuildTests(
 
     def findActionByDescription(
         self, description: str, recipe: dict[str, Any]
-    ) -> Optional[dict[str, Any]]:
+    ) -> dict[str, Any] | None:
         """Find an action with description in recipe. Return it."""
         for action in recipe["actions"]:
             if action.get("description") == description:
@@ -106,7 +106,6 @@ class SimpleSystemImageBuildTests(
         self.assertEqual(
             self.task.data.dict(),
             {
-                'notifications': None,
                 'bootstrap_options': {
                     'architecture': 'amd64',
                     'extra_packages': ['hello', 'bye'],
@@ -140,12 +139,7 @@ class SimpleSystemImageBuildTests(
 
     def test_analyze_worker(self):
         """Test the analyze_worker() method."""
-        mock_result = MagicMock()
-        mock_result.stdout = b"arm64\n"
-        with (
-            patch("subprocess.run", autospec=True, return_value=mock_result),
-            patch("os.access", return_value=True),
-        ):
+        with patch("os.access", return_value=True):
             self.mock_is_command_available(
                 {
                     "/lib/systemd/systemd-resolved": True,
@@ -155,18 +149,10 @@ class SimpleSystemImageBuildTests(
             )
             metadata = self.task.analyze_worker()
             self.assertTrue(metadata["simplesystemimagebuild:available"])
-            self.assertEqual(
-                metadata["simplesystemimagebuild:architecture"], "arm64"
-            )
 
     def test_analyze_worker_debos_not_available(self):
         """analyze_worker() handles debos not being available."""
-        mock_result = MagicMock()
-        mock_result.stdout = b"arm64\n"
-        with (
-            patch("subprocess.run", autospec=True, return_value=mock_result),
-            patch("os.access", return_value=True),
-        ):
+        with patch("os.access", return_value=True):
             self.mock_is_command_available(
                 {
                     "debos": False,
@@ -179,12 +165,7 @@ class SimpleSystemImageBuildTests(
 
     def test_analyze_worker_systemd_resolved_not_available(self):
         """analyze_worker() handles systemd-resolved not being available."""
-        mock_result = MagicMock()
-        mock_result.stdout = b"arm64\n"
-        with (
-            patch("subprocess.run", autospec=True, return_value=mock_result),
-            patch("os.access", return_value=True),
-        ):
+        with patch("os.access", return_value=True):
             self.mock_is_command_available(
                 {
                     "/lib/systemd/systemd-resolved": False,
@@ -197,12 +178,7 @@ class SimpleSystemImageBuildTests(
 
     def test_analyze_worker_kvm_not_available(self):
         """analyze_worker() handles KVM not being available."""
-        mock_result = MagicMock()
-        mock_result.stdout = b"arm64\n"
-        with (
-            patch("subprocess.run", autospec=True, return_value=mock_result),
-            patch("os.access", return_value=False),
-        ):
+        with patch("os.access", return_value=False):
             self.mock_is_command_available(
                 {
                     "/lib/systemd/systemd-resolved": True,
@@ -215,12 +191,7 @@ class SimpleSystemImageBuildTests(
 
     def test_analyze_worker_uml(self):
         """analyze_worker() will accept UML instead of KVM."""
-        mock_result = MagicMock()
-        mock_result.stdout = b"arm64\n"
-        with (
-            patch("subprocess.run", autospec=True, return_value=mock_result),
-            patch("os.access", return_value=False),
-        ):
+        with patch("os.access", return_value=False):
             self.mock_is_command_available(
                 {
                     "/lib/systemd/systemd-resolved": True,
@@ -236,10 +207,10 @@ class SimpleSystemImageBuildTests(
         self.assertTrue(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "simplesystemimagebuild:available": True,
                     "simplesystemimagebuild:version": self.task.TASK_VERSION,
-                    "simplesystemimagebuild:architecture": "amd64",
                 }
             )
         )
@@ -249,6 +220,7 @@ class SimpleSystemImageBuildTests(
         self.assertFalse(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "simplesystemimagebuild:available": True,
                     "simplesystemimagebuild:version": (
@@ -263,8 +235,10 @@ class SimpleSystemImageBuildTests(
         self.assertFalse(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "simplesystemimagebuild:available": False,
+                    "simplesystemimagebuild:version": self.task.TASK_VERSION,
                 }
             )
         )
@@ -274,10 +248,10 @@ class SimpleSystemImageBuildTests(
         self.assertFalse(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["alpha"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "simplesystemimagebuild:available": True,
                     "simplesystemimagebuild:version": self.task.TASK_VERSION,
-                    "simplesystemimagebuild:architecture": "alpha",
                 }
             )
         )
@@ -290,10 +264,10 @@ class SimpleSystemImageBuildTests(
         self.assertTrue(
             self.task.can_run_on(
                 {
+                    "system:architectures": ["amd64", "i386"],
                     "system:worker_type": WorkerType.EXTERNAL,
                     "simplesystemimagebuild:available": True,
                     "simplesystemimagebuild:version": self.task.TASK_VERSION,
-                    "simplesystemimagebuild:architecture": "amd64",
                 }
             )
         )
@@ -1225,8 +1199,8 @@ Running /debos --artifactdir /some/folder debusine2.yaml --internal-image
         calls.append(
             call(
                 expected_system_artifact,
-                workspace=self.task.workspace,
-                work_request=self.task.work_request,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
             )
         )
 
@@ -1309,8 +1283,91 @@ Running /debos --artifactdir /some/folder debusine2.yaml --internal-image
         calls.append(
             call(
                 expected_system_artifact,
-                workspace=self.task.workspace,
-                work_request=self.task.work_request,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
+            )
+        )
+
+        debusine_mock.upload_artifact.assert_has_calls(calls)
+
+    def test_upload_artifacts_unstable(self):
+        """Test upload_artifacts on unstable()."""
+        directory = self.create_temporary_directory()
+
+        debos_log = directory / SimpleSystemImageBuild.CAPTURE_OUTPUT_FILENAME
+        debos_log.write_text(
+            """\
+Running /debos --artifactdir /some/folder debusine2.yaml --internal-image
+2024/01/18 10:15:44 ==== Bootstrap stable minbase ====
+2024/01/18 10:15:44 Debootstrap | I: Target architecture can be executed
+2024/01/18 10:15:44 Debootstrap | I: Retrieving InRelease
+2024/01/18 10:16:56 apt | Building dependency tree...
+2024/01/18 10:16:56 apt | Reading state information...
+2024/01/18 10:16:56 ==== Get os-release ====
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release |\
+ PRETTY_NAME="Debian GNU/Linux trixie/sid"
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release | NAME="Debian GNU/Linux"
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release | VERSION_CODENAME="trixie"
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release | ID=debian
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release |\
+ HOME_URL="http://www.debian.org/"
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release |\
+ SUPPORT_URL="http://www.debian.org/support"
+2024/02/21 17:13:38 cat ${ROOTDIR}/etc/os-release |\
+ BUG_REPORT_URL="https://bugs.debian.org/"
+2024/01/18 10:16:56 ==== Get package list ====
+2024/01/18 10:16:56 dpkg-query --root ${ROOTDIR} -W | adduser	3.134
+2024/01/18 10:16:56 dpkg-query --root ${ROOTDIR} -W | apt	2.6.1
+2024/01/18 10:16:56 dpkg-query --root ${ROOTDIR} -W | base-files	12.4+deb12u4
+2024/01/18 10:16:56 dpkg-query --root ${ROOTDIR} -W | base-passwd	3.6.1
+2024/01/18 10:16:56 ==== tar image ====
+2024/01/18 10:16:56 Compressing to /some/folder/image.tar.xz
+"""
+        )
+
+        system_image = directory / self.task.filename
+        system_image.write_bytes(b"Generated image")
+
+        # Debusine.upload_artifact is mocked to verify the call only
+        debusine_mock = self.mock_debusine()
+
+        self.task.data.bootstrap_repositories[0].suite = "unstable"
+        self.task.upload_artifacts(directory, execution_success=True)
+
+        calls = []
+
+        expected_system_artifact = DebianSystemImageArtifact.create(
+            system_image,
+            data={
+                "variant": None,
+                "architecture": self.SAMPLE_TASK_DATA["bootstrap_options"][
+                    "architecture"
+                ],
+                "vendor": "debian",
+                "codename": "sid",
+                "pkglist": {
+                    "adduser": "3.134",
+                    "apt": "2.6.1",
+                    "base-files": "12.4+deb12u4",
+                    "base-passwd": "3.6.1",
+                },
+                "with_dev": True,
+                "with_init": True,
+                "mirror": self.SAMPLE_TASK_DATA["bootstrap_repositories"][0][
+                    "mirror"
+                ],
+                "image_format": "raw",
+                "filesystem": "ext4",
+                "size": 2e9,
+                "boot_mechanism": "efi",
+            },
+        )
+
+        calls.append(
+            call(
+                expected_system_artifact,
+                workspace=self.task.workspace_name,
+                work_request=self.task.work_request_id,
             )
         )
 
