@@ -38,7 +38,11 @@ from debusine.test import TestHelpersMixin
 from debusine.test.django import ChannelsHelpersMixin
 
 
-class TestChannelsHelpersMixinTests(ChannelsHelpersMixin, TestCase):
+# TODO: coverage is confused by something here, possibly
+# https://github.com/python/cpython/issues/106749
+class TestChannelsHelpersMixinTests(
+    ChannelsHelpersMixin, TestCase
+):  # pragma: no cover
     """Tests for methods in ChannelsHelpersMixin."""
 
     # Default channel name to be used during the tests.
@@ -102,6 +106,8 @@ class TestChannelsHelpersMixinTests(ChannelsHelpersMixin, TestCase):
 class TestHelpersMixinTests(TestHelpersMixin, TestCase):
     """Tests for methods in TestHelpersMixin."""
 
+    playground_memory_file_store = False
+
     def test_create_temp_config_directory(self):
         """create_temp_config_directory write the configuration."""
         config = {
@@ -286,7 +292,8 @@ class TestHelpersMixinTests(TestHelpersMixin, TestCase):
         contents = b"some-test-data"
 
         fileobj = self.create_file_in_backend(
-            file_backend, contents, remove_file_from_backend=False
+            file_backend,
+            contents,
         )
 
         self.assertIsInstance(fileobj.id, int)
@@ -515,16 +522,14 @@ class TestHelpersMixinTests(TestHelpersMixin, TestCase):
 
     def test_create_token_existing_user(self):
         """Test create_token_enabled with_user use the existing user."""
-        user = get_user_model().objects.create_user(
-            username="usertest", password="userpassword"
-        )
-        self.assertEqual(get_user_model().objects.count(), 1)
+        user = self.get_test_user()
+        user_count = get_user_model().objects.count()
 
         token = self.create_token_enabled(with_user=True)
         token.refresh_from_db()
         self.assertEqual(token.user, user)
 
-        self.assertEqual(get_user_model().objects.count(), 1)
+        self.assertEqual(get_user_model().objects.count(), user_count)
 
     @staticmethod
     def _process_json_response(response):
