@@ -13,13 +13,14 @@ import shutil
 import tempfile
 from datetime import timedelta
 from pathlib import Path
-from typing import Any, Generic, TYPE_CHECKING, TypeVar, Union, cast
+from typing import Any, Generic, TYPE_CHECKING, TypeVar, cast
 
 from django.core.exceptions import ValidationError
 from django.db import transaction
 from django.db.models import Model
 from django.forms import (
     BaseForm,
+    BooleanField,
     CharField,
     ChoiceField,
     ClearableFileInput,
@@ -27,6 +28,7 @@ from django.forms import (
     DateTimeInput,
     Field,
     FileField,
+    Form,
     IntegerField,
     ModelChoiceField,
     ModelForm,
@@ -44,6 +46,8 @@ import yaml
 from debusine.artifacts import LocalArtifact
 from debusine.db.models import (
     Artifact,
+    Collection,
+    CollectionItem,
     File,
     FileInArtifact,
     Token,
@@ -51,6 +55,8 @@ from debusine.db.models import (
     Workspace,
 )
 from debusine.tasks import BaseTask
+from debusine.tasks.models import TaskTypes
+from debusine.web.utils import ui_prototype
 
 if TYPE_CHECKING:
     ModelFormBase = ModelForm
@@ -243,16 +249,35 @@ class WorkRequestForm(YamlMixin, BootstrapMixin, ModelFormBase[WorkRequest]):
         super().__init__(*args, **kwargs)
 
         self.fields["workspace"] = WorkspaceChoiceField(user=self.user)
+        if self.user.is_superuser:
+            tasks = BaseTask.task_names(TaskTypes.WORKER) + BaseTask.task_names(
+                TaskTypes.SERVER
+            )
+        else:
+            tasks = BaseTask.worker_task_names()
         self.fields["task_name"] = ChoiceField(
-            choices=lambda: [
-                [name, name] for name in sorted(BaseTask.task_names())
-            ]
+            choices=lambda: [[name, name] for name in sorted(tasks)]
         )
 
+    def _get_task_type(self, name: str) -> TaskTypes:
+        """Infer a task type from a task name."""
+        if self.user.is_superuser and not BaseTask.is_worker_task(name):
+            return TaskTypes.SERVER
+        return TaskTypes.WORKER
+
+    def full_clean(self):
+        """Make sure task_type is filled."""
+        if "task_name" in self.data:
+            self.instance.task_type = self._get_task_type(
+                self.data["task_name"]
+            )
+        super().full_clean()
+
     def save(self, commit=True):
         """Save the work request."""
         instance = super().save(commit=False)
         instance.created_by = self.user
+        instance.task_type = self._get_task_type(instance.task_name)
 
         if commit:
             instance.save()
@@ -283,7 +308,7 @@ class MultipleFileField(FileField):
         kwargs.setdefault("widget", MultipleFileInput())
         super().__init__(*args, **kwargs, allow_empty_file=True)
 
-    def clean(self, data: Union[list[Any], tuple[Any, ...]], initial=None):
+    def clean(self, data: list[Any] | tuple[Any, ...], initial=None):
         """Call super().clean() for each file."""  # noqa: D402
         single_file_clean = super().clean
         return [single_file_clean(file, initial) for file in data]
@@ -407,3 +432,28 @@ class ArtifactForm(BootstrapMixin, YamlMixin, ModelFormBase[Artifact]):
     class Meta:
         model = Artifact
         fields = ["category", "workspace", "files", "data"]
+
+
+@ui_prototype
+class CollectionSearchForm(Form):
+    """Form for collection search fields."""
+
+    category = ChoiceField(required=False)
+    name = CharField(required=False)
+    historical = BooleanField(required=False)
+
+    def __init__(self, *args, instance: Collection, **kwargs):
+        """Initialize category choices from the database."""
+        super().__init__(*args, **kwargs)
+        self.instance = instance
+        choices = [("", "All")]
+        choices.extend(
+            (name, name)
+            for name in CollectionItem.objects.filter(
+                parent_collection=self.instance
+            )
+            .values_list("category", flat=True)
+            .distinct()
+            .order_by("category")
+        )
+        cast(ChoiceField, self.fields["category"]).choices = choices
