@@ -11,6 +11,7 @@
 import textwrap
 from datetime import timedelta
 from pathlib import Path
+from typing import Any, ClassVar
 
 from django import forms
 from django.contrib.auth import get_user_model
@@ -26,9 +27,10 @@ except ImportError:
 import yaml
 
 from debusine.artifacts import PackageBuildLog
-from debusine.db.models import Artifact, Token, Workspace
-from debusine.db.tests.test_models import FileHelpersMixin
+from debusine.artifacts.models import ArtifactCategory
+from debusine.db.models import Artifact, Token, User, Workspace
 from debusine.tasks import BaseTask
+from debusine.tasks.models import TaskTypes
 from debusine.test import TestHelpersMixin
 from debusine.web.forms import (
     ArtifactForm,
@@ -68,9 +70,12 @@ class BootstrapMixinForm(BootstrapMixin, forms.Form):
 class BootstrapMixinTests(TestCase):
     """Tests for BootstrapMixin."""
 
+    form: ClassVar[BootstrapMixinForm]
+
     @classmethod
     def setUpTestData(cls):
         """Set up common data for tests."""
+        super().setUpTestData()
         cls.form = BootstrapMixinForm()
 
     def test_charfield_bootstrap_class(self):
@@ -153,9 +158,12 @@ class BootstrapMixinTests(TestCase):
 class TokenFormTests(TestCase):
     """Tests for TokenForm."""
 
+    user: ClassVar[User]
+
     @classmethod
     def setUpTestData(cls):
         """Initialize test objects."""
+        super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
@@ -207,9 +215,13 @@ class TokenFormTests(TestCase):
 class WorkRequestFormTests(TestCase):
     """Tests for WorkRequestForm."""
 
+    user: ClassVar[User]
+    workspace: ClassVar[Workspace]
+
     @classmethod
     def setUpTestData(cls):
         """Initialize test objects."""
+        super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
@@ -232,7 +244,7 @@ class WorkRequestFormTests(TestCase):
         host_architecture: amd64
         backend: schroot
         input:
-            source_artifact_id: 5
+            source_artifact: 5
         """  # noqa: E501
         )
 
@@ -317,7 +329,7 @@ class WorkRequestFormTests(TestCase):
         - all
         host_architecture: amd64
         input:
-          source_artifact_id: 5
+          source_artifact: 5
         """  # noqa: E501
         )
 
@@ -332,17 +344,17 @@ class WorkRequestFormTests(TestCase):
         self.assertFalse(form.is_valid())
         self.assertRegex(
             form.errors["task_data"][0],
-            'The backend "auto" requires "environment_id" to be set',
+            'The backend "auto" requires "environment" to be set',
         )
 
     def test_clean_data_yaml_return_dictionary(self):
         """clean_data_yaml return parsed YAML in a dictionary."""
         task_data = {
             "input": {
-                "source_artifact_id": 1,
+                "source_artifact": 1,
             },
             "host_architecture": "amd64",
-            "environment_id": 1,
+            "environment": "debian/match:codename=bookworm",
             "build_components": ["all"],
         }
         form = WorkRequestForm(
@@ -382,23 +394,53 @@ class WorkRequestFormTests(TestCase):
         """The field "task_name" is a choice field with the expected names."""
         form = WorkRequestForm(user=self.user)
 
-        task_names = [[name, name] for name in sorted(BaseTask.task_names())]
+        task_names = [
+            [name, name]
+            for name in sorted(BaseTask.worker_task_names())
+            if "internal" not in name
+        ]
+
+        self.assertEqual(
+            list(form.fields["task_name"].widget.choices), task_names
+        )
+
+    def test_task_name_superuser(self):
+        """The field "task_name" is a choice field with the expected names."""
+        user = get_user_model().objects.create_superuser(
+            username="testsuperuser",
+            email="superuser@mail.none",
+            password="testsuperpass",
+        )
+        form = WorkRequestForm(user=user)
+
+        task_names = [
+            [name, name]
+            for name in sorted(
+                BaseTask.task_names(TaskTypes.WORKER)
+                + BaseTask.task_names(TaskTypes.SERVER)
+            )
+        ]
 
         self.assertEqual(
             list(form.fields["task_name"].widget.choices), task_names
         )
 
 
-class ArtifactFormTests(FileHelpersMixin, TestCase):
+class ArtifactFormTests(TestHelpersMixin, TestCase):
     """Tests for ArtifactForm."""
 
+    user: ClassVar[User]
+    artifact_category: ClassVar[ArtifactCategory]
+    form_data: ClassVar[dict[str, Any]]
+
     @classmethod
     def setUpTestData(cls):
         """Initialize test objects."""
+        super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
-        cls.artifact_category = "debusine:work-request-debug-logs"
+        cls.artifact_category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         cls.form_data = {
             "category": cls.artifact_category,
             "workspace": Workspace.objects.first(),
@@ -533,28 +575,14 @@ class ArtifactFormTests(FileHelpersMixin, TestCase):
         """Check expected category choices."""
         form = ArtifactForm(user=self.user)
         choices = []
-        for category in sorted(
-            [
-                "debian:autopkgtest",
-                "debian:binary-package",
-                "debian:binary-packages",
-                "debian:blhc",
-                "debian:lintian",
-                "debian:package-build-log",
-                "debian:source-package",
-                "debian:system-image",
-                "debian:system-tarball",
-                "debian:upload",
-                "debusine:work-request-debug-logs",
-            ]
-        ):
-            choices.append((category, category))
+        for category in sorted(ArtifactCategory):
+            choices.append((str(category), str(category)))
 
         self.assertEqual(form.fields["category"].choices, choices)
 
     def test_file_zero_bytes(self):
         """The form is valid with a file of zero bytes."""
-        category = "debusine:work-request-debug-logs"
+        category = ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
         file_name = "build.txt"
 
         form_data = {
@@ -577,7 +605,7 @@ class ArtifactFormTests(FileHelpersMixin, TestCase):
         The data is YAML valid but the LocalArtifact does not validate.
         LocalArtifact checks the data with the contents of the files.
         """
-        category = "debian:package-build-log"
+        category = ArtifactCategory.PACKAGE_BUILD_LOG
         data = {
             "source": "test-source",
             "version": "1.2.3",
@@ -621,7 +649,7 @@ class ArtifactFormTests(FileHelpersMixin, TestCase):
         The data is YAML valid but the LocalArtifact does not validate.
         LocalArtifact checks the data with the contents of the files.
         """
-        category = "debian:package-build-log"
+        category = ArtifactCategory.PACKAGE_BUILD_LOG
         data = {
             "source": "test-source",
             "version": "1.2.3",
@@ -662,9 +690,12 @@ class ArtifactFormTests(FileHelpersMixin, TestCase):
 class WorkspaceChoiceFieldTests(TestHelpersMixin, TestCase):
     """Tests for WorkspaceChoiceField."""
 
+    user: ClassVar[User]
+
     @classmethod
     def setUpTestData(cls):
         """Initialize test objects."""
+        super().setUpTestData()
         cls.user = get_user_model().objects.create_user(
             username="testuser", password="testpass"
         )
