@@ -19,7 +19,7 @@ from django.test import TestCase
 from django.urls import Resolver404
 
 from debusine.db.models import WorkRequest
-from debusine.web.tests.views.test_views import (
+from debusine.web.views.tests.test_views import (
     _html_work_request_result,
     _html_work_request_status,
 )
