@@ -11,20 +11,30 @@
 
 from django.urls import path
 
-from debusine.web.views import CreateArtifactView, DownloadPathView
+import debusine.web.views.artifacts as views
 
 app_name = "artifacts"
 
 urlpatterns = [
     path(
-        "<int:artifact_id>/<path:path>",
-        DownloadPathView.as_view(),
-        name="detail-path",
+        "<int:artifact_id>/download/<path:path>",
+        views.DownloadPathView.as_view(),
+        name="download-path",
+    ),
+    path(
+        "<int:artifact_id>/download/",
+        views.DownloadPathView.as_view(),
+        name="download",
     ),
     path(
         "<int:artifact_id>/",
-        DownloadPathView.as_view(),
+        views.ArtifactDetailView.as_view(),
         name="detail",
     ),
-    path("create/", CreateArtifactView.as_view(), name="create"),
+    path(
+        "build-log/<int:artifact_id>/",
+        views.BuildLogView.as_view(),
+        name="build-log",
+    ),
+    path("create/", views.CreateArtifactView.as_view(), name="create"),
 ]
