@@ -11,28 +11,29 @@
 
 from django.urls import path
 
-from debusine.web.views.work_request import (
-    WorkRequestCreateView,
-    WorkRequestDetailView,
-    WorkRequestListView,
-)
+import debusine.web.views.work_request as views
 
 app_name = "work_requests"
 
 urlpatterns = [
     path(
         "",
-        WorkRequestListView.as_view(),
+        views.WorkRequestListView.as_view(),
         name="list",
     ),
     path(
         "<int:pk>/",
-        WorkRequestDetailView.as_view(),
+        views.WorkRequestDetailView.as_view(),
         name="detail",
     ),
+    path(
+        "<int:pk>/retry/",
+        views.WorkRequestRetryView.as_view(),
+        name="retry",
+    ),
     path(
         "create/",
-        WorkRequestCreateView.as_view(),
+        views.WorkRequestCreateView.as_view(),
         name="create",
     ),
 ]
