@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright 2023--2024 The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,16 +9,28 @@
 
 """URLs related to workspaces."""
 
-from django.urls import path
+from django.urls import include, path
 
-from debusine.web.views import WorkspaceListView
+import debusine.web.views.workspace as views
 
 app_name = "workspaces"
 
 urlpatterns = [
     path(
         "",
-        WorkspaceListView.as_view(),
+        views.WorkspaceListView.as_view(),
         name="list",
     ),
+    path(
+        "<str:wname>/view/",
+        views.WorkspaceDetailView.as_view(),
+        name="detail",
+    ),
+    path(
+        "<str:wname>/collection/",
+        include(
+            "debusine.web.urls.collections",
+            namespace="collections",
+        ),
+    ),
 ]
