@@ -12,8 +12,6 @@
 from debusine.web.views.autopkgtest import AutopkgtestView  # noqa: F401
 from debusine.web.views.lintian import LintianView  # noqa: F401
 from debusine.web.views.views import (  # noqa: F401
-    CreateArtifactView,
-    DownloadPathView,
     HomepageView,
     LoginView,
     LogoutView,
@@ -21,5 +19,4 @@ from debusine.web.views.views import (  # noqa: F401
     UserTokenDeleteView,
     UserTokenListView,
     UserTokenUpdateView,
-    WorkspaceListView,
 )
