@@ -10,40 +10,38 @@
 """debusine WorkRequest view."""
 
 import functools
-from typing import Any, Optional, TYPE_CHECKING
+from typing import Any, Optional
 
+from django.contrib import messages
 from django.core.exceptions import ValidationError
+from django.http import Http404, HttpRequest, HttpResponse
+from django.shortcuts import redirect
 from django.urls import reverse
-from django.views.generic import CreateView, DetailView, ListView
+from django.views.generic.base import View
 
 import yaml
 
 from debusine.db.models import Artifact, WorkRequest
+from debusine.db.models.work_requests import CannotRetry
 from debusine.server.views import IsUserAuthenticated, ValidatePermissionsMixin
+from debusine.tasks.models import TaskTypes
 from debusine.web.forms import WorkRequestForm
+from debusine.web.views.base import (
+    CreateViewBase,
+    DetailViewBase,
+    ListViewBase,
+)
 from debusine.web.views.mixins import PaginationMixin
 
-if TYPE_CHECKING:
-    CreateViewBase = CreateView
-    DetailViewBase = DetailView
-    ListViewBase = ListView
-else:
-    # Django's generic views don't support generic types at run-time yet.
-    class _CreateViewBase:
-        def __class_getitem__(*args):
-            return CreateView
 
-    class _DetailViewBase:
-        def __class_getitem__(*args):
-            return DetailView
+class WorkRequestDumper(yaml.SafeDumper):
+    """A YAML dumper that represents multi-line strings in the literal style."""
 
-    class _ListViewBase:
-        def __class_getitem__(*args):
-            return ListView
-
-    CreateViewBase = _CreateViewBase
-    DetailViewBase = _DetailViewBase
-    ListViewBase = _ListViewBase
+    def represent_scalar(self, tag, value, style=None):
+        """Represent multi-line strings in the literal style."""
+        if style is None and "\n" in value:
+            style = "|"
+        return super().represent_scalar(tag, value, style=style)
 
 
 class WorkRequestDetailView(DetailViewBase[WorkRequest]):
@@ -54,7 +52,7 @@ class WorkRequestDetailView(DetailViewBase[WorkRequest]):
     default_template_name = "web/work_request-detail.html"
 
     @functools.cached_property
-    def _validation_error_message(self) -> Optional[str]:
+    def _validation_error_message(self) -> str | None:
         """If the work request fails validation, return a suitable message."""
         try:
             self.object.full_clean()
@@ -101,8 +99,10 @@ class WorkRequestDetailView(DetailViewBase[WorkRequest]):
 
         context["artifacts"] = Artifact.objects.filter(
             created_by_work_request=self.object
-        ).order_by("category")
-        context["task_data"] = yaml.safe_dump(self.object.task_data)
+        ).order_by("category", "id")
+        context["task_data"] = yaml.dump(
+            self.object.task_data, Dumper=WorkRequestDumper
+        )
         return context
 
     def get_template_names(self):
@@ -151,7 +151,7 @@ class WorkRequestListView(PaginationMixin, ListViewBase[WorkRequest]):
 
     def get_queryset(self):
         """Filter work requests displayed by the workspace GET parameter."""
-        queryset = super().get_queryset()
+        queryset = super().get_queryset().exclude(task_type=TaskTypes.INTERNAL)
 
         workspace_name = self.request.GET.get("workspace")
         if workspace_name is not None:
@@ -167,7 +167,7 @@ class WorkRequestListView(PaginationMixin, ListViewBase[WorkRequest]):
     def get_ordering(self):
         """Return field used for sorting."""
         order = self.request.GET.get("order")
-        if order in ("id", "created_at", "status", "result"):
+        if order in ("id", "created_at", "task_name", "status", "result"):
             if self.request.GET["asc"] == "0":
                 return "-" + order
             else:
@@ -211,3 +211,32 @@ class WorkRequestCreateView(
     def get_success_url(self):
         """Redirect to work_requests:detail for the created WorkRequest."""
         return reverse("work_requests:detail", kwargs={"pk": self.object.id})
+
+
+class WorkRequestRetryView(ValidatePermissionsMixin, View):
+    """Form view for retrying a work request."""
+
+    permission_denied_message = (
+        "You need to be authenticated to retry a Work Request"
+    )
+    permission_classes = [IsUserAuthenticated]
+
+    @functools.cached_property
+    def work_request(self) -> WorkRequest:
+        """Return the work request for this request."""
+        try:
+            return WorkRequest.objects.get(pk=self.kwargs["pk"])
+        except WorkRequest.DoesNotExist:
+            raise Http404(f"Work request {self.kwargs['pk']} not found")
+
+    def post(
+        self, request: HttpRequest, *args, **kwargs  # noqa: U100
+    ) -> HttpResponse:
+        """Handle POST requests."""
+        try:
+            new_work_request = self.work_request.retry()
+        except CannotRetry as e:
+            messages.error(self.request, f"Cannot retry: {e}")
+            return redirect("work_requests:detail", pk=self.work_request.pk)
+
+        return redirect("work_requests:detail", pk=new_work_request.pk)
