@@ -48,8 +48,9 @@ import secrets
 import signal
 import socket
 from concurrent.futures import Future, ThreadPoolExecutor
+from pathlib import Path
 from threading import Lock
-from typing import Any, Optional
+from typing import Any, Literal, assert_never
 
 import aiohttp
 from aiohttp.web_exceptions import HTTPCreated, HTTPOk
@@ -66,6 +67,7 @@ from debusine.client.exceptions import TokenDisabledError
 from debusine.client.models import WorkRequestResponse
 from debusine.tasks import BaseExternalTask, BaseTask, TaskConfigError
 from debusine.tasks.executors import analyze_worker_all_executors
+from debusine.tasks.models import TaskTypes
 from debusine.worker.config import ConfigHandler
 from debusine.worker.debusine_async_http_client import DebusineAsyncHttpClient
 from debusine.worker.system_information import WorkerType, system_metadata
@@ -79,10 +81,13 @@ class Worker:
     def __init__(
         self,
         *,
-        log_file: Optional[str] = None,
-        log_level=None,
-        config=None,
-    ):
+        log_file: str | None = None,
+        log_level: str | None = None,
+        worker_type: Literal[
+            WorkerType.EXTERNAL, WorkerType.SIGNING
+        ] = WorkerType.EXTERNAL,
+        config: ConfigHandler | None = None,
+    ) -> None:
         """
         Initialize Worker.
 
@@ -94,10 +99,19 @@ class Worker:
         """
         self._original_log_file = log_file
         self._original_log_level = log_level
+        self._worker_type = worker_type
 
-        self._config: ConfigHandler
         if config is None:
-            self._config = ConfigHandler()
+            if worker_type == WorkerType.SIGNING:
+                self._config = ConfigHandler(
+                    directories=[
+                        str(Path.home() / ".config/debusine/signing"),
+                        "/etc/debusine/signing",
+                    ],
+                    require_https=True,
+                )
+            else:
+                self._config = ConfigHandler()
         else:
             self._config = config
 
@@ -105,22 +119,25 @@ class Worker:
 
         self._config.validate_config_or_fail()
 
-        self._aiohttp_client_session: Optional[aiohttp.ClientSession] = None
+        self._aiohttp_client_session: aiohttp.ClientSession | None = None
 
         self._async_http_client = DebusineAsyncHttpClient(logging, self._config)
 
-        # ThreadPool for BaseTask.execute (e.g. Sbuild.exec)
-        self._task_executor = ThreadPoolExecutor()
+        # ThreadPool for BaseTask.execute (e.g. Sbuild.exec).
+        # We only allow a single thread.  We only run one task at a time
+        # anyway, and signing workers need to run Django code in tasks which is
+        # safest if done from only one thread.
+        self._task_executor = ThreadPoolExecutor(max_workers=1)
 
         # BaseTask.execute (e.g. Sbuild.exec) future is stored here (avoids
         # garbage collection deletion)
-        self._task_exec_future: Optional[Future[bool]] = None
+        self._task_exec_future: Future[bool] | None = None
 
         # Used by ThreadPoolExecutor to submit tasks to the main event loop
-        self._main_event_loop = None
+        self._main_event_loop: asyncio.AbstractEventLoop | None = None
 
         # Running task
-        self._task_running: Optional[BaseTask[Any]] = None
+        self._task_running: BaseTask[Any, Any] | None = None
 
         # Lock to protect from concurrent work request execution
         self._task_lock: Lock = Lock()
@@ -217,7 +234,11 @@ class Worker:
 
         register_path = '/1.0/worker/register/'
 
-        data = {'token': token, 'fqdn': socket.getfqdn()}
+        data = {
+            "token": token,
+            "fqdn": socket.getfqdn(),
+            "worker_type": self._worker_type,
+        }
 
         try:
             response = await self._async_http_client.post(
@@ -254,6 +275,7 @@ class Worker:
 
     @staticmethod
     async def _asyncio_sleep(delay):
+        """Sleep asynchronously (mocked in tests)."""
         await asyncio.sleep(delay)
 
     def _raise_try_again(self):
@@ -300,7 +322,6 @@ class Worker:
                     worker_connect_ws_url, headers=headers, heartbeat=60
                 ) as ws:
                     self._reset_retry_attempt_number = True
-                    logging.info("Connected to %s", self._config.api_url)
                     msg: aiohttp.WSMessage
                     async for msg in ws:  # pragma: no branch
                         await self._process_message(msg)
@@ -385,6 +406,9 @@ class Worker:
          and it logs the reason.
         """
 
+        def connected(msg_content: dict[Any, Any]) -> None:  # noqa: U100
+            logging.info("Connected to %s", self._config.api_url)
+
         async def work_request_available(
             msg_content: dict[Any, Any]  # noqa: U100
         ):
@@ -393,13 +417,14 @@ class Worker:
         return await Debusine.process_async_message(
             msg,
             {
+                "connected": connected,
                 "request_dynamic_metadata": self._send_dynamic_metadata,
                 "work_request_available": work_request_available,
             },
             logger=logging,
         )
 
-    async def _request_work_request(self) -> Optional[WorkRequestResponse]:
+    async def _request_work_request(self) -> WorkRequestResponse | None:
         """Request a work request and returns it."""
         work_request = await self._async_http_client.get(
             '/1.0/work-request/get-next-for-worker/'
@@ -428,7 +453,7 @@ class Worker:
         dynamic_metadata_path = '/1.0/worker/dynamic-metadata/'
 
         metadata = {
-            **system_metadata(WorkerType.EXTERNAL),
+            **system_metadata(self._worker_type),
             **analyze_worker_all_executors(),
             **BaseTask.analyze_worker_all_tasks(),
         }
@@ -446,10 +471,45 @@ class Worker:
     async def _execute_work_request_and_submit_result(
         self, work_request: WorkRequestResponse
     ):
+        task_type = TaskTypes(work_request.task_type)
+        match self._worker_type:
+            case WorkerType.EXTERNAL:
+                expected_task_type = TaskTypes.WORKER
+            case WorkerType.SIGNING:
+                expected_task_type = TaskTypes.SIGNING
+            case _ as unreachable:
+                assert_never(unreachable)
+        if task_type != expected_task_type:
+            logging.error(
+                "Task: %s is of type %s, not %s",
+                work_request.task_name,
+                task_type,
+                expected_task_type,
+            )
+            self._debusine.work_request_completed_update(
+                work_request.id, "error"
+            )
+            self._reset_state()
+            return
+
         try:
-            self._task_running = BaseTask.class_from_name(
-                work_request.task_name
-            )(work_request.task_data)
+            task_class = BaseTask.class_from_name(
+                TaskTypes(work_request.task_type), work_request.task_name
+            )
+        except ValueError as exc:
+            logging.error(  # noqa: G200
+                'Task: %s Error setup: %s', work_request.task_name, exc
+            )
+            self._debusine.work_request_completed_update(
+                work_request.id, "error"
+            )
+            self._reset_state()
+            return
+
+        try:
+            self._task_running = task_class(
+                work_request.task_data, work_request.dynamic_task_data
+            )
         except TaskConfigError as exc:
             logging.error(  # noqa: G200
                 'Task: %s Error configure: %s', work_request.task_name, exc
@@ -460,16 +520,7 @@ class Worker:
             self._reset_state()
             return
 
-        if self._task_running.TASK_INTERNAL:
-            logging.error(
-                "Task: %s cannot run on an external worker",
-                work_request.task_name,
-            )
-            self._debusine.work_request_completed_update(
-                work_request.id, "error"
-            )
-            self._reset_state()
-            return
+        assert self._task_running.TASK_TYPE == work_request.task_type
         assert isinstance(self._task_running, BaseExternalTask)
 
         logger = logging.getLogger()
@@ -477,8 +528,8 @@ class Worker:
         self._task_running.configure_server_access(
             Debusine(self._config.api_url, self._config.token, logger=logger)
         )
-        self._task_running.work_request = work_request.id
-        self._task_running.workspace = work_request.workspace
+        self._task_running.work_request_id = work_request.id
+        self._task_running.workspace_name = work_request.workspace
 
         self._task_name = self._task_running.name
 
@@ -502,6 +553,11 @@ class Worker:
             # work request
             self._task_lock.release()
 
+    @staticmethod
+    async def _exit_thread():  # pragma: no cover
+        """Exit current thread, mocked in tests."""
+        raise SystemExit(1)
+
     def _send_task_result(self, work_request_id: int, task_exec_future) -> None:
         """
         Send the result of the task to the debusine server.
@@ -531,9 +587,22 @@ class Worker:
             else:
                 result = 'success' if task_exec_future.result() else 'failure'
 
-            self._debusine.work_request_completed_update(
-                work_request_id, result
-            )
+            try:
+                self._debusine.work_request_completed_update(
+                    work_request_id, result
+                )
+            except Exception:
+                # Don't leave the worker stuck in "Running" state on the server
+                logging.exception(
+                    "Cannot reach server to report work request completed,"
+                    " exiting."
+                )
+                # _main_event_loop initialized in main and never unset
+                assert self._main_event_loop is not None
+                # Exit main asyncio thread from this secondary thread
+                asyncio.run_coroutine_threadsafe(
+                    self._exit_thread(), self._main_event_loop
+                )
         finally:
             self._reset_state()
 
