@@ -11,7 +11,7 @@
 
 from dataclasses import dataclass, field
 from enum import Enum, auto
-from typing import Any, Optional, Union
+from typing import Any
 
 from aiohttp import web
 from aiohttp.test_utils import AioHTTPTestCase
@@ -32,13 +32,13 @@ from debusine.client.models import (
 class ServerConfig:
     """Settings that define server's behaviour when receiving requests."""
 
-    registered_token: Optional[str] = None
+    registered_token: str | None = None
 
     # HTTP status code returned by /api/1.0/worker/register/ endpoint
     register_response_status_code: int = HTTPCreated.status_code
 
     # The response body when a websocket connection is established
-    # If None the server does not send anything
+    # If None the server only sends a "connected" message
     # (/api/ws/1.0/worker/connect/)
     connect_response_bodies: list[dict[str, str]] = field(default_factory=list)
 
@@ -49,11 +49,12 @@ class ServerConfig:
         INVALID_WORK_REQUEST = auto()  # missing mandatory fields
         INVALID_JSON = auto()  # server did not return valid JSON
 
-    get_next_for_worker: Union[GetNextForWorkerResponse, web.Response] = (
+    get_next_for_worker: GetNextForWorkerResponse | web.Response = (
         GetNextForWorkerResponse.VALID_WORK_REQUEST
     )
 
     RESPONSES = {
+        'connected': {'text': 'connected'},
         'request_dynamic_metadata': {'text': 'request_dynamic_metadata'},
         'work_request_available': {'text': 'work_request_available'},
         'token_disabled_error': {
@@ -67,16 +68,16 @@ class ServerConfig:
 class RequestStorage:
     """Storage for the latest requests received by the server's endpoints."""
 
-    method: Optional[str] = None
-    path: Optional[str] = None
-    json_content: Optional[dict[Any, Any]] = None
+    method: str | None = None
+    path: str | None = None
+    json_content: dict[Any, Any] | None = None
 
     # the token received from the Worker matched the registered_token
     # from the server's registered_token
     is_authenticated: bool = False
 
     # received headers
-    headers: Optional[CIMultiDictProxy[str]] = None
+    headers: CIMultiDictProxy[str] | None = None
 
 
 class DebusineAioHTTPTestCase(AioHTTPTestCase):
@@ -89,7 +90,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
     """
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
-        """Initialise configuration of the server."""
+        """Initialize configuration of the server."""
         super().__init__(*args, **kwargs)
 
         # server_config changes what the server returns to the Worker:
@@ -101,7 +102,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
         self.server_requests: list[RequestStorage] = []
 
     @property
-    def server_latest_request(self) -> Optional[RequestStorage]:
+    def server_latest_request(self) -> RequestStorage | None:
         """Return the latest request done to the server or None."""
         if self.server_requests:
             return self.server_requests[-1]
@@ -157,7 +158,9 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
 
             await collect_request_information(request)
 
-            for body in self.server_config.connect_response_bodies:
+            for body in self.server_config.connect_response_bodies or [
+                self.server_config.RESPONSES["connected"]
+            ]:
                 await ws.send_json(body)
 
             return ws
@@ -165,7 +168,7 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
         def _create_valid_work_request():
             task_data = {
                 "input": {
-                    "source_artifact_id": 5,
+                    "source_artifact": 5,
                 },
                 "distribution": "bullseye",
                 "backend": "schroot",
@@ -174,14 +177,11 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
                     "any",
                     "all",
                 ],
-                "sbuild_options": [
-                    "--post-build-commands=/usr/local/bin/"
-                    "post-process %SBUILD_CHANGES",
-                ],
             }
 
             next_work_request = WorkRequestResponse(
                 id=52,
+                task_type="Worker",
                 task_name="sbuild",
                 created_at="2022-01-05T11:14:22.242178Z",
                 started_at=None,
@@ -189,6 +189,8 @@ class DebusineAioHTTPTestCase(AioHTTPTestCase):
                 duration=None,
                 worker=58,
                 task_data=task_data,
+                priority_base=0,
+                priority_adjustment=0,
                 status="running",
                 result="",
                 artifacts=[],
