@@ -16,13 +16,13 @@ import logging
 import signal
 import socket
 import threading
-from typing import Any, Optional
+from pathlib import Path
+from typing import Any, Literal
 from unittest import mock
 from unittest.mock import MagicMock
 
 import aiohttp
 from aiohttp import RequestInfo, WSMessage, WSMsgType
-from aiohttp.test_utils import unittest_run_loop
 from aiohttp.web import json_response
 from aiohttp.web_exceptions import (
     HTTPForbidden,
@@ -40,10 +40,11 @@ from debusine.client.models import (
     WorkRequestResponse,
     model_to_json_serializable_dict,
 )
-from debusine.tasks import BaseTask, TaskConfigError
+from debusine.tasks import BaseExternalTask, BaseTask, TaskConfigError
+from debusine.tasks.models import TaskTypes, WorkerType
 from debusine.tasks.noop import Noop
 from debusine.tasks.sbuild import Sbuild
-from debusine.test import TestHelpersMixin
+from debusine.test import TestCase
 from debusine.worker import Worker
 from debusine.worker.config import ConfigHandler
 from debusine.worker.system_information import system_metadata
@@ -51,13 +52,13 @@ from debusine.worker.tests import server
 from debusine.worker.tests.server import ServerConfig
 
 
-class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
+class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
     """Test Worker class."""
 
     async def setUpAsync(self) -> None:
         """Initialize TestWorker."""
         await super().setUpAsync()
-        self.worker: Optional[Worker] = None
+        self.worker: Worker | None = None
 
         self.api_url = str(self.server.make_url('')) + "/api"
 
@@ -97,6 +98,9 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         connect=False,
         log_file=None,
         log_level=None,
+        worker_type: Literal[
+            WorkerType.EXTERNAL, WorkerType.SIGNING
+        ] = WorkerType.EXTERNAL,
         assert_raise_try_again_called=False,
         disable_endless_retry=True,
     ):
@@ -107,12 +111,13 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         self.worker = Worker(
             log_file=log_file,
             log_level=log_level,
+            worker_type=worker_type,
             config=config,
         )
 
         try:
             # If the caller of setup_worker created an event loop
-            # (e.g. using @unittest_run_loop) then let the worker use it.
+            # then let the worker use it.
             self.worker._set_main_event_loop()
         except RuntimeError:
             pass
@@ -178,7 +183,26 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         # Worker() created a ConfigHandler default object and set it
         # to Worker._config
         self.assertIsNotNone(worker._config)
-        mocked_config_handler.assert_called()
+        mocked_config_handler.assert_called_once_with()
+
+    def test_worker_init_signing(self):
+        """Signing workers use different dirs and require an HTTPS api-url."""
+        mocked_config_handler = self.patch_config_handler()
+
+        worker = Worker(
+            log_file="/dev/null",
+            log_level=logging.WARNING,
+            worker_type=WorkerType.SIGNING,
+        )
+
+        self.assertIsNotNone(worker._config)
+        mocked_config_handler.assert_called_once_with(
+            directories=[
+                str(Path.home() / ".config/debusine/signing"),
+                "/etc/debusine/signing",
+            ],
+            require_https=True,
+        )
 
     def test_worker_init_use_config_handler(self):
         """worker.set_config_handler() sets the ConfigHandler."""
@@ -211,7 +235,11 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         # Server side received expected registration data
         self.assertEqual(
             registration_request.json_content,
-            {'token': self.config.token, 'fqdn': socket.getfqdn()},
+            {
+                'token': self.config.token,
+                'fqdn': socket.getfqdn(),
+                'worker_type': WorkerType.EXTERNAL,
+            },
         )
 
     def test_registration_failure(self):
@@ -267,7 +295,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         )
         self.assertFalse(self.server_latest_request.is_authenticated)
 
-    @unittest_run_loop
     async def test_connection_reset_error(self):
         """Worker client handles ConnectionResetError exception."""
         self.setup_valid_token()
@@ -294,7 +321,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         # and exception because side_effect=[None]
         self.assertEqual(mock_wait_before_reconnect.call_count, 2)
 
-    @unittest_run_loop
     async def test_connection_read_error(self):
         """Worker cannot read data from the server (low level disconnection)."""
         self.setup_worker()
@@ -316,7 +342,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertTrue(mock_post.called)
 
-    @unittest_run_loop
     async def test_client_connector_error(self):
         """Worker cannot connect to the server: server unreachable."""
         self.setup_worker()
@@ -354,7 +379,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         ):
             self.async_run(self.worker.connect())
 
-    @unittest_run_loop
     async def test_connection_error_connection_refused(self):
         """Worker client cannot connect: connection refused."""
         # Because the token exists the worker will try to connect
@@ -384,7 +408,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         # and exception because mocked_sleep.side_effect=[None, None]
         self.assertEqual(mocked_sleep.call_count, 3)
 
-    @unittest_run_loop
     async def test_do_wait_for_messages_raise_value_error(self):
         """Exception ValueError is raised (not retried via tenacity)."""
         self.setup_valid_token()
@@ -400,7 +423,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         with self.assertRaises(exception):
             await self.worker.connect()
 
-    @unittest_run_loop
     async def test_connection_error_token_disabled_try_again(self):
         """Worker tries to connect again if server returns "TOKEN_DISABLED"."""
         self.setup_valid_token()
@@ -436,7 +458,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertTrue(token_is_disabled_found)
 
-    @unittest_run_loop
     async def test_process_message_dynamic_metadata_request(self):
         """Worker receives a request to send the dynamic metadata."""
         executors_metadata = {"executor:unshare:available": True}
@@ -452,6 +473,7 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         )
         mocked_analyze_executors = analyze_executors_patcher.start()
         mocked_analyze_executors.return_value = executors_metadata
+        self.addCleanup(analyze_executors_patcher.stop)
 
         analyze_tasks_patcher = mock.patch(
             'debusine.tasks.BaseTask.analyze_worker_all_tasks', autospec=True
@@ -491,6 +513,34 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             },
         )
 
+    async def test_process_message_dynamic_metadata_request_signing(self):
+        """Signing workers send the correct worker type in dynamic metadata."""
+        self.setup_valid_token()
+        self.server_config.connect_response_bodies = [
+            ServerConfig.RESPONSES["request_dynamic_metadata"]
+        ]
+        self.assertIsNone(self.server_latest_request)
+
+        self.setup_worker(worker_type=WorkerType.SIGNING)
+        with mock.patch(
+            "debusine.worker._worker.analyze_worker_all_executors",
+            return_value={},
+        ), mock.patch(
+            "debusine.tasks.BaseTask.analyze_worker_all_tasks", return_value={}
+        ):
+            await self.worker.connect()
+
+        self.assertEqual(
+            self.server_latest_request.path, "/api/1.0/worker/dynamic-metadata/"
+        )
+        self.assertTrue(self.server_latest_request.is_authenticated)
+        self.assertEqual(
+            self.server_latest_request.json_content,
+            {
+                **system_metadata(task_models.WorkerType.SIGNING),
+            },
+        )
+
     def assert_request_done_times(self, method, path, times):
         """Assert that request method regexp_path has been done times."""
         count = 0
@@ -500,16 +550,28 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertEqual(count, times)
 
-    async def receive_task_submit_result(self, execute_result, json_result):
+    async def receive_task_submit_result(
+        self,
+        task_type: TaskTypes,
+        task_name: str,
+        execute_result,
+        json_result,
+        worker_type: Literal[
+            WorkerType.EXTERNAL, WorkerType.SIGNING
+        ] = WorkerType.EXTERNAL,
+    ):
         """Worker fetches the task and validates submitted JSON."""
-        # Mock Sbuild.execute()
-        patcher_execute = mock.patch.object(Sbuild, 'execute', autospec=True)
+        task_class = BaseTask.class_from_name(task_type, task_name)
+
+        patcher_execute = mock.patch.object(
+            task_class, 'execute', autospec=True
+        )
         mocked_execute = patcher_execute.start()
         mocked_execute.return_value = execute_result
         self.addCleanup(patcher_execute.stop)
 
         patcher_configure_server_access = mock.patch.object(
-            Sbuild, "configure_server_access", autospec=True
+            task_class, "configure_server_access", autospec=True
         )
         mocked_configure_server_access = patcher_configure_server_access.start()
         self.addCleanup(patcher_configure_server_access.stop)
@@ -522,7 +584,8 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         # No request to any URL...
         self.assertIsNone(self.server_latest_request)
 
-        self.setup_worker()
+        self.setup_worker(worker_type=worker_type)
+        assert self.worker is not None
 
         work_request_completed_mocked = (
             self.patch_debusine_work_request_completed_update()
@@ -535,7 +598,7 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         # Check the workspace assigned in server.py
         self.assertEqual(
-            mocked_execute.call_args[0][0].workspace, "TestWorkspace"
+            mocked_execute.call_args[0][0].workspace_name, "TestWorkspace"
         )
 
         # BaseTask.configure_server_access was called
@@ -556,17 +619,18 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             work_request_completed_mocked.mock_calls[0][1][1], json_result
         )
 
-    @unittest_run_loop
     async def test_task_is_received_executed_submit_success(self):
         """Receive a task, execute it and submit success completion."""
-        await self.receive_task_submit_result(True, 'success')
+        await self.receive_task_submit_result(
+            TaskTypes.WORKER, "sbuild", True, "success"
+        )
 
-    @unittest_run_loop
     async def test_task_is_received_executed_submit_failure(self):
         """Receive a task, executes it (failure) and submits failure."""
-        await self.receive_task_submit_result(False, 'failure')
+        await self.receive_task_submit_result(
+            TaskTypes.WORKER, "sbuild", False, "failure"
+        )
 
-    @unittest_run_loop
     async def test_print_no_work_request_available(self):
         """Worker logs 'No work request available'."""
         self.setup_valid_token()
@@ -584,7 +648,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         with self.assertLogsContains(log_message, level=logging.DEBUG):
             await self.worker.connect()
 
-    @unittest_run_loop
     async def test_log_work_request_ignored_already_running(self):
         """Worker receive two Work Request available, fetches only one."""
         lock = threading.Lock()
@@ -595,11 +658,17 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             lock.release()
 
         patcher = mock.patch.object(Sbuild, "execute", autospec=True)
-
         mocked_execute = patcher.start()
         mocked_execute.side_effect = wait
         self.addCleanup(patcher.stop)
 
+        # avoid NotFoundError in test env
+        patcher = mock.patch(
+            "debusine.client.debusine.Debusine.work_request_completed_update"
+        )
+        patcher.start()
+        self.addCleanup(patcher.stop)
+
         self.setup_valid_token()
 
         work_request_available = ServerConfig.RESPONSES[
@@ -641,6 +710,45 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         while self.worker._task_exec_future is not None:
             await asyncio.sleep(0)
 
+    async def test_log_work_request_cannot_report_exit(self):
+        """Worker can't report to the server, exits."""
+        # Can't catch SystemExit from main thread in async tests
+        patcher = mock.patch(
+            'debusine.worker.Worker._exit_thread', autospec=True
+        )
+        patched_exit_thread = patcher.start()
+        self.addCleanup(patcher.stop)
+
+        # Mock Sbuild.execute()
+        patcher_execute = mock.patch.object(Sbuild, 'execute', autospec=True)
+        mocked_execute = patcher_execute.start()
+        mocked_execute.return_value = True
+        self.addCleanup(patcher_execute.stop)
+
+        patcher_configure_server_access = mock.patch.object(
+            Sbuild, "configure_server_access", autospec=True
+        )
+        patcher_configure_server_access.start()
+        self.addCleanup(patcher_configure_server_access.stop)
+
+        self.setup_valid_token()
+        self.server_config.connect_response_bodies = [
+            ServerConfig.RESPONSES['work_request_available']
+        ]
+
+        self.setup_worker()
+
+        work_request_completed_mocked = (
+            self.patch_debusine_work_request_completed_update()
+        )
+        work_request_completed_mocked.side_effect = Exception("network error")
+
+        with self.assertLogsContains(
+            "Cannot reach server", level=logging.ERROR
+        ):
+            await self.worker.connect()
+            patched_exit_thread.assert_called()
+
     async def assert_invalid_get_next_for_worker_logging(
         self, get_next_for_worker
     ):
@@ -658,28 +766,33 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         ):
             await self.worker.connect()
 
-    @unittest_run_loop
     async def test_request_work_request_invalid_work_request_response(self):
         """Worker logs 'Invalid content of...'."""
         await self.assert_invalid_get_next_for_worker_logging(
             ServerConfig.GetNextForWorkerResponse.INVALID_WORK_REQUEST
         )
 
-    @unittest_run_loop
     async def test_request_work_request_invalid_json_response(self):
         """Worker logs 'Invalid JSON response...'."""
         await self.assert_invalid_get_next_for_worker_logging(
             ServerConfig.GetNextForWorkerResponse.INVALID_JSON
         )
 
-    async def assert_execute_work_request_send_and_log_error(self, error_msg):
+    async def assert_execute_work_request_send_and_log_error(
+        self,
+        error_msg,
+        worker_type: Literal[
+            WorkerType.EXTERNAL, WorkerType.SIGNING
+        ] = WorkerType.EXTERNAL,
+    ):
         """Assert error is sent to the server and logged."""
         self.setup_valid_token()
         self.server_config.connect_response_bodies = [
             ServerConfig.RESPONSES['work_request_available']
         ]
 
-        self.setup_worker()
+        self.setup_worker(worker_type=worker_type)
+        assert self.worker is not None
 
         work_request_completed_mocked = (
             self.patch_debusine_work_request_completed_update()
@@ -703,7 +816,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         )
         self.assertEqual(work_request_completed_mocked.call_args[0][1], "error")
 
-    @unittest_run_loop
     async def test_task_configure_error_submit_error(self):
         """Sbuild.configure fails, Worker submits error."""
         patcher = mock.patch.object(Sbuild, '_configure', autospec=True)
@@ -715,28 +827,40 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             'Task: sbuild Error configure: Invalid schema'
         )
 
-    @unittest_run_loop
-    async def test_internal_task(self):
-        """An external worker refuses to run an internal task."""
+    async def test_server_task_on_external_worker(self):
+        """An external worker refuses to run a server task."""
 
-        class InternalTestData(task_models.BaseTaskData):
+        class ServerTestData(task_models.BaseTaskData):
             pass
 
-        class InternalTest(BaseTask[InternalTestData]):
+        class ServerTest(
+            BaseTask[ServerTestData, task_models.BaseDynamicTaskData]
+        ):
             TASK_VERSION = 1
-            TASK_INTERNAL = True
+            TASK_TYPE = TaskTypes.SERVER
 
-        self.addCleanup(BaseTask._sub_tasks.__delitem__, "internaltest")
+            def _execute(self) -> bool:
+                raise NotImplementedError()
+
+            def _upload_work_request_debug_logs(self):
+                raise NotImplementedError()
+
+        self.addCleanup(
+            BaseTask._sub_tasks[TaskTypes.SERVER].__delitem__, "servertest"
+        )
 
         next_work_request = WorkRequestResponse(
             id=52,
-            task_name="internaltest",
+            task_type="Server",
+            task_name="servertest",
             created_at="2022-01-05T11:14:22.242178Z",
             started_at=None,
             completed_at=None,
             duration=None,
             worker=58,
             task_data={},
+            priority_base=0,
+            priority_adjustment=0,
             status="running",
             result="",
             artifacts=[],
@@ -747,7 +871,155 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         )
 
         await self.assert_execute_work_request_send_and_log_error(
-            "Task: internaltest cannot run on an external worker"
+            "Task: servertest is of type Server, not Worker"
+        )
+
+    async def test_signing_task_on_external_worker(self):
+        """An external worker refuses to run a signing task."""
+
+        class SigningTestData(task_models.BaseTaskData):
+            pass
+
+        class SigningTest(
+            BaseExternalTask[SigningTestData, task_models.BaseDynamicTaskData]
+        ):
+            TASK_VERSION = 1
+            TASK_TYPE = TaskTypes.SIGNING
+
+            def run(self, execute_directory: Path) -> bool:  # noqa: U100
+                raise NotImplementedError()
+
+        self.addCleanup(
+            BaseTask._sub_tasks[TaskTypes.SIGNING].__delitem__, "signingtest"
+        )
+
+        next_work_request = WorkRequestResponse(
+            id=52,
+            task_type="Signing",
+            task_name="signingtest",
+            created_at="2022-01-05T11:14:22.242178Z",
+            started_at=None,
+            completed_at=None,
+            duration=None,
+            worker=58,
+            task_data={},
+            priority_base=0,
+            priority_adjustment=0,
+            status="running",
+            result="",
+            artifacts=[],
+            workspace="TestWorkspace",
+        )
+        self.server_config.get_next_for_worker = json_response(
+            model_to_json_serializable_dict(next_work_request)
+        )
+
+        await self.assert_execute_work_request_send_and_log_error(
+            "Task: signingtest is of type Signing, not Worker"
+        )
+
+    async def test_worker_task_on_signing_worker(self):
+        """A signing worker refuses to run a worker task."""
+        next_work_request = WorkRequestResponse(
+            id=52,
+            task_type="Worker",
+            task_name="noop",
+            created_at="2022-01-05T11:14:22.242178Z",
+            started_at=None,
+            completed_at=None,
+            duration=None,
+            worker=58,
+            task_data={},
+            priority_base=0,
+            priority_adjustment=0,
+            status="running",
+            result="",
+            artifacts=[],
+            workspace="TestWorkspace",
+        )
+        self.server_config.get_next_for_worker = json_response(
+            model_to_json_serializable_dict(next_work_request)
+        )
+
+        await self.assert_execute_work_request_send_and_log_error(
+            "Task: noop is of type Worker, not Signing",
+            worker_type=WorkerType.SIGNING,
+        )
+
+    async def test_signing_task_on_signing_worker(self):
+        """A signing worker can run a signing task."""
+
+        class SigningTestData(task_models.BaseTaskData):
+            pass
+
+        class SigningTest(
+            BaseExternalTask[SigningTestData, task_models.BaseDynamicTaskData]
+        ):
+            TASK_VERSION = 1
+            TASK_TYPE = TaskTypes.SIGNING
+
+            def run(self, execute_directory: Path) -> bool:  # noqa: U100
+                raise NotImplementedError()
+
+        self.addCleanup(
+            BaseTask._sub_tasks[TaskTypes.SIGNING].__delitem__, "signingtest"
+        )
+
+        next_work_request = WorkRequestResponse(
+            id=52,
+            task_type="Signing",
+            task_name="signingtest",
+            created_at="2022-01-05T11:14:22.242178Z",
+            started_at=None,
+            completed_at=None,
+            duration=None,
+            worker=58,
+            task_data={},
+            priority_base=0,
+            priority_adjustment=0,
+            status="running",
+            result="",
+            artifacts=[],
+            workspace="TestWorkspace",
+        )
+        self.server_config.get_next_for_worker = json_response(
+            model_to_json_serializable_dict(next_work_request)
+        )
+
+        await self.receive_task_submit_result(
+            TaskTypes.SIGNING,
+            "signingtest",
+            True,
+            "success",
+            worker_type=WorkerType.SIGNING,
+        )
+
+    async def test_unknown_task_name(self):
+        """A worker refuses to run a task with an unknown name."""
+        next_work_request = WorkRequestResponse(
+            id=52,
+            task_type="Worker",
+            task_name="nonexistent",
+            created_at="2022-01-05T11:14:22.242178Z",
+            started_at=None,
+            completed_at=None,
+            duration=None,
+            worker=58,
+            task_data={},
+            priority_base=0,
+            priority_adjustment=0,
+            status="running",
+            result="",
+            artifacts=[],
+            workspace="TestWorkspace",
+        )
+        self.server_config.get_next_for_worker = json_response(
+            model_to_json_serializable_dict(next_work_request)
+        )
+
+        await self.assert_execute_work_request_send_and_log_error(
+            "Task: nonexistent Error setup:"
+            " 'nonexistent' is not a registered Worker task_name"
         )
 
     def patch_debusine_work_request_completed_update(self) -> MagicMock:
@@ -764,7 +1036,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         self.addCleanup(patcher.stop)
         return mocked
 
-    @unittest_run_loop
     async def test_task_execution_error_submit_error(self):
         """Sbuild.execute raises TaskConfigError, Worker submits error."""
         patcher_execute = mock.patch.object(Sbuild, 'execute', autospec=True)
@@ -776,7 +1047,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
             'Task: sbuild Error execute: No suitable schroot'
         )
 
-    @unittest_run_loop
     async def test_process_message_invalid_msg(self):
         """Worker._process_message returns False."""
         msg = WSMessage(WSMsgType.TEXT, '{"text": "something"}', None)
@@ -787,7 +1057,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertFalse(message_processed)
 
-    @unittest_run_loop
     async def test_process_message_txt_without_msg_key(self):
         """Worker._process_message returns False (no msg text)."""
         msg = WSMessage(WSMsgType.TEXT, '{"foo": "bar"}', None)
@@ -800,7 +1069,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertFalse(message_processed)
 
-    @unittest_run_loop
     async def test_process_message_raise_token_disabled_error_disabled(self):
         """Worker._process_message logs reason of worker's not connection."""
         msg = WSMessage(
@@ -815,7 +1083,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertEqual(str(exc.exception), "Disabled token")
 
-    @unittest_run_loop
     async def test_process_message_raise_token_disabled_error_unknown(self):
         """Worker._process_message logs reason of worker's, exc is 'Unknown'."""
         msg = WSMessage(
@@ -830,7 +1097,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertEqual(str(exc.exception), "Unknown")
 
-    @unittest_run_loop
     async def test_process_message_unknown_reason_code_is_logged_ignored(self):
         """Worker logs and returns False for msg with an unknown reason_code."""
         msg = WSMessage(
@@ -847,7 +1113,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertFalse(processed)
 
-    @unittest_run_loop
     async def test_process_message_non_text_type(self):
         """Worker._process_message returns False for non-text type."""
         msg = WSMessage(WSMsgType.PING, None, None)
@@ -857,7 +1122,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertFalse(message_processed)
 
-    @unittest_run_loop
     async def test_process_message_invalid_msg_type(self):
         """Worker._process_message return False for an invalid message type."""
         msg = 'Something not an instance of WMessage'
@@ -873,7 +1137,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertFalse(message_processed)
 
-    @unittest_run_loop
     async def test_process_message_invalid_json(self):
         """Worker._process_message return False if the JSON is invalid."""
         msg = WSMessage(WSMsgType.TEXT, '{[', None)
@@ -897,7 +1160,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         return fail_mocked
 
-    @unittest_run_loop
     async def test_invalid_configuration_missing_debusine_url(self):
         """Worker() raises an exception: api_url is not in [General]."""
         config_temp_directory = self.create_temp_config_directory(
@@ -918,7 +1180,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         fail_mocked.assert_called_with(log_message)
 
-    @unittest_run_loop
     async def test_invalid_configuration_missing_general_section(self):
         """Worker() raises an exception: [General] section is not available."""
         config_temp_directory = self.create_temp_config_directory({})
@@ -936,6 +1197,38 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         fail_mocked.assert_called_with(log_message)
 
+    async def test_invalid_configuration_api_url_not_https(self):
+        """A worker configured to require an HTTPS api-url fails with HTTP."""
+        config_temp_directory = self.create_temp_config_directory(
+            {"General": {"api-url": "http://example.org/"}}
+        )
+        config = ConfigHandler(
+            directories=[config_temp_directory], require_https=True
+        )
+
+        log_message = (
+            f'api-url in {config.active_configuration_file} does not use '
+            f'HTTPS: http://example.org/'
+        )
+
+        fail_mocked = self.patch_config_handler_fail(config)
+
+        with self.assertRaisesSystemExit(3):
+            self.setup_worker(config)
+
+        fail_mocked.assert_called_with(log_message)
+
+    async def test_valid_configuration_api_url_https(self):
+        """A worker configured to require an HTTPS api-url accepts HTTPS."""
+        config_temp_directory = self.create_temp_config_directory(
+            {"General": {"api-url": "https://example.org/"}}
+        )
+        config = ConfigHandler(
+            directories=[config_temp_directory], require_https=True
+        )
+
+        self.setup_worker(config)
+
     def assert_functools_partial_equal(self, partial1, partial2):
         """Assert two functools.partials are semantically equal."""
         # functools.partial does not implement __eq__
@@ -943,7 +1236,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         self.assertEqual(partial1.args, partial2.args)
         self.assertEqual(partial1.keywords, partial2.keywords)
 
-    @unittest_run_loop
     async def test_main_calls_connect(self):
         """Ensure worker.main() calls worker.connect()."""
         self.setup_worker()
@@ -979,7 +1271,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         mock_connect.assert_awaited()
 
-    @unittest_run_loop
     async def test_create_task_signal_int_term_handler(self):
         """
         Test Task returned from Worker._create_task_signal_int_term_handler.
@@ -1109,7 +1400,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
         ):
             self.worker.log_forced_exit(signal.SIGINT)
 
-    @unittest_run_loop
     async def test_asyncio_sleep(self):
         """Worker._asyncio_sleep calls asyncio.sleep."""
         patched = mock.patch('asyncio.sleep', autospec=True)
@@ -1120,7 +1410,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         mocked.assert_called_with(1.5)
 
-    @unittest_run_loop
     async def test_send_task_result_do_not_send_aborted_task(self):
         """Assert that _send_task_result does not send the result (aborted)."""
         self.setup_worker()
@@ -1140,7 +1429,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         work_request_completed_update.assert_not_called()
 
-    @unittest_run_loop
     async def test_signal_int_term_handler(self):
         """Test signal handler logs "Terminated..." and closes the session."""
         self.setup_worker()
@@ -1154,7 +1442,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertTrue(self.worker._aiohttp_client_session.closed)
 
-    @unittest_run_loop
     async def test_signal_int_term_handler_cancels_task(self):
         """Test signal handler aborts() the task."""
         self.setup_worker()
@@ -1167,7 +1454,6 @@ class WorkerTests(TestHelpersMixin, server.DebusineAioHTTPTestCase):
 
         self.assertTrue(self.worker._task_running.aborted)
 
-    @unittest_run_loop
     async def test_setup_signal_int_term_handler(self):
         """Test that SIGINT/SIGTERM setup is done in Worker.main()."""
         self.setup_worker()
